/*     */ package net.rubyeye.xmemcached;
/*     */ 
/*     */ import java.io.UnsupportedEncodingException;
/*     */ import java.security.MessageDigest;
/*     */ import java.security.NoSuchAlgorithmException;
/*     */ import java.util.zip.CRC32;
/*     */ import net.rubyeye.xmemcached.utils.ByteUtils;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public enum HashAlgorithm
/*     */ {
/*     */   private static final long FNV_64_INIT = -3750763034362895579L;
/*     */   private static final long FNV_64_PRIME = 1099511628211L;
/*     */   private static final long FNV_32_INIT = 2166136261L;
/*     */   private static final long FNV_32_PRIME = 16777619L;
/*     */   
/*     */   static
/*     */   {
/*  22 */     NATIVE_HASH = new HashAlgorithm("NATIVE_HASH", 0);
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*  28 */     CRC32_HASH = new HashAlgorithm("CRC32_HASH", 1);
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*  37 */     FNV1_64_HASH = new HashAlgorithm("FNV1_64_HASH", 2);
/*     */     
/*     */ 
/*     */ 
/*  41 */     FNV1A_64_HASH = new HashAlgorithm("FNV1A_64_HASH", 3);
/*     */     
/*     */ 
/*     */ 
/*  45 */     FNV1_32_HASH = new HashAlgorithm("FNV1_32_HASH", 4);
/*     */     
/*     */ 
/*     */ 
/*  49 */     FNV1A_32_HASH = new HashAlgorithm("FNV1A_32_HASH", 5);
/*     */     
/*     */ 
/*     */ 
/*  53 */     KETAMA_HASH = new HashAlgorithm("KETAMA_HASH", 6);
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*  58 */     MYSQL_HASH = new HashAlgorithm("MYSQL_HASH", 7);
/*     */     
/*  60 */     ELF_HASH = new HashAlgorithm("ELF_HASH", 8);
/*     */     
/*  62 */     RS_HASH = new HashAlgorithm("RS_HASH", 9);
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*  67 */     LUA_HASH = new HashAlgorithm("LUA_HASH", 10);
/*     */     
/*  69 */     ELECTION_HASH = new HashAlgorithm("ELECTION_HASH", 11);
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*  74 */     ONE_AT_A_TIME = new HashAlgorithm("ONE_AT_A_TIME", 12);$VALUES = new HashAlgorithm[] { NATIVE_HASH, CRC32_HASH, FNV1_64_HASH, FNV1A_64_HASH, FNV1_32_HASH, FNV1A_32_HASH, KETAMA_HASH, MYSQL_HASH, ELF_HASH, RS_HASH, LUA_HASH, ELECTION_HASH, ONE_AT_A_TIME };
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public long hash(String k)
/*     */   {
/*  88 */     long rv = 0L;
/*  89 */     switch (this) {
/*     */     case NATIVE_HASH: 
/*  91 */       rv = k.hashCode();
/*  92 */       break;
/*     */     
/*     */     case CRC32_HASH: 
/*  95 */       CRC32 crc32 = new CRC32();
/*  96 */       crc32.update(ByteUtils.getBytes(k));
/*  97 */       rv = crc32.getValue() >> 16 & 0x7FFF;
/*  98 */       break;
/*     */     
/*     */     case FNV1_64_HASH: 
/* 101 */       rv = -3750763034362895579L;
/* 102 */       int len = k.length();
/* 103 */       for (int i = 0; i < len; i++) {
/* 104 */         rv *= 1099511628211L;
/* 105 */         rv ^= k.charAt(i);
/*     */       }
/*     */       
/* 108 */       break;
/*     */     case FNV1A_64_HASH: 
/* 110 */       rv = -3750763034362895579L;
/* 111 */       int len = k.length();
/* 112 */       for (int i = 0; i < len; i++) {
/* 113 */         rv ^= k.charAt(i);
/* 114 */         rv *= 1099511628211L;
/*     */       }
/*     */       
/* 117 */       break;
/*     */     case FNV1_32_HASH: 
/* 119 */       rv = 2166136261L;
/* 120 */       int len = k.length();
/* 121 */       for (int i = 0; i < len; i++) {
/* 122 */         rv *= 16777619L;
/* 123 */         rv ^= k.charAt(i);
/*     */       }
/*     */       
/* 126 */       break;
/*     */     case FNV1A_32_HASH: 
/* 128 */       rv = 2166136261L;
/* 129 */       int len = k.length();
/* 130 */       for (int i = 0; i < len; i++) {
/* 131 */         rv ^= k.charAt(i);
/* 132 */         rv *= 16777619L;
/*     */       }
/*     */       
/* 135 */       break;
/*     */     case ELECTION_HASH: 
/*     */     case KETAMA_HASH: 
/* 138 */       byte[] bKey = computeMd5(k);
/* 139 */       rv = (bKey[3] & 0xFF) << 24 | (bKey[2] & 0xFF) << 16 | (bKey[1] & 0xFF) << 8 | bKey[0] & 0xFF;
/*     */       
/* 141 */       break;
/*     */     
/*     */     case MYSQL_HASH: 
/* 144 */       int nr2 = 4;
/* 145 */       for (int i = 0; i < k.length(); i++) {
/* 146 */         rv ^= ((rv & 0x3F) + nr2) * k.charAt(i) + (rv << 8);
/* 147 */         nr2 += 3;
/*     */       }
/* 149 */       break;
/*     */     case ELF_HASH: 
/* 151 */       long x = 0L;
/* 152 */       for (int i = 0; i < k.length(); i++) {
/* 153 */         rv = (rv << 4) + k.charAt(i);
/* 154 */         if ((x = rv & 0xF0000000) != 0L) {
/* 155 */           rv ^= x >> 24;
/* 156 */           rv &= (x ^ 0xFFFFFFFFFFFFFFFF);
/*     */         }
/*     */       }
/* 159 */       rv &= 0x7FFFFFFF;
/* 160 */       break;
/*     */     case RS_HASH: 
/* 162 */       long b = 378551L;
/* 163 */       long a = 63689L;
/* 164 */       for (int i = 0; i < k.length(); i++) {
/* 165 */         rv = rv * a + k.charAt(i);
/* 166 */         a *= b;
/*     */       }
/* 168 */       rv &= 0x7FFFFFFF;
/* 169 */       break;
/*     */     case LUA_HASH: 
/* 171 */       int step = (k.length() >> 5) + 1;
/* 172 */       rv = k.length();
/* 173 */       for (int len = k.length(); len >= step; len -= step) {
/* 174 */         rv ^= (rv << 5) + (rv >> 2) + k.charAt(len - 1);
/*     */       }
/*     */     case ONE_AT_A_TIME: 
/*     */       try {
/* 178 */         int hash = 0;
/* 179 */         for (byte bt : k.getBytes("utf-8")) {
/* 180 */           hash += (bt & 0xFF);
/* 181 */           hash += (hash << 10);
/* 182 */           hash ^= hash >>> 6;
/*     */         }
/* 184 */         hash += (hash << 3);
/* 185 */         hash ^= hash >>> 11;
/* 186 */         hash += (hash << 15);
/* 187 */         return hash;
/*     */       } catch (UnsupportedEncodingException e) {
/* 189 */         throw new IllegalStateException("Hash function error", e);
/*     */       }
/*     */     }
/* 192 */     if (!$assertionsDisabled) { throw new AssertionError();
/*     */     }
/*     */     
/* 195 */     return rv & 0xFFFFFFFF;
/*     */   }
/*     */   
/* 198 */   private static ThreadLocal<MessageDigest> md5Local = new ThreadLocal();
/*     */   
/*     */ 
/*     */ 
/*     */   public static byte[] computeMd5(String k)
/*     */   {
/* 204 */     MessageDigest md5 = (MessageDigest)md5Local.get();
/* 205 */     if (md5 == null) {
/*     */       try {
/* 207 */         md5 = MessageDigest.getInstance("MD5");
/* 208 */         md5Local.set(md5);
/*     */       } catch (NoSuchAlgorithmException e) {
/* 210 */         throw new RuntimeException("MD5 not supported", e);
/*     */       }
/*     */     }
/* 213 */     md5.reset();
/* 214 */     md5.update(ByteUtils.getBytes(k));
/* 215 */     return md5.digest();
/*     */   }
/*     */   
/*     */   private HashAlgorithm() {}
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\net\rubyeye\xmemcached\HashAlgorithm.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */