/*     */ package net.rubyeye.xmemcached.monitor;
/*     */ 
/*     */ import java.io.IOException;
/*     */ import java.lang.management.ManagementFactory;
/*     */ import java.net.InetAddress;
/*     */ import java.rmi.registry.LocateRegistry;
/*     */ import java.rmi.registry.Registry;
/*     */ import javax.management.MBeanServer;
/*     */ import javax.management.ObjectName;
/*     */ import javax.management.remote.JMXConnectorServer;
/*     */ import javax.management.remote.JMXConnectorServerFactory;
/*     */ import javax.management.remote.JMXServiceURL;
/*     */ import org.slf4j.Logger;
/*     */ import org.slf4j.LoggerFactory;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public final class XMemcachedMbeanServer
/*     */ {
/*  61 */   private static final Logger log = LoggerFactory.getLogger(XMemcachedMbeanServer.class);
/*     */   
/*     */ 
/*  64 */   private MBeanServer mbserver = null;
/*     */   
/*  66 */   private static XMemcachedMbeanServer instance = new XMemcachedMbeanServer();
/*     */   
/*     */   private JMXConnectorServer connectorServer;
/*     */   private Thread shutdownHookThread;
/*  70 */   private volatile boolean isHutdownHookCalled = false;
/*     */   
/*     */   private XMemcachedMbeanServer() {
/*  73 */     initialize();
/*     */   }
/*     */   
/*     */   private void initialize() {
/*  77 */     if ((this.mbserver != null) && (this.connectorServer != null) && (this.connectorServer.isActive()))
/*     */     {
/*  79 */       return;
/*     */     }
/*     */     
/*  82 */     String hostName = null;
/*     */     try {
/*  84 */       InetAddress addr = InetAddress.getLocalHost();
/*     */       
/*  86 */       hostName = addr.getHostName();
/*     */     } catch (IOException e) {
/*  88 */       log.error("Get HostName Error", e);
/*  89 */       hostName = "localhost";
/*     */     }
/*  91 */     String host = System.getProperty("hostName", hostName);
/*     */     try {
/*  93 */       boolean enableJMX = Boolean.parseBoolean(System.getProperty("xmemcached.jmx.enable", "false"));
/*     */       
/*  95 */       if (enableJMX) {
/*  96 */         this.mbserver = ManagementFactory.getPlatformMBeanServer();
/*  97 */         int port = Integer.parseInt(System.getProperty("xmemcached.rmi.port", "7077"));
/*     */         
/*  99 */         String rmiName = System.getProperty("xmemcached.rmi.name", "xmemcachedServer");
/*     */         
/* 101 */         Registry registry = null;
/*     */         try {
/* 103 */           registry = LocateRegistry.getRegistry(port);
/* 104 */           registry.list();
/*     */         } catch (Exception e) {
/* 106 */           registry = null;
/*     */         }
/* 108 */         if (null == registry) {
/* 109 */           registry = LocateRegistry.createRegistry(port);
/*     */         }
/* 111 */         registry.list();
/* 112 */         String serverURL = "service:jmx:rmi:///jndi/rmi://" + host + ":" + port + "/" + rmiName;
/*     */         
/* 114 */         JMXServiceURL url = new JMXServiceURL(serverURL);
/* 115 */         this.connectorServer = JMXConnectorServerFactory.newJMXConnectorServer(url, null, this.mbserver);
/*     */         
/* 117 */         this.connectorServer.start();
/* 118 */         this.shutdownHookThread = new Thread()
/*     */         {
/*     */           public void run() {
/*     */             try {
/* 122 */               XMemcachedMbeanServer.this.isHutdownHookCalled = true;
/* 123 */               if (XMemcachedMbeanServer.this.connectorServer.isActive())
/*     */               {
/* 125 */                 XMemcachedMbeanServer.this.connectorServer.stop();
/*     */                 
/* 127 */                 XMemcachedMbeanServer.log.warn("JMXConnector stop");
/*     */               }
/*     */             } catch (IOException e) {
/* 130 */               XMemcachedMbeanServer.log.error("Shutdown Xmemcached MBean server error", e);
/*     */             }
/*     */             
/*     */           }
/*     */           
/* 135 */         };
/* 136 */         Runtime.getRuntime().addShutdownHook(this.shutdownHookThread);
/* 137 */         log.warn("jmx url: " + serverURL);
/*     */       }
/*     */     } catch (Exception e) {
/* 140 */       log.error("create MBServer error", e);
/*     */     }
/*     */   }
/*     */   
/*     */   public static XMemcachedMbeanServer getInstance() {
/* 145 */     return instance;
/*     */   }
/*     */   
/*     */   public final void shutdown() {
/*     */     try {
/* 150 */       if ((this.connectorServer != null) && (this.connectorServer.isActive())) {
/* 151 */         this.connectorServer.stop();
/* 152 */         log.warn("JMXConnector stop");
/* 153 */         if (!this.isHutdownHookCalled) {
/* 154 */           Runtime.getRuntime().removeShutdownHook(this.shutdownHookThread);
/*     */         }
/*     */       }
/*     */     } catch (IOException e) {
/* 158 */       log.error("Shutdown Xmemcached MBean server error", e);
/*     */     }
/*     */   }
/*     */   
/*     */   public boolean isRegistered(String name) {
/*     */     try {
/* 164 */       return (this.mbserver != null) && (this.mbserver.isRegistered(new ObjectName(name)));
/*     */     }
/*     */     catch (Exception e) {
/* 167 */       throw new RuntimeException(e);
/*     */     }
/*     */   }
/*     */   
/*     */   public boolean isActive() {
/* 172 */     return (this.mbserver != null) && (this.connectorServer != null) && (this.connectorServer.isActive());
/*     */   }
/*     */   
/*     */   public int getMBeanCount()
/*     */   {
/* 177 */     if (this.mbserver != null) {
/* 178 */       return this.mbserver.getMBeanCount().intValue();
/*     */     }
/* 180 */     return 0;
/*     */   }
/*     */   
/*     */   public void registMBean(Object o, String name)
/*     */   {
/* 185 */     if (isRegistered(name)) {
/* 186 */       return;
/*     */     }
/*     */     
/* 189 */     if (this.mbserver != null) {
/*     */       try {
/* 191 */         this.mbserver.registerMBean(o, new ObjectName(name));
/*     */       } catch (Exception e) {
/* 193 */         throw new RuntimeException(e);
/*     */       }
/*     */     }
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\net\rubyeye\xmemcached\monitor\XMemcachedMbeanServer.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */