/*     */ package org.apache.avro.io;
/*     */ 
/*     */ import java.io.EOFException;
/*     */ import java.io.IOException;
/*     */ import java.io.InputStream;
/*     */ import java.nio.ByteBuffer;
/*     */ import org.apache.avro.util.Utf8;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class BinaryDecoder
/*     */   extends Decoder
/*     */ {
/*  40 */   private ByteSource source = null;
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*  48 */   private byte[] buf = null;
/*  49 */   private int minPos = 0;
/*  50 */   private int pos = 0;
/*  51 */   private int limit = 0;
/*     */   
/*     */   BufferAccessor getBufferAccessor() {
/*  54 */     return new BufferAccessor(this, null);
/*     */   }
/*     */   
/*     */ 
/*     */   protected BinaryDecoder() {}
/*     */   
/*     */ 
/*     */   BinaryDecoder(InputStream in, int bufferSize)
/*     */   {
/*  63 */     configure(in, bufferSize);
/*     */   }
/*     */   
/*     */   BinaryDecoder(byte[] data, int offset, int length)
/*     */   {
/*  68 */     configure(data, offset, length);
/*     */   }
/*     */   
/*     */   BinaryDecoder configure(InputStream in, int bufferSize) {
/*  72 */     configureSource(bufferSize, new InputStreamByteSource(in, null));
/*  73 */     return this;
/*     */   }
/*     */   
/*     */   BinaryDecoder configure(byte[] data, int offset, int length) {
/*  77 */     configureSource(8192, new ByteArrayByteSource(data, offset, length, null));
/*     */     
/*  79 */     return this;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   private void configureSource(int bufferSize, ByteSource source)
/*     */   {
/*  93 */     if (null != this.source) {
/*  94 */       this.source.detach();
/*     */     }
/*  96 */     source.attach(bufferSize, this);
/*  97 */     this.source = source;
/*     */   }
/*     */   
/*     */   public void readNull()
/*     */     throws IOException
/*     */   {}
/*     */   
/*     */   public boolean readBoolean()
/*     */     throws IOException
/*     */   {
/* 107 */     if (this.limit == this.pos) {
/* 108 */       this.limit = this.source.tryReadRaw(this.buf, 0, this.buf.length);
/* 109 */       this.pos = 0;
/* 110 */       if (this.limit == 0) {
/* 111 */         throw new EOFException();
/*     */       }
/*     */     }
/* 114 */     int n = this.buf[(this.pos++)] & 0xFF;
/* 115 */     return n == 1;
/*     */   }
/*     */   
/*     */   public int readInt() throws IOException
/*     */   {
/* 120 */     ensureBounds(5);
/* 121 */     int len = 1;
/* 122 */     int b = this.buf[this.pos] & 0xFF;
/* 123 */     int n = b & 0x7F;
/* 124 */     if (b > 127) {
/* 125 */       b = this.buf[(this.pos + len++)] & 0xFF;
/* 126 */       n ^= (b & 0x7F) << 7;
/* 127 */       if (b > 127) {
/* 128 */         b = this.buf[(this.pos + len++)] & 0xFF;
/* 129 */         n ^= (b & 0x7F) << 14;
/* 130 */         if (b > 127) {
/* 131 */           b = this.buf[(this.pos + len++)] & 0xFF;
/* 132 */           n ^= (b & 0x7F) << 21;
/* 133 */           if (b > 127) {
/* 134 */             b = this.buf[(this.pos + len++)] & 0xFF;
/* 135 */             n ^= (b & 0x7F) << 28;
/* 136 */             if (b > 127) {
/* 137 */               throw new IOException("Invalid int encoding");
/*     */             }
/*     */           }
/*     */         }
/*     */       }
/*     */     }
/* 143 */     this.pos += len;
/* 144 */     if (this.pos > this.limit) {
/* 145 */       throw new EOFException();
/*     */     }
/* 147 */     return n >>> 1 ^ -(n & 0x1);
/*     */   }
/*     */   
/*     */   public long readLong() throws IOException
/*     */   {
/* 152 */     ensureBounds(10);
/* 153 */     int b = this.buf[(this.pos++)] & 0xFF;
/* 154 */     int n = b & 0x7F;
/*     */     long l;
/* 156 */     long l; if (b > 127) {
/* 157 */       b = this.buf[(this.pos++)] & 0xFF;
/* 158 */       n ^= (b & 0x7F) << 7;
/* 159 */       long l; if (b > 127) {
/* 160 */         b = this.buf[(this.pos++)] & 0xFF;
/* 161 */         n ^= (b & 0x7F) << 14;
/* 162 */         long l; if (b > 127) {
/* 163 */           b = this.buf[(this.pos++)] & 0xFF;
/* 164 */           n ^= (b & 0x7F) << 21;
/* 165 */           long l; if (b > 127)
/*     */           {
/*     */ 
/* 168 */             l = innerLongDecode(n);
/*     */           } else {
/* 170 */             l = n;
/*     */           }
/*     */         } else {
/* 173 */           l = n;
/*     */         }
/*     */       } else {
/* 176 */         l = n;
/*     */       }
/*     */     } else {
/* 179 */       l = n;
/*     */     }
/* 181 */     if (this.pos > this.limit) {
/* 182 */       throw new EOFException();
/*     */     }
/* 184 */     return l >>> 1 ^ -(l & 1L);
/*     */   }
/*     */   
/*     */   private long innerLongDecode(long l)
/*     */     throws IOException
/*     */   {
/* 190 */     int len = 1;
/* 191 */     int b = this.buf[this.pos] & 0xFF;
/* 192 */     l ^= (b & 0x7F) << 28;
/* 193 */     if (b > 127) {
/* 194 */       b = this.buf[(this.pos + len++)] & 0xFF;
/* 195 */       l ^= (b & 0x7F) << 35;
/* 196 */       if (b > 127) {
/* 197 */         b = this.buf[(this.pos + len++)] & 0xFF;
/* 198 */         l ^= (b & 0x7F) << 42;
/* 199 */         if (b > 127) {
/* 200 */           b = this.buf[(this.pos + len++)] & 0xFF;
/* 201 */           l ^= (b & 0x7F) << 49;
/* 202 */           if (b > 127) {
/* 203 */             b = this.buf[(this.pos + len++)] & 0xFF;
/* 204 */             l ^= (b & 0x7F) << 56;
/* 205 */             if (b > 127) {
/* 206 */               b = this.buf[(this.pos + len++)] & 0xFF;
/* 207 */               l ^= (b & 0x7F) << 63;
/* 208 */               if (b > 127) {
/* 209 */                 throw new IOException("Invalid long encoding");
/*     */               }
/*     */             }
/*     */           }
/*     */         }
/*     */       }
/*     */     }
/* 216 */     this.pos += len;
/* 217 */     return l;
/*     */   }
/*     */   
/*     */   public float readFloat() throws IOException
/*     */   {
/* 222 */     ensureBounds(4);
/* 223 */     int len = 1;
/* 224 */     int n = this.buf[this.pos] & 0xFF | (this.buf[(this.pos + len++)] & 0xFF) << 8 | (this.buf[(this.pos + len++)] & 0xFF) << 16 | (this.buf[(this.pos + len++)] & 0xFF) << 24;
/*     */     
/* 226 */     if (this.pos + 4 > this.limit) {
/* 227 */       throw new EOFException();
/*     */     }
/* 229 */     this.pos += 4;
/* 230 */     return Float.intBitsToFloat(n);
/*     */   }
/*     */   
/*     */   public double readDouble() throws IOException
/*     */   {
/* 235 */     ensureBounds(8);
/* 236 */     int len = 1;
/* 237 */     int n1 = this.buf[this.pos] & 0xFF | (this.buf[(this.pos + len++)] & 0xFF) << 8 | (this.buf[(this.pos + len++)] & 0xFF) << 16 | (this.buf[(this.pos + len++)] & 0xFF) << 24;
/*     */     
/* 239 */     int n2 = this.buf[(this.pos + len++)] & 0xFF | (this.buf[(this.pos + len++)] & 0xFF) << 8 | (this.buf[(this.pos + len++)] & 0xFF) << 16 | (this.buf[(this.pos + len++)] & 0xFF) << 24;
/*     */     
/* 241 */     if (this.pos + 8 > this.limit) {
/* 242 */       throw new EOFException();
/*     */     }
/* 244 */     this.pos += 8;
/* 245 */     return Double.longBitsToDouble(n1 & 0xFFFFFFFF | n2 << 32);
/*     */   }
/*     */   
/*     */   public Utf8 readString(Utf8 old)
/*     */     throws IOException
/*     */   {
/* 251 */     int length = readInt();
/* 252 */     Utf8 result = old != null ? old : new Utf8();
/* 253 */     result.setByteLength(length);
/* 254 */     if (0 != length) {
/* 255 */       doReadBytes(result.getBytes(), 0, length);
/*     */     }
/* 257 */     return result;
/*     */   }
/*     */   
/* 260 */   private final Utf8 scratchUtf8 = new Utf8();
/*     */   
/*     */   public String readString() throws IOException
/*     */   {
/* 264 */     return readString(this.scratchUtf8).toString();
/*     */   }
/*     */   
/*     */   public void skipString() throws IOException
/*     */   {
/* 269 */     doSkipBytes(readInt());
/*     */   }
/*     */   
/*     */   public ByteBuffer readBytes(ByteBuffer old) throws IOException
/*     */   {
/* 274 */     int length = readInt();
/*     */     ByteBuffer result;
/* 276 */     if ((old != null) && (length <= old.capacity())) {
/* 277 */       ByteBuffer result = old;
/* 278 */       result.clear();
/*     */     } else {
/* 280 */       result = ByteBuffer.allocate(length);
/*     */     }
/* 282 */     doReadBytes(result.array(), result.position(), length);
/* 283 */     result.limit(length);
/* 284 */     return result;
/*     */   }
/*     */   
/*     */   public void skipBytes() throws IOException
/*     */   {
/* 289 */     doSkipBytes(readInt());
/*     */   }
/*     */   
/*     */   public void readFixed(byte[] bytes, int start, int length) throws IOException
/*     */   {
/* 294 */     doReadBytes(bytes, start, length);
/*     */   }
/*     */   
/*     */   public void skipFixed(int length) throws IOException
/*     */   {
/* 299 */     doSkipBytes(length);
/*     */   }
/*     */   
/*     */   public int readEnum() throws IOException
/*     */   {
/* 304 */     return readInt();
/*     */   }
/*     */   
/*     */   protected void doSkipBytes(long length) throws IOException {
/* 308 */     int remaining = this.limit - this.pos;
/* 309 */     if (length <= remaining) {
/* 310 */       this.pos = ((int)(this.pos + length));
/*     */     } else {
/* 312 */       this.limit = (this.pos = 0);
/* 313 */       length -= remaining;
/* 314 */       this.source.skipSourceBytes(length);
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected void doReadBytes(byte[] bytes, int start, int length)
/*     */     throws IOException
/*     */   {
/* 327 */     int remaining = this.limit - this.pos;
/* 328 */     if (length <= remaining) {
/* 329 */       System.arraycopy(this.buf, this.pos, bytes, start, length);
/* 330 */       this.pos += length;
/*     */     }
/*     */     else {
/* 333 */       System.arraycopy(this.buf, this.pos, bytes, start, remaining);
/* 334 */       start += remaining;
/* 335 */       length -= remaining;
/* 336 */       this.pos = this.limit;
/*     */       
/* 338 */       this.source.readRaw(bytes, start, length);
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected long doReadItemCount()
/*     */     throws IOException
/*     */   {
/* 350 */     long result = readLong();
/* 351 */     if (result < 0L) {
/* 352 */       readLong();
/* 353 */       result = -result;
/*     */     }
/* 355 */     return result;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   private long doSkipItems()
/*     */     throws IOException
/*     */   {
/* 371 */     long result = readInt();
/* 372 */     while (result < 0L) {
/* 373 */       long bytecount = readLong();
/* 374 */       doSkipBytes(bytecount);
/* 375 */       result = readInt();
/*     */     }
/* 377 */     return result;
/*     */   }
/*     */   
/*     */   public long readArrayStart() throws IOException
/*     */   {
/* 382 */     return doReadItemCount();
/*     */   }
/*     */   
/*     */   public long arrayNext() throws IOException
/*     */   {
/* 387 */     return doReadItemCount();
/*     */   }
/*     */   
/*     */   public long skipArray() throws IOException
/*     */   {
/* 392 */     return doSkipItems();
/*     */   }
/*     */   
/*     */   public long readMapStart() throws IOException
/*     */   {
/* 397 */     return doReadItemCount();
/*     */   }
/*     */   
/*     */   public long mapNext() throws IOException
/*     */   {
/* 402 */     return doReadItemCount();
/*     */   }
/*     */   
/*     */   public long skipMap() throws IOException
/*     */   {
/* 407 */     return doSkipItems();
/*     */   }
/*     */   
/*     */   public int readIndex() throws IOException
/*     */   {
/* 412 */     return readInt();
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public boolean isEnd()
/*     */     throws IOException
/*     */   {
/* 425 */     if (this.limit - this.pos > 0)
/*     */     {
/* 427 */       return false;
/*     */     }
/* 429 */     if (this.source.isEof()) {
/* 430 */       return true;
/*     */     }
/*     */     
/* 433 */     int read = this.source.tryReadRaw(this.buf, 0, this.buf.length);
/* 434 */     this.pos = 0;
/* 435 */     this.limit = read;
/* 436 */     if (0 == read)
/*     */     {
/* 438 */       return true;
/*     */     }
/* 440 */     return false;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   private void ensureBounds(int num)
/*     */     throws IOException
/*     */   {
/* 457 */     int remaining = this.limit - this.pos;
/* 458 */     if (remaining < num)
/*     */     {
/* 460 */       this.source.compactAndFill(this.buf, this.pos, this.minPos, remaining);
/* 461 */       if (this.pos >= this.limit) {
/* 462 */         throw new EOFException();
/*     */       }
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public InputStream inputStream()
/*     */   {
/* 474 */     return this.source;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   static class BufferAccessor
/*     */   {
/*     */     private final BinaryDecoder decoder;
/*     */     
/*     */ 
/*     */     private byte[] buf;
/*     */     
/*     */ 
/*     */     private int pos;
/*     */     
/*     */ 
/*     */     private int limit;
/*     */     
/*     */ 
/* 494 */     boolean detached = false;
/*     */     
/*     */     private BufferAccessor(BinaryDecoder decoder) {
/* 497 */       this.decoder = decoder;
/*     */     }
/*     */     
/*     */     void detach() {
/* 501 */       this.buf = this.decoder.buf;
/* 502 */       this.pos = this.decoder.pos;
/* 503 */       this.limit = this.decoder.limit;
/* 504 */       this.detached = true;
/*     */     }
/*     */     
/*     */     int getPos() {
/* 508 */       if (this.detached) {
/* 509 */         return this.pos;
/*     */       }
/* 511 */       return this.decoder.pos;
/*     */     }
/*     */     
/*     */     int getLim() {
/* 515 */       if (this.detached) {
/* 516 */         return this.limit;
/*     */       }
/* 518 */       return this.decoder.limit;
/*     */     }
/*     */     
/*     */     byte[] getBuf() {
/* 522 */       if (this.detached) {
/* 523 */         return this.buf;
/*     */       }
/* 525 */       return this.decoder.buf;
/*     */     }
/*     */     
/*     */     void setPos(int pos) {
/* 529 */       if (this.detached) {
/* 530 */         this.pos = pos;
/*     */       } else
/* 532 */         this.decoder.pos = pos;
/*     */     }
/*     */     
/*     */     void setLimit(int limit) {
/* 536 */       if (this.detached) {
/* 537 */         this.limit = limit;
/*     */       } else
/* 539 */         this.decoder.limit = limit;
/*     */     }
/*     */     
/*     */     void setBuf(byte[] buf, int offset, int length) {
/* 543 */       if (this.detached) {
/* 544 */         this.buf = buf;
/* 545 */         this.limit = (offset + length);
/* 546 */         this.pos = offset;
/*     */       } else {
/* 548 */         this.decoder.buf = buf;
/* 549 */         this.decoder.limit = (offset + length);
/* 550 */         this.decoder.pos = offset;
/* 551 */         this.decoder.minPos = offset;
/*     */       }
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   static abstract class ByteSource
/*     */     extends InputStream
/*     */   {
/*     */     protected BinaryDecoder.BufferAccessor ba;
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     abstract boolean isEof();
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     protected void attach(int bufferSize, BinaryDecoder decoder)
/*     */     {
/* 601 */       decoder.buf = new byte[bufferSize];
/* 602 */       decoder.pos = 0;
/* 603 */       decoder.minPos = 0;
/* 604 */       decoder.limit = 0;
/* 605 */       this.ba = new BinaryDecoder.BufferAccessor(decoder, null);
/*     */     }
/*     */     
/*     */     protected void detach()
/*     */     {
/* 610 */       this.ba.detach();
/*     */     }
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     protected abstract void skipSourceBytes(long paramLong)
/*     */       throws IOException;
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     protected abstract long trySkipBytes(long paramLong)
/*     */       throws IOException;
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     protected abstract void readRaw(byte[] paramArrayOfByte, int paramInt1, int paramInt2)
/*     */       throws IOException;
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     protected abstract int tryReadRaw(byte[] paramArrayOfByte, int paramInt1, int paramInt2)
/*     */       throws IOException;
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     protected void compactAndFill(byte[] buf, int pos, int minPos, int remaining)
/*     */       throws IOException
/*     */     {
/* 679 */       System.arraycopy(buf, pos, buf, minPos, remaining);
/* 680 */       this.ba.setPos(minPos);
/* 681 */       int newLimit = remaining + tryReadRaw(buf, minPos + remaining, buf.length - remaining);
/*     */       
/* 683 */       this.ba.setLimit(newLimit);
/*     */     }
/*     */     
/*     */     public int read(byte[] b, int off, int len) throws IOException
/*     */     {
/* 688 */       int lim = this.ba.getLim();
/* 689 */       int pos = this.ba.getPos();
/* 690 */       byte[] buf = this.ba.getBuf();
/* 691 */       int remaining = lim - pos;
/* 692 */       if (remaining >= len) {
/* 693 */         System.arraycopy(buf, pos, b, off, len);
/* 694 */         pos += len;
/* 695 */         this.ba.setPos(pos);
/* 696 */         return len;
/*     */       }
/*     */       
/* 699 */       System.arraycopy(buf, pos, b, off, remaining);
/* 700 */       pos += remaining;
/* 701 */       this.ba.setPos(pos);
/*     */       
/* 703 */       int inputRead = remaining + tryReadRaw(b, off + remaining, len - remaining);
/*     */       
/* 705 */       if (inputRead == 0) {
/* 706 */         return -1;
/*     */       }
/* 708 */       return inputRead;
/*     */     }
/*     */     
/*     */ 
/*     */     public long skip(long n)
/*     */       throws IOException
/*     */     {
/* 715 */       int lim = this.ba.getLim();
/* 716 */       int pos = this.ba.getPos();
/* 717 */       int remaining = lim - pos;
/* 718 */       if (remaining > n) {
/* 719 */         pos = (int)(pos + n);
/* 720 */         this.ba.setPos(pos);
/* 721 */         return n;
/*     */       }
/* 723 */       pos = lim;
/* 724 */       this.ba.setPos(pos);
/* 725 */       long isSkipCount = trySkipBytes(n - remaining);
/* 726 */       return isSkipCount + remaining;
/*     */     }
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     public int available()
/*     */       throws IOException
/*     */     {
/* 736 */       return this.ba.getLim() - this.ba.getPos();
/*     */     }
/*     */   }
/*     */   
/*     */   private static class InputStreamByteSource extends BinaryDecoder.ByteSource {
/*     */     private InputStream in;
/* 742 */     protected boolean isEof = false;
/*     */     
/*     */     private InputStreamByteSource(InputStream in)
/*     */     {
/* 746 */       this.in = in;
/*     */     }
/*     */     
/*     */     protected void skipSourceBytes(long length) throws IOException
/*     */     {
/* 751 */       boolean readZero = false;
/* 752 */       while (length > 0L) {
/* 753 */         long n = this.in.skip(length);
/* 754 */         if (n > 0L) {
/* 755 */           length -= n;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */         }
/* 762 */         else if (n == 0L) {
/* 763 */           if (readZero) {
/* 764 */             this.isEof = true;
/* 765 */             throw new EOFException();
/*     */           }
/* 767 */           readZero = true;
/*     */         }
/*     */         else
/*     */         {
/* 771 */           this.isEof = true;
/* 772 */           throw new EOFException();
/*     */         }
/*     */       }
/*     */     }
/*     */     
/*     */     protected long trySkipBytes(long length) throws IOException {
/* 778 */       long leftToSkip = length;
/*     */       try {
/* 780 */         boolean readZero = false;
/* 781 */         while (leftToSkip > 0L) {
/* 782 */           long n = this.in.skip(length);
/* 783 */           if (n > 0L) {
/* 784 */             leftToSkip -= n;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */           }
/* 791 */           else if (n == 0L) {
/* 792 */             if (readZero) {
/* 793 */               this.isEof = true;
/*     */             }
/*     */             else {
/* 796 */               readZero = true;
/*     */             }
/*     */           }
/*     */           else {
/* 800 */             this.isEof = true;
/*     */           }
/*     */         }
/*     */       }
/*     */       catch (EOFException eof) {
/* 805 */         this.isEof = true;
/*     */       }
/* 807 */       return length - leftToSkip;
/*     */     }
/*     */     
/*     */     protected void readRaw(byte[] data, int off, int len) throws IOException
/*     */     {
/* 812 */       while (len > 0) {
/* 813 */         int read = this.in.read(data, off, len);
/* 814 */         if (read < 0) {
/* 815 */           this.isEof = true;
/* 816 */           throw new EOFException();
/*     */         }
/* 818 */         len -= read;
/* 819 */         off += read;
/*     */       }
/*     */     }
/*     */     
/*     */     protected int tryReadRaw(byte[] data, int off, int len) throws IOException
/*     */     {
/* 825 */       int leftToCopy = len;
/*     */       try {
/* 827 */         while (leftToCopy > 0) {
/* 828 */           int read = this.in.read(data, off, leftToCopy);
/* 829 */           if (read < 0) {
/* 830 */             this.isEof = true;
/* 831 */             break;
/*     */           }
/* 833 */           leftToCopy -= read;
/* 834 */           off += read;
/*     */         }
/*     */       } catch (EOFException eof) {
/* 837 */         this.isEof = true;
/*     */       }
/* 839 */       return len - leftToCopy;
/*     */     }
/*     */     
/*     */     public int read() throws IOException
/*     */     {
/* 844 */       if (this.ba.getLim() - this.ba.getPos() == 0) {
/* 845 */         return this.in.read();
/*     */       }
/* 847 */       int position = this.ba.getPos();
/* 848 */       int result = this.ba.getBuf()[position] & 0xFF;
/* 849 */       this.ba.setPos(position + 1);
/* 850 */       return result;
/*     */     }
/*     */     
/*     */ 
/*     */     public boolean isEof()
/*     */     {
/* 856 */       return this.isEof;
/*     */     }
/*     */     
/*     */     public void close() throws IOException
/*     */     {
/* 861 */       this.in.close();
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */   private static class ByteArrayByteSource
/*     */     extends BinaryDecoder.ByteSource
/*     */   {
/*     */     private byte[] data;
/*     */     
/*     */     private int position;
/*     */     
/*     */     private int max;
/* 874 */     private boolean compacted = false;
/*     */     
/*     */ 
/*     */ 
/*     */     private ByteArrayByteSource(byte[] data, int start, int len)
/*     */     {
/* 880 */       if ((data.length < 16) || (len < 16)) {
/* 881 */         this.data = new byte[16];
/* 882 */         System.arraycopy(data, start, this.data, 0, len);
/* 883 */         this.position = 0;
/* 884 */         this.max = len;
/*     */       }
/*     */       else {
/* 887 */         this.data = data;
/* 888 */         this.position = start;
/* 889 */         this.max = (start + len);
/*     */       }
/*     */     }
/*     */     
/*     */ 
/*     */     protected void attach(int bufferSize, BinaryDecoder decoder)
/*     */     {
/* 896 */       decoder.buf = this.data;
/* 897 */       decoder.pos = this.position;
/* 898 */       decoder.minPos = this.position;
/* 899 */       decoder.limit = this.max;
/* 900 */       this.ba = new BinaryDecoder.BufferAccessor(decoder, null);
/*     */     }
/*     */     
/*     */     protected void skipSourceBytes(long length)
/*     */       throws IOException
/*     */     {
/* 906 */       long skipped = trySkipBytes(length);
/* 907 */       if (skipped < length) {
/* 908 */         throw new EOFException();
/*     */       }
/*     */     }
/*     */     
/*     */     protected long trySkipBytes(long length)
/*     */       throws IOException
/*     */     {
/* 915 */       this.max = this.ba.getLim();
/* 916 */       this.position = this.ba.getPos();
/* 917 */       long remaining = this.max - this.position;
/* 918 */       if (remaining >= length) {
/* 919 */         this.position = ((int)(this.position + length));
/* 920 */         this.ba.setPos(this.position);
/* 921 */         return length;
/*     */       }
/* 923 */       this.position = ((int)(this.position + remaining));
/* 924 */       this.ba.setPos(this.position);
/* 925 */       return remaining;
/*     */     }
/*     */     
/*     */     protected void readRaw(byte[] data, int off, int len)
/*     */       throws IOException
/*     */     {
/* 931 */       int read = tryReadRaw(data, off, len);
/* 932 */       if (read < len) {
/* 933 */         throw new EOFException();
/*     */       }
/*     */     }
/*     */     
/*     */     protected int tryReadRaw(byte[] data, int off, int len)
/*     */       throws IOException
/*     */     {
/* 940 */       return 0;
/*     */     }
/*     */     
/*     */ 
/*     */ 
/*     */     protected void compactAndFill(byte[] buf, int pos, int minPos, int remaining)
/*     */       throws IOException
/*     */     {
/* 948 */       if (!this.compacted)
/*     */       {
/* 950 */         byte[] tinybuf = new byte[remaining + 16];
/* 951 */         System.arraycopy(buf, pos, tinybuf, 0, remaining);
/* 952 */         this.ba.setBuf(tinybuf, 0, remaining);
/* 953 */         this.compacted = true;
/*     */       }
/*     */     }
/*     */     
/*     */     public int read() throws IOException
/*     */     {
/* 959 */       this.max = this.ba.getLim();
/* 960 */       this.position = this.ba.getPos();
/* 961 */       if (this.position >= this.max) {
/* 962 */         return -1;
/*     */       }
/* 964 */       int result = this.ba.getBuf()[(this.position++)] & 0xFF;
/* 965 */       this.ba.setPos(this.position);
/* 966 */       return result;
/*     */     }
/*     */     
/*     */     public void close()
/*     */       throws IOException
/*     */     {
/* 972 */       this.ba.setPos(this.ba.getLim());
/*     */     }
/*     */     
/*     */     public boolean isEof()
/*     */     {
/* 977 */       int remaining = this.ba.getLim() - this.ba.getPos();
/* 978 */       return remaining == 0;
/*     */     }
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\org\apache\avro\io\BinaryDecoder.class
 * Java compiler version: 6 (50.0)
 * JD-Core Version:       0.7.1
 */