/*     */ package org.apache.avro.io.parsing;
/*     */ 
/*     */ import java.io.IOException;
/*     */ import java.util.Arrays;
/*     */ import org.apache.avro.AvroTypeException;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class Parser
/*     */ {
/*     */   protected final ActionHandler symbolHandler;
/*     */   protected Symbol[] stack;
/*     */   protected int pos;
/*     */   
/*     */   public Parser(Symbol root, ActionHandler symbolHandler)
/*     */     throws IOException
/*     */   {
/*  56 */     this.symbolHandler = symbolHandler;
/*  57 */     this.stack = new Symbol[5];
/*  58 */     this.stack[0] = root;
/*  59 */     this.pos = 1;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   private void expandStack()
/*     */   {
/*  66 */     this.stack = ((Symbol[])Arrays.copyOf(this.stack, this.stack.length + Math.max(this.stack.length, 1024)));
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public final Symbol advance(Symbol input)
/*     */     throws IOException
/*     */   {
/*     */     for (;;)
/*     */     {
/*  81 */       Symbol top = this.stack[(--this.pos)];
/*  82 */       if (top == input) {
/*  83 */         return top;
/*     */       }
/*     */       
/*  86 */       Symbol.Kind k = top.kind;
/*  87 */       if (k == Symbol.Kind.IMPLICIT_ACTION) {
/*  88 */         Symbol result = this.symbolHandler.doAction(input, top);
/*  89 */         if (result != null)
/*  90 */           return result;
/*     */       } else {
/*  92 */         if (k == Symbol.Kind.TERMINAL) {
/*  93 */           throw new AvroTypeException("Attempt to process a " + input + " when a " + top + " was expected.");
/*     */         }
/*     */         
/*  96 */         if ((k == Symbol.Kind.REPEATER) && (input == ((Symbol.Repeater)top).end))
/*     */         {
/*  98 */           return input;
/*     */         }
/* 100 */         pushProduction(top);
/*     */       }
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public final void processImplicitActions()
/*     */     throws IOException
/*     */   {
/* 112 */     while (this.pos > 1) {
/* 113 */       Symbol top = this.stack[(this.pos - 1)];
/* 114 */       if (top.kind == Symbol.Kind.IMPLICIT_ACTION) {
/* 115 */         this.pos -= 1;
/* 116 */         this.symbolHandler.doAction(null, top);
/* 117 */       } else { if (top.kind == Symbol.Kind.TERMINAL) break;
/* 118 */         this.pos -= 1;
/* 119 */         pushProduction(top);
/*     */       }
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   public final void processTrailingImplicitActions()
/*     */     throws IOException
/*     */   {
/* 130 */     while (this.pos >= 1) {
/* 131 */       Symbol top = this.stack[(this.pos - 1)];
/* 132 */       if ((top.kind != Symbol.Kind.IMPLICIT_ACTION) || (!((Symbol.ImplicitAction)top).isTrailing))
/*     */         break;
/* 134 */       this.pos -= 1;
/* 135 */       this.symbolHandler.doAction(null, top);
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public final void pushProduction(Symbol sym)
/*     */   {
/* 149 */     Symbol[] p = sym.production;
/* 150 */     while (this.pos + p.length > this.stack.length) {
/* 151 */       expandStack();
/*     */     }
/* 153 */     System.arraycopy(p, 0, this.stack, this.pos, p.length);
/* 154 */     this.pos += p.length;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   public Symbol popSymbol()
/*     */   {
/* 161 */     return this.stack[(--this.pos)];
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   public Symbol topSymbol()
/*     */   {
/* 168 */     return this.stack[(this.pos - 1)];
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   public void pushSymbol(Symbol sym)
/*     */   {
/* 175 */     if (this.pos == this.stack.length) {
/* 176 */       expandStack();
/*     */     }
/* 178 */     this.stack[(this.pos++)] = sym;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   public int depth()
/*     */   {
/* 185 */     return this.pos;
/*     */   }
/*     */   
/*     */   public void reset() {
/* 189 */     this.pos = 1;
/*     */   }
/*     */   
/*     */   public static abstract interface ActionHandler
/*     */   {
/*     */     public abstract Symbol doAction(Symbol paramSymbol1, Symbol paramSymbol2)
/*     */       throws IOException;
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\org\apache\avro\io\parsing\Parser.class
 * Java compiler version: 6 (50.0)
 * JD-Core Version:       0.7.1
 */