/*     */ package org.apache.avro.specific;
/*     */ 
/*     */ import java.lang.reflect.Constructor;
/*     */ import java.lang.reflect.Field;
/*     */ import java.lang.reflect.ParameterizedType;
/*     */ import java.lang.reflect.Type;
/*     */ import java.nio.ByteBuffer;
/*     */ import java.util.Collection;
/*     */ import java.util.LinkedHashMap;
/*     */ import java.util.List;
/*     */ import java.util.Map;
/*     */ import java.util.WeakHashMap;
/*     */ import java.util.concurrent.ConcurrentHashMap;
/*     */ import org.apache.avro.AvroRuntimeException;
/*     */ import org.apache.avro.AvroTypeException;
/*     */ import org.apache.avro.Protocol;
/*     */ import org.apache.avro.Schema;
/*     */ import org.apache.avro.Schema.Type;
/*     */ import org.apache.avro.generic.GenericData;
/*     */ import org.apache.avro.io.DatumReader;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class SpecificData
/*     */   extends GenericData
/*     */ {
/*  41 */   private static final SpecificData INSTANCE = new SpecificData();
/*     */   
/*     */   private final ClassLoader classLoader;
/*     */   
/*  45 */   private static final Class<?>[] NO_ARG = new Class[0];
/*  46 */   private static final Class<?>[] SCHEMA_ARG = { Schema.class };
/*  47 */   private static final Map<Class, Constructor> CTOR_CACHE = new ConcurrentHashMap();
/*     */   
/*     */   protected SpecificData()
/*     */   {
/*  51 */     this(SpecificData.class.getClassLoader());
/*     */   }
/*     */   
/*     */   public SpecificData(ClassLoader classLoader) {
/*  55 */     this.classLoader = classLoader;
/*     */   }
/*     */   
/*     */   public DatumReader createDatumReader(Schema schema)
/*     */   {
/*  60 */     return new SpecificDatumReader(schema, schema, this);
/*     */   }
/*     */   
/*     */   public static SpecificData get() {
/*  64 */     return INSTANCE;
/*     */   }
/*     */   
/*     */   protected boolean isEnum(Object datum) {
/*  68 */     return ((datum instanceof Enum)) || (super.isEnum(datum));
/*     */   }
/*     */   
/*     */   protected Schema getEnumSchema(Object datum)
/*     */   {
/*  73 */     return (datum instanceof Enum) ? getSchema(datum.getClass()) : super.getEnumSchema(datum);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*  78 */   private Map<String, Class> classCache = new ConcurrentHashMap();
/*     */   
/*  80 */   private static final Class NO_CLASS = new Object() {}.getClass();
/*  81 */   private static final Schema NULL_SCHEMA = Schema.create(Schema.Type.NULL);
/*     */   
/*     */   public Class getClass(Schema schema)
/*     */   {
/*  85 */     switch (schema.getType()) {
/*     */     case FIXED: 
/*     */     case RECORD: 
/*     */     case ENUM: 
/*  89 */       String name = schema.getFullName();
/*  90 */       if (name == null) return null;
/*  91 */       Class c = (Class)this.classCache.get(name);
/*  92 */       if (c == null) {
/*     */         try {
/*  94 */           c = this.classLoader.loadClass(getClassName(schema));
/*     */         } catch (ClassNotFoundException e) {
/*  96 */           c = NO_CLASS;
/*     */         }
/*  98 */         this.classCache.put(name, c);
/*     */       }
/* 100 */       return c == NO_CLASS ? null : c;
/* 101 */     case ARRAY:  return List.class;
/* 102 */     case MAP:  return Map.class;
/*     */     case UNION: 
/* 104 */       List<Schema> types = schema.getTypes();
/* 105 */       if ((types.size() == 2) && (types.contains(NULL_SCHEMA)))
/* 106 */         return getWrapper((Schema)types.get(((Schema)types.get(0)).equals(NULL_SCHEMA) ? 1 : 0));
/* 107 */       return Object.class;
/*     */     case STRING: 
/* 109 */       if ("String".equals(schema.getProp("avro.java.string")))
/* 110 */         return String.class;
/* 111 */       return CharSequence.class;
/* 112 */     case BYTES:  return ByteBuffer.class;
/* 113 */     case INT:  return Integer.TYPE;
/* 114 */     case LONG:  return Long.TYPE;
/* 115 */     case FLOAT:  return Float.TYPE;
/* 116 */     case DOUBLE:  return Double.TYPE;
/* 117 */     case BOOLEAN:  return Boolean.TYPE;
/* 118 */     case NULL:  return Void.TYPE; }
/* 119 */     throw new AvroRuntimeException("Unknown type: " + schema);
/*     */   }
/*     */   
/*     */   private Class getWrapper(Schema schema)
/*     */   {
/* 124 */     switch (schema.getType()) {
/* 125 */     case INT:  return Integer.class;
/* 126 */     case LONG:  return Long.class;
/* 127 */     case FLOAT:  return Float.class;
/* 128 */     case DOUBLE:  return Double.class;
/* 129 */     case BOOLEAN:  return Boolean.class;
/*     */     }
/* 131 */     return getClass(schema);
/*     */   }
/*     */   
/*     */   public static String getClassName(Schema schema)
/*     */   {
/* 136 */     String namespace = schema.getNamespace();
/* 137 */     String name = schema.getName();
/* 138 */     if ((namespace == null) || ("".equals(namespace)))
/* 139 */       return name;
/* 140 */     String dot = namespace.endsWith("$") ? "" : ".";
/* 141 */     return namespace + dot + name;
/*     */   }
/*     */   
/* 144 */   private final WeakHashMap<Type, Schema> schemaCache = new WeakHashMap();
/*     */   
/*     */ 
/*     */   public Schema getSchema(Type type)
/*     */   {
/* 149 */     Schema schema = (Schema)this.schemaCache.get(type);
/* 150 */     if (schema == null) {
/* 151 */       schema = createSchema(type, new LinkedHashMap());
/* 152 */       this.schemaCache.put(type, schema);
/*     */     }
/* 154 */     return schema;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   protected Schema createSchema(Type type, Map<String, Schema> names)
/*     */   {
/* 161 */     if (((type instanceof Class)) && (CharSequence.class.isAssignableFrom((Class)type)))
/*     */     {
/* 163 */       return Schema.create(Schema.Type.STRING); }
/* 164 */     if (type == ByteBuffer.class)
/* 165 */       return Schema.create(Schema.Type.BYTES);
/* 166 */     if ((type == Integer.class) || (type == Integer.TYPE))
/* 167 */       return Schema.create(Schema.Type.INT);
/* 168 */     if ((type == Long.class) || (type == Long.TYPE))
/* 169 */       return Schema.create(Schema.Type.LONG);
/* 170 */     if ((type == Float.class) || (type == Float.TYPE))
/* 171 */       return Schema.create(Schema.Type.FLOAT);
/* 172 */     if ((type == Double.class) || (type == Double.TYPE))
/* 173 */       return Schema.create(Schema.Type.DOUBLE);
/* 174 */     if ((type == Boolean.class) || (type == Boolean.TYPE))
/* 175 */       return Schema.create(Schema.Type.BOOLEAN);
/* 176 */     if ((type == Void.class) || (type == Void.TYPE))
/* 177 */       return Schema.create(Schema.Type.NULL);
/* 178 */     if ((type instanceof ParameterizedType)) {
/* 179 */       ParameterizedType ptype = (ParameterizedType)type;
/* 180 */       Class raw = (Class)ptype.getRawType();
/* 181 */       Type[] params = ptype.getActualTypeArguments();
/* 182 */       if (Collection.class.isAssignableFrom(raw)) {
/* 183 */         if (params.length != 1)
/* 184 */           throw new AvroTypeException("No array type specified.");
/* 185 */         return Schema.createArray(createSchema(params[0], names)); }
/* 186 */       if (Map.class.isAssignableFrom(raw)) {
/* 187 */         Type key = params[0];
/* 188 */         Type value = params[1];
/* 189 */         if ((!(type instanceof Class)) || (!CharSequence.class.isAssignableFrom((Class)type)))
/*     */         {
/* 191 */           throw new AvroTypeException("Map key class not CharSequence: " + key); }
/* 192 */         return Schema.createMap(createSchema(value, names));
/*     */       }
/* 194 */       return createSchema(raw, names);
/*     */     }
/* 196 */     if ((type instanceof Class)) {
/* 197 */       Class c = (Class)type;
/* 198 */       String fullName = c.getName();
/* 199 */       Schema schema = (Schema)names.get(fullName);
/* 200 */       if (schema == null)
/*     */         try {
/* 202 */           schema = (Schema)c.getDeclaredField("SCHEMA$").get(null);
/*     */           
/* 204 */           if (!fullName.equals(getClassName(schema)))
/*     */           {
/* 206 */             schema = Schema.parse(schema.toString().replace(schema.getNamespace(), c.getPackage().getName()));
/*     */           }
/*     */         }
/*     */         catch (NoSuchFieldException e) {
/* 210 */           throw new AvroRuntimeException("Not a Specific class: " + c);
/*     */         } catch (IllegalAccessException e) {
/* 212 */           throw new AvroRuntimeException(e);
/*     */         }
/* 214 */       names.put(fullName, schema);
/* 215 */       return schema;
/*     */     }
/* 217 */     throw new AvroTypeException("Unknown type: " + type);
/*     */   }
/*     */   
/*     */   public Protocol getProtocol(Class iface)
/*     */   {
/*     */     try {
/* 223 */       Protocol p = (Protocol)iface.getDeclaredField("PROTOCOL").get(null);
/* 224 */       if (!p.getNamespace().equals(iface.getPackage().getName())) {}
/*     */       
/* 226 */       return Protocol.parse(p.toString().replace(p.getNamespace(), iface.getPackage().getName()));
/*     */     }
/*     */     catch (NoSuchFieldException e)
/*     */     {
/* 230 */       throw new AvroRuntimeException("Not a Specific protocol: " + iface);
/*     */     } catch (IllegalAccessException e) {
/* 232 */       throw new AvroRuntimeException(e);
/*     */     }
/*     */   }
/*     */   
/*     */   protected int compare(Object o1, Object o2, Schema s, boolean eq)
/*     */   {
/* 238 */     switch (s.getType()) {
/*     */     case ENUM: 
/* 240 */       if ((o1 instanceof Enum))
/* 241 */         return ((Enum)o1).ordinal() - ((Enum)o2).ordinal();
/*     */       break; }
/* 243 */     return super.compare(o1, o2, s, eq);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static Object newInstance(Class c, Schema s)
/*     */   {
/* 252 */     boolean useSchema = SchemaConstructable.class.isAssignableFrom(c);
/*     */     Object result;
/*     */     try {
/* 255 */       Constructor meth = (Constructor)CTOR_CACHE.get(c);
/* 256 */       if (meth == null) {
/* 257 */         meth = c.getDeclaredConstructor(useSchema ? SCHEMA_ARG : NO_ARG);
/* 258 */         meth.setAccessible(true);
/* 259 */         CTOR_CACHE.put(c, meth);
/*     */       }
/* 261 */       result = meth.newInstance(useSchema ? new Object[] { s } : (Object[])null);
/*     */     } catch (Exception e) {
/* 263 */       throw new RuntimeException(e);
/*     */     }
/* 265 */     return result;
/*     */   }
/*     */   
/*     */   public Object createFixed(Object old, Schema schema)
/*     */   {
/* 270 */     Class c = get().getClass(schema);
/* 271 */     if (c == null) return super.createFixed(old, schema);
/* 272 */     return c.isInstance(old) ? old : newInstance(c, schema);
/*     */   }
/*     */   
/*     */   public Object newRecord(Object old, Schema schema)
/*     */   {
/* 277 */     Class c = get().getClass(schema);
/* 278 */     if (c == null) return super.newRecord(old, schema);
/* 279 */     return c.isInstance(old) ? old : newInstance(c, schema);
/*     */   }
/*     */   
/*     */   public static abstract interface SchemaConstructable {}
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\org\apache\avro\specific\SpecificData.class
 * Java compiler version: 6 (50.0)
 * JD-Core Version:       0.7.1
 */