/*     */ package org.codehaus.janino;
/*     */ 
/*     */ import java.io.ByteArrayOutputStream;
/*     */ import java.io.File;
/*     */ import java.io.IOException;
/*     */ import java.io.InputStream;
/*     */ import java.io.OutputStream;
/*     */ import java.util.HashMap;
/*     */ import java.util.Iterator;
/*     */ import java.util.Map;
/*     */ import java.util.Map.Entry;
/*     */ import java.util.Set;
/*     */ import org.codehaus.janino.util.ClassFile;
/*     */ import org.codehaus.janino.util.resource.DirectoryResourceCreator;
/*     */ import org.codehaus.janino.util.resource.DirectoryResourceFinder;
/*     */ import org.codehaus.janino.util.resource.PathResourceFinder;
/*     */ import org.codehaus.janino.util.resource.Resource;
/*     */ import org.codehaus.janino.util.resource.ResourceCreator;
/*     */ import org.codehaus.janino.util.resource.ResourceFinder;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class CachingJavaSourceClassLoader
/*     */   extends JavaSourceClassLoader
/*     */ {
/*     */   private final ResourceFinder classFileCacheResourceFinder;
/*     */   private final ResourceCreator classFileCacheResourceCreator;
/*     */   private final ResourceFinder sourceFinder;
/*     */   
/*     */   public CachingJavaSourceClassLoader(ClassLoader parentClassLoader, File[] optionalSourcePath, String optionalCharacterEncoding, File cacheDirectory)
/*     */   {
/*  68 */     this(parentClassLoader, optionalSourcePath == null ? new DirectoryResourceFinder(new File(".")) : new PathResourceFinder(optionalSourcePath), optionalCharacterEncoding, new DirectoryResourceFinder(cacheDirectory), new DirectoryResourceCreator(cacheDirectory));
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public CachingJavaSourceClassLoader(ClassLoader parentClassLoader, ResourceFinder sourceFinder, String optionalCharacterEncoding, ResourceFinder classFileCacheResourceFinder, ResourceCreator classFileCacheResourceCreator)
/*     */   {
/* 107 */     super(parentClassLoader, sourceFinder, optionalCharacterEncoding);
/* 108 */     this.classFileCacheResourceFinder = classFileCacheResourceFinder;
/* 109 */     this.classFileCacheResourceCreator = classFileCacheResourceCreator;
/* 110 */     this.sourceFinder = sourceFinder;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected Map generateBytecodes(String className)
/*     */     throws ClassNotFoundException
/*     */   {
/* 123 */     Resource classFileResource = this.classFileCacheResourceFinder.findResource(ClassFile.getClassFileResourceName(className));
/*     */     
/*     */ 
/* 126 */     if (classFileResource != null)
/*     */     {
/*     */ 
/* 129 */       Resource sourceResource = this.sourceFinder.findResource(ClassFile.getSourceResourceName(className));
/* 130 */       if (sourceResource == null) { return null;
/*     */       }
/*     */       
/* 133 */       if (sourceResource.lastModified() < classFileResource.lastModified())
/*     */       {
/*     */         byte[] bytecode;
/*     */         try
/*     */         {
/* 138 */           bytecode = readResource(classFileResource);
/*     */         } catch (IOException ex) {
/* 140 */           throw new ClassNotFoundException("Reading class file from \"" + classFileResource + "\"", ex);
/*     */         }
/* 142 */         Map m = new HashMap();
/* 143 */         m.put(className, bytecode);
/* 144 */         return m;
/*     */       }
/*     */     }
/*     */     
/*     */ 
/*     */ 
/* 150 */     Map bytecodes = super.generateBytecodes(className);
/* 151 */     if (bytecodes == null) { return null;
/*     */     }
/*     */     
/* 154 */     for (Iterator it = bytecodes.entrySet().iterator(); it.hasNext();) {
/* 155 */       Map.Entry me = (Map.Entry)it.next();
/* 156 */       String className2 = (String)me.getKey();
/* 157 */       byte[] bytecode = (byte[])me.getValue();
/*     */       try
/*     */       {
/* 160 */         writeResource(this.classFileCacheResourceCreator, ClassFile.getClassFileResourceName(className2), bytecode);
/*     */ 
/*     */       }
/*     */       catch (IOException ex)
/*     */       {
/*     */ 
/* 166 */         throw new ClassNotFoundException("Writing class file to \"" + ClassFile.getClassFileResourceName(className2) + "\"", ex);
/*     */       }
/*     */     }
/*     */     
/*     */ 
/*     */ 
/*     */ 
/* 173 */     return bytecodes;
/*     */   }
/*     */   
/*     */ 
/*     */   private static byte[] readResource(Resource r)
/*     */     throws IOException
/*     */   {
/* 180 */     baos = new ByteArrayOutputStream();
/* 181 */     byte[] buffer = new byte['က'];
/*     */     
/* 183 */     InputStream is = r.open();
/*     */     try {
/*     */       for (;;) {
/* 186 */         int cnt = is.read(buffer);
/* 187 */         if (cnt == -1) break;
/* 188 */         baos.write(buffer, 0, cnt);
/*     */       }
/*     */       
/*     */ 
/*     */ 
/*     */ 
/* 194 */       return baos.toByteArray();
/*     */     }
/*     */     finally
/*     */     {
/*     */       try
/*     */       {
/* 191 */         is.close();
/*     */       }
/*     */       catch (IOException ex) {}
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   private static void writeResource(ResourceCreator resourceCreator, String resourceName, byte[] data)
/*     */     throws IOException
/*     */   {
/* 205 */     OutputStream os = resourceCreator.createResource(resourceName);
/*     */     try {
/* 207 */       os.write(data); return;
/*     */     } finally {
/* 209 */       try { os.close();
/*     */       }
/*     */       catch (IOException ex) {}
/*     */     }
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\org\codehaus\janino\CachingJavaSourceClassLoader.class
 * Java compiler version: 1 (45.3)
 * JD-Core Version:       0.7.1
 */