/*      */ package org.codehaus.janino;
/*      */ 
/*      */ import java.io.IOException;
/*      */ import java.util.ArrayList;
/*      */ import java.util.List;
/*      */ import org.codehaus.commons.compiler.CompileException;
/*      */ import org.codehaus.commons.compiler.Location;
/*      */ import org.codehaus.janino.util.enumerator.Enumerator;
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ public class Parser
/*      */ {
/*      */   private final Scanner scanner;
/*      */   
/*      */   public Parser(Scanner scanner)
/*      */   {
/*   44 */     this.scanner = scanner;
/*      */   }
/*      */   
/*      */   public Scanner getScanner() {
/*   48 */     return this.scanner;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public Java.CompilationUnit parseCompilationUnit()
/*      */     throws CompileException, IOException
/*      */   {
/*   59 */     Java.CompilationUnit compilationUnit = new Java.CompilationUnit(location().getFileName());
/*      */     
/*   61 */     if (peekKeyword("package")) {
/*   62 */       compilationUnit.setPackageDeclaration(parsePackageDeclaration());
/*      */     }
/*      */     
/*   65 */     while (peekKeyword("import")) {
/*   66 */       compilationUnit.addImportDeclaration(parseImportDeclaration());
/*      */     }
/*      */     
/*   69 */     while (!this.scanner.peek().isEOF()) {
/*   70 */       if (peekOperator(";")) {
/*   71 */         eatToken();
/*      */       }
/*      */       else {
/*   74 */         compilationUnit.addPackageMemberTypeDeclaration(parsePackageMemberTypeDeclaration());
/*      */       }
/*      */     }
/*      */     
/*   78 */     return compilationUnit;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   public Java.PackageDeclaration parsePackageDeclaration()
/*      */     throws CompileException, IOException
/*      */   {
/*   87 */     readKeyword("package");
/*   88 */     Location loc = location();
/*   89 */     String packageName = join(parseQualifiedIdentifier(), ".");
/*   90 */     readOperator(";");
/*   91 */     verifyStringIsConventionalPackageName(packageName, loc);
/*   92 */     return new Java.PackageDeclaration(loc, packageName);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   public Java.CompilationUnit.ImportDeclaration parseImportDeclaration()
/*      */     throws CompileException, IOException
/*      */   {
/*  101 */     readKeyword("import");
/*  102 */     Java.CompilationUnit.ImportDeclaration importDeclaration = parseImportDeclarationBody();
/*  103 */     readOperator(";");
/*  104 */     return importDeclaration;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   public Java.CompilationUnit.ImportDeclaration parseImportDeclarationBody()
/*      */     throws CompileException, IOException
/*      */   {
/*  113 */     Location loc = location();
/*      */     boolean isStatic;
/*  115 */     if (peekKeyword("static")) {
/*  116 */       boolean isStatic = true;
/*  117 */       eatToken();
/*      */     }
/*      */     else {
/*  120 */       isStatic = false;
/*      */     }
/*  122 */     List l = new ArrayList();
/*  123 */     l.add(readIdentifier());
/*      */     for (;;) {
/*  125 */       if (!peekOperator(".")) {
/*  126 */         String[] identifiers = (String[])l.toArray(new String[l.size()]);
/*  127 */         return isStatic ? new Java.CompilationUnit.SingleStaticImportDeclaration(loc, identifiers) : new Java.CompilationUnit.SingleTypeImportDeclaration(loc, identifiers);
/*      */       }
/*      */       
/*      */ 
/*      */ 
/*      */ 
/*  133 */       readOperator(".");
/*  134 */       if (peekOperator("*")) {
/*  135 */         eatToken();
/*  136 */         String[] identifiers = (String[])l.toArray(new String[l.size()]);
/*  137 */         return isStatic ? new Java.CompilationUnit.StaticImportOnDemandDeclaration(loc, identifiers) : new Java.CompilationUnit.TypeImportOnDemandDeclaration(loc, identifiers);
/*      */       }
/*      */       
/*      */ 
/*      */ 
/*      */ 
/*  143 */       l.add(readIdentifier());
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */   public String[] parseQualifiedIdentifier()
/*      */     throws CompileException, IOException
/*      */   {
/*  151 */     if (!this.scanner.peek().isIdentifier()) throwCompileException("Identifier expected");
/*  152 */     List l = new ArrayList();
/*  153 */     l.add(readIdentifier());
/*  154 */     while ((peekOperator(".")) && (this.scanner.peekNextButOne().isIdentifier())) {
/*  155 */       eatToken();
/*  156 */       l.add(readIdentifier());
/*      */     }
/*  158 */     return (String[])l.toArray(new String[l.size()]);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public Java.PackageMemberTypeDeclaration parsePackageMemberTypeDeclaration()
/*      */     throws CompileException, IOException
/*      */   {
/*  169 */     String optionalDocComment = this.scanner.doc();
/*      */     
/*  171 */     short modifiers = parseModifiersOpt();
/*      */     
/*      */     Java.PackageMemberTypeDeclaration res;
/*  174 */     if (peekKeyword("class")) {
/*  175 */       if (optionalDocComment == null) warning("CDCM", "Class doc comment missing", location());
/*  176 */       eatToken();
/*  177 */       res = (Java.PackageMemberClassDeclaration)parseClassDeclarationRest(optionalDocComment, modifiers, ClassDeclarationContext.COMPILATION_UNIT);
/*      */     }
/*      */     else
/*      */     {
/*      */       Java.PackageMemberTypeDeclaration res;
/*      */       
/*  183 */       if (peekKeyword("interface")) {
/*  184 */         if (optionalDocComment == null) warning("IDCM", "Interface doc comment missing", location());
/*  185 */         eatToken();
/*  186 */         res = (Java.PackageMemberInterfaceDeclaration)parseInterfaceDeclarationRest(optionalDocComment, modifiers, InterfaceDeclarationContext.COMPILATION_UNIT);
/*      */ 
/*      */ 
/*      */       }
/*      */       else
/*      */       {
/*      */ 
/*  193 */         throwCompileException("Unexpected token \"" + this.scanner.peek() + "\" in class or interface declaration");
/*      */         
/*      */ 
/*      */ 
/*      */ 
/*  198 */         return null; } }
/*      */     Java.PackageMemberTypeDeclaration res;
/*  200 */     return res;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public short parseModifiersOpt()
/*      */     throws CompileException, IOException
/*      */   {
/*  211 */     short mod = 0;
/*  212 */     while (peekKeyword()) {
/*  213 */       String kw = this.scanner.peek().getKeyword();
/*  214 */       short x = kw == "strictfp" ? 2048 : kw == "volatile" ? 64 : kw == "transient" ? 128 : kw == "synchronized" ? 32 : kw == "native" ? 256 : kw == "final" ? 16 : kw == "abstract" ? 1024 : kw == "static" ? 8 : kw == "private" ? 2 : kw == "protected" ? 4 : kw == "public" ? 1 : -1;
/*      */       
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  228 */       if (x == -1)
/*      */         break;
/*  230 */       eatToken();
/*  231 */       if ((mod & x) != 0) throwCompileException("Duplicate modifier \"" + kw + "\"");
/*  232 */       for (int i = 0; i < MUTUALS.length; i++) {
/*  233 */         short m = MUTUALS[i];
/*  234 */         if (((x & m) != 0) && ((mod & m) != 0)) {
/*  235 */           throwCompileException("Only one of \"" + Mod.shortToString(m) + "\" allowed");
/*      */         }
/*      */       }
/*  238 */       mod = (short)(mod | x);
/*      */     }
/*  240 */     return mod; }
/*      */   
/*  242 */   private static final short[] MUTUALS = { 7 };
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public Java.NamedClassDeclaration parseClassDeclarationRest(String optionalDocComment, short modifiers, ClassDeclarationContext context)
/*      */     throws CompileException, IOException
/*      */   {
/*  260 */     Location location = location();
/*  261 */     String className = readIdentifier();
/*  262 */     verifyIdentifierIsConventionalClassOrInterfaceName(className, location);
/*      */     
/*  264 */     Java.ReferenceType optionalExtendedType = null;
/*  265 */     if (peekKeyword("extends")) {
/*  266 */       eatToken();
/*  267 */       optionalExtendedType = parseReferenceType();
/*      */     }
/*      */     
/*  270 */     Java.ReferenceType[] implementedTypes = new Java.ReferenceType[0];
/*  271 */     if (peekKeyword("implements")) {
/*  272 */       eatToken();
/*  273 */       implementedTypes = parseReferenceTypeList();
/*      */     }
/*      */     
/*      */     Java.NamedClassDeclaration namedClassDeclaration;
/*  277 */     if (context == ClassDeclarationContext.COMPILATION_UNIT) {
/*  278 */       namedClassDeclaration = new Java.PackageMemberClassDeclaration(location, optionalDocComment, modifiers, className, optionalExtendedType, implementedTypes);
/*      */     }
/*      */     else
/*      */     {
/*      */       Java.NamedClassDeclaration namedClassDeclaration;
/*      */       
/*      */ 
/*      */ 
/*      */ 
/*  287 */       if (context == ClassDeclarationContext.TYPE_DECLARATION) {
/*  288 */         namedClassDeclaration = new Java.MemberClassDeclaration(location, optionalDocComment, modifiers, className, optionalExtendedType, implementedTypes);
/*      */       }
/*      */       else
/*      */       {
/*      */         Java.NamedClassDeclaration namedClassDeclaration;
/*      */         
/*      */ 
/*      */ 
/*      */ 
/*  297 */         if (context == ClassDeclarationContext.BLOCK) {
/*  298 */           namedClassDeclaration = new Java.LocalClassDeclaration(location, optionalDocComment, modifiers, className, optionalExtendedType, implementedTypes);
/*      */ 
/*      */ 
/*      */ 
/*      */         }
/*      */         else
/*      */         {
/*      */ 
/*      */ 
/*      */ 
/*  308 */           throw new JaninoRuntimeException("SNO: Class declaration in unexpected context " + context); }
/*      */       } }
/*      */     Java.NamedClassDeclaration namedClassDeclaration;
/*  311 */     parseClassBody(namedClassDeclaration);
/*  312 */     return namedClassDeclaration;
/*      */   }
/*      */   
/*  315 */   public static final class ClassDeclarationContext extends Enumerator { public static final ClassDeclarationContext BLOCK = new ClassDeclarationContext("block");
/*  316 */     public static final ClassDeclarationContext TYPE_DECLARATION = new ClassDeclarationContext("type_declaration");
/*  317 */     public static final ClassDeclarationContext COMPILATION_UNIT = new ClassDeclarationContext("compilation_unit");
/*  318 */     private ClassDeclarationContext(String name) { super(); }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public void parseClassBody(Java.ClassDeclaration classDeclaration)
/*      */     throws CompileException, IOException
/*      */   {
/*  329 */     if (!peekOperator("{")) throwCompileException("\"{\" expected at start of class body");
/*  330 */     eatToken();
/*      */     for (;;)
/*      */     {
/*  333 */       if (peekOperator("}")) {
/*  334 */         eatToken();
/*  335 */         return;
/*      */       }
/*      */       
/*  338 */       parseClassBodyDeclaration(classDeclaration);
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public void parseClassBodyDeclaration(Java.ClassDeclaration classDeclaration)
/*      */     throws CompileException, IOException
/*      */   {
/*  363 */     if (peekOperator(";")) {
/*  364 */       eatToken();
/*  365 */       return;
/*      */     }
/*      */     
/*  368 */     String optionalDocComment = this.scanner.doc();
/*  369 */     short modifiers = parseModifiersOpt();
/*      */     
/*      */ 
/*  372 */     if (peekOperator("{")) {
/*  373 */       if ((modifiers & 0xFFFFFFF7) != 0) {
/*  374 */         throwCompileException("Only modifier \"static\" allowed on initializer");
/*      */       }
/*      */       
/*  377 */       Java.Initializer initializer = new Java.Initializer(location(), (modifiers & 0x8) != 0, parseBlock());
/*      */       
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  383 */       classDeclaration.addVariableDeclaratorOrInitializer(initializer);
/*  384 */       return;
/*      */     }
/*      */     
/*      */ 
/*  388 */     if (peekKeyword("void")) {
/*  389 */       Location location = location();
/*  390 */       eatToken();
/*  391 */       if (optionalDocComment == null) warning("MDCM", "Method doc comment missing", location);
/*  392 */       String name = readIdentifier();
/*  393 */       classDeclaration.addDeclaredMethod(parseMethodDeclarationRest(optionalDocComment, modifiers, new Java.BasicType(location, 0), name));
/*      */       
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  399 */       return;
/*      */     }
/*      */     
/*      */ 
/*  403 */     if (peekKeyword("class")) {
/*  404 */       if (optionalDocComment == null) warning("MCDCM", "Member class doc comment missing", location());
/*  405 */       eatToken();
/*  406 */       classDeclaration.addMemberTypeDeclaration((Java.MemberTypeDeclaration)parseClassDeclarationRest(optionalDocComment, modifiers, ClassDeclarationContext.TYPE_DECLARATION));
/*      */       
/*      */ 
/*      */ 
/*      */ 
/*  411 */       return;
/*      */     }
/*      */     
/*      */ 
/*  415 */     if (peekKeyword("interface")) {
/*  416 */       if (optionalDocComment == null) {
/*  417 */         warning("MIDCM", "Member interface doc comment missing", location());
/*      */       }
/*  419 */       eatToken();
/*  420 */       classDeclaration.addMemberTypeDeclaration((Java.MemberTypeDeclaration)parseInterfaceDeclarationRest(optionalDocComment, (short)(modifiers | 0x8), InterfaceDeclarationContext.NAMED_TYPE_DECLARATION));
/*      */       
/*      */ 
/*      */ 
/*      */ 
/*  425 */       return;
/*      */     }
/*      */     
/*      */ 
/*  429 */     if (((classDeclaration instanceof Java.NamedClassDeclaration)) && (this.scanner.peek().isIdentifier(((Java.NamedClassDeclaration)classDeclaration).getName())) && (this.scanner.peekNextButOne().isOperator("(")))
/*      */     {
/*      */ 
/*      */ 
/*      */ 
/*  434 */       if (optionalDocComment == null) warning("CDCM", "Constructor doc comment missing", location());
/*  435 */       classDeclaration.addConstructor(parseConstructorDeclarator(optionalDocComment, modifiers));
/*      */       
/*      */ 
/*      */ 
/*  439 */       return;
/*      */     }
/*      */     
/*      */ 
/*  443 */     Java.Type memberType = parseType();
/*  444 */     Location location = location();
/*  445 */     String memberName = readIdentifier();
/*      */     
/*      */ 
/*  448 */     if (peekOperator("(")) {
/*  449 */       if (optionalDocComment == null) warning("MDCM", "Method doc comment missing", location());
/*  450 */       classDeclaration.addDeclaredMethod(parseMethodDeclarationRest(optionalDocComment, modifiers, memberType, memberName));
/*      */       
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  456 */       return;
/*      */     }
/*      */     
/*      */ 
/*  460 */     if (optionalDocComment == null) warning("FDCM", "Field doc comment missing", location());
/*  461 */     Java.FieldDeclaration fd = new Java.FieldDeclaration(location, optionalDocComment, modifiers, memberType, parseFieldDeclarationRest(memberName));
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  468 */     readOperator(";");
/*  469 */     classDeclaration.addVariableDeclaratorOrInitializer(fd);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public Java.InterfaceDeclaration parseInterfaceDeclarationRest(String optionalDocComment, short modifiers, InterfaceDeclarationContext context)
/*      */     throws CompileException, IOException
/*      */   {
/*  485 */     Location location = location();
/*  486 */     String interfaceName = readIdentifier();
/*  487 */     verifyIdentifierIsConventionalClassOrInterfaceName(interfaceName, location);
/*      */     
/*  489 */     Java.ReferenceType[] extendedTypes = new Java.ReferenceType[0];
/*  490 */     if (peekKeyword("extends")) {
/*  491 */       eatToken();
/*  492 */       extendedTypes = parseReferenceTypeList();
/*      */     }
/*      */     
/*      */     Java.InterfaceDeclaration interfaceDeclaration;
/*  496 */     if (context == InterfaceDeclarationContext.COMPILATION_UNIT) {
/*  497 */       interfaceDeclaration = new Java.PackageMemberInterfaceDeclaration(location, optionalDocComment, modifiers, interfaceName, extendedTypes);
/*      */     }
/*      */     else
/*      */     {
/*      */       Java.InterfaceDeclaration interfaceDeclaration;
/*      */       
/*      */ 
/*      */ 
/*  505 */       if (context == InterfaceDeclarationContext.NAMED_TYPE_DECLARATION) {
/*  506 */         interfaceDeclaration = new Java.MemberInterfaceDeclaration(location, optionalDocComment, modifiers, interfaceName, extendedTypes);
/*      */ 
/*      */ 
/*      */ 
/*      */       }
/*      */       else
/*      */       {
/*      */ 
/*      */ 
/*  515 */         throw new JaninoRuntimeException("SNO: Interface declaration in unexpected context " + context); }
/*      */     }
/*      */     Java.InterfaceDeclaration interfaceDeclaration;
/*  518 */     parseInterfaceBody(interfaceDeclaration);
/*  519 */     return interfaceDeclaration;
/*      */   }
/*      */   
/*      */   public static final class InterfaceDeclarationContext extends Enumerator {
/*  523 */     public static final InterfaceDeclarationContext NAMED_TYPE_DECLARATION = new InterfaceDeclarationContext("named_type_declaration");
/*  524 */     public static final InterfaceDeclarationContext COMPILATION_UNIT = new InterfaceDeclarationContext("compilation_unit");
/*      */     
/*      */     InterfaceDeclarationContext(String name) {
/*  527 */       super();
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public void parseInterfaceBody(Java.InterfaceDeclaration interfaceDeclaration)
/*      */     throws CompileException, IOException
/*      */   {
/*  549 */     readOperator("{");
/*      */     for (;;)
/*      */     {
/*  552 */       if (peekOperator("}")) {
/*  553 */         eatToken();
/*  554 */         break;
/*      */       }
/*      */       
/*  557 */       if (peekOperator(";")) {
/*  558 */         eatToken();
/*      */       }
/*      */       else
/*      */       {
/*  562 */         String optionalDocComment = this.scanner.doc();
/*  563 */         short modifiers = parseModifiersOpt();
/*      */         
/*      */ 
/*  566 */         if (peekKeyword("void")) {
/*  567 */           if (optionalDocComment == null) warning("MDCM", "Method doc comment missing", location());
/*  568 */           Location location = location();
/*  569 */           eatToken();
/*  570 */           String name = readIdentifier();
/*  571 */           interfaceDeclaration.addDeclaredMethod(parseMethodDeclarationRest(optionalDocComment, (short)(modifiers | 0x400 | 0x1), new Java.BasicType(location, 0), name));
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */         }
/*  580 */         else if (peekKeyword("class")) {
/*  581 */           if (optionalDocComment == null) {
/*  582 */             warning("MCDCM", "Member class doc comment missing", location());
/*      */           }
/*  584 */           eatToken();
/*  585 */           interfaceDeclaration.addMemberTypeDeclaration((Java.MemberTypeDeclaration)parseClassDeclarationRest(optionalDocComment, (short)(modifiers | 0x8 | 0x1), ClassDeclarationContext.TYPE_DECLARATION));
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */         }
/*  595 */         else if (peekKeyword("interface")) {
/*  596 */           if (optionalDocComment == null) {
/*  597 */             warning("MIDCM", "Member interface doc comment missing", location());
/*      */           }
/*  599 */           eatToken();
/*  600 */           interfaceDeclaration.addMemberTypeDeclaration((Java.MemberTypeDeclaration)parseInterfaceDeclarationRest(optionalDocComment, (short)(modifiers | 0x8 | 0x1), InterfaceDeclarationContext.NAMED_TYPE_DECLARATION));
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */         }
/*      */         else
/*      */         {
/*      */ 
/*      */ 
/*      */ 
/*  611 */           Java.Type memberType = parseType();
/*  612 */           if (!this.scanner.peek().isIdentifier()) {
/*  613 */             throwCompileException("Identifier expected in member declaration");
/*      */           }
/*  615 */           Location location = location();
/*  616 */           String memberName = readIdentifier();
/*      */           
/*      */ 
/*  619 */           if (peekOperator("(")) {
/*  620 */             if (optionalDocComment == null) warning("MDCM", "Method doc comment missing", location());
/*  621 */             interfaceDeclaration.addDeclaredMethod(parseMethodDeclarationRest(optionalDocComment, (short)(modifiers | 0x400 | 0x1), memberType, memberName));
/*      */ 
/*      */ 
/*      */ 
/*      */           }
/*      */           else
/*      */           {
/*      */ 
/*      */ 
/*      */ 
/*  631 */             if (optionalDocComment == null) warning("FDCM", "Field doc comment missing", location());
/*  632 */             Java.FieldDeclaration fd = new Java.FieldDeclaration(location, optionalDocComment, (short)(modifiers | 0x1 | 0x8 | 0x10), memberType, parseFieldDeclarationRest(memberName));
/*      */             
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  642 */             interfaceDeclaration.addConstantDeclaration(fd);
/*      */           }
/*      */         }
/*      */       }
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public Java.ConstructorDeclarator parseConstructorDeclarator(String optionalDocComment, short modifiers)
/*      */     throws CompileException, IOException
/*      */   {
/*  664 */     Location location = location();
/*  665 */     readIdentifier();
/*      */     
/*      */ 
/*  668 */     Java.FunctionDeclarator.FormalParameter[] formalParameters = parseFormalParameters();
/*      */     
/*      */     Java.ReferenceType[] thrownExceptions;
/*      */     
/*      */     Java.ReferenceType[] thrownExceptions;
/*  673 */     if (peekKeyword("throws")) {
/*  674 */       eatToken();
/*  675 */       thrownExceptions = parseReferenceTypeList();
/*      */     } else {
/*  677 */       thrownExceptions = new Java.ReferenceType[0];
/*      */     }
/*      */     
/*      */ 
/*  681 */     location = location();
/*  682 */     readOperator("{");
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*  687 */     Java.ConstructorInvocation optionalConstructorInvocation = null;
/*  688 */     List statements = new ArrayList();
/*  689 */     if ((peekKeyword(new String[] { "this", "super", "new", "void", "byte", "char", "short", "int", "long", "float", "double", "boolean" })) || (this.scanner.peek().isLiteral()) || (this.scanner.peek().isIdentifier()))
/*      */     {
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  697 */       Java.Atom a = parseExpression();
/*  698 */       if ((a instanceof Java.ConstructorInvocation)) {
/*  699 */         readOperator(";");
/*  700 */         optionalConstructorInvocation = (Java.ConstructorInvocation)a;
/*      */       } else {
/*      */         Java.Statement s;
/*  703 */         if (this.scanner.peek().isIdentifier()) {
/*  704 */           Java.Type variableType = a.toTypeOrPE();
/*  705 */           Java.Statement s = new Java.LocalVariableDeclarationStatement(a.getLocation(), (short)0, variableType, parseLocalVariableDeclarators());
/*      */           
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  711 */           readOperator(";");
/*      */         } else {
/*  713 */           s = new Java.ExpressionStatement(a.toRvalueOrPE());
/*  714 */           readOperator(";");
/*      */         }
/*  716 */         statements.add(s);
/*      */       }
/*      */     }
/*  719 */     statements.addAll(parseBlockStatements());
/*      */     
/*  721 */     readOperator("}");
/*      */     
/*  723 */     return new Java.ConstructorDeclarator(location, optionalDocComment, modifiers, formalParameters, thrownExceptions, optionalConstructorInvocation, statements);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public Java.MethodDeclarator parseMethodDeclarationRest(String optionalDocComment, short modifiers, Java.Type type, String name)
/*      */     throws CompileException, IOException
/*      */   {
/*  749 */     Location location = location();
/*      */     
/*  751 */     verifyIdentifierIsConventionalMethodName(name, location);
/*      */     
/*  753 */     Java.FunctionDeclarator.FormalParameter[] formalParameters = parseFormalParameters();
/*      */     
/*  755 */     for (int i = parseBracketsOpt(); i > 0; i--) type = new Java.ArrayType(type);
/*      */     Java.ReferenceType[] thrownExceptions;
/*      */     Java.ReferenceType[] thrownExceptions;
/*  758 */     if (peekKeyword("throws")) {
/*  759 */       eatToken();
/*  760 */       thrownExceptions = parseReferenceTypeList();
/*      */     } else {
/*  762 */       thrownExceptions = new Java.ReferenceType[0];
/*      */     }
/*      */     List optionalStatements;
/*      */     List optionalStatements;
/*  766 */     if (peekOperator(";")) {
/*  767 */       if ((modifiers & 0x500) == 0) {
/*  768 */         throwCompileException("Non-abstract, non-native method must have a body");
/*      */       }
/*  770 */       eatToken();
/*  771 */       optionalStatements = null;
/*      */     } else {
/*  773 */       if ((modifiers & 0x500) != 0) {
/*  774 */         throwCompileException("Abstract or native method must not have a body");
/*      */       }
/*  776 */       readOperator("{");
/*  777 */       optionalStatements = parseBlockStatements();
/*  778 */       readOperator("}");
/*      */     }
/*  780 */     return new Java.MethodDeclarator(location, optionalDocComment, modifiers, type, name, formalParameters, thrownExceptions, optionalStatements);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public Java.ArrayInitializerOrRvalue parseVariableInitializer()
/*      */     throws CompileException, IOException
/*      */   {
/*  800 */     if (peekOperator("{")) {
/*  801 */       return parseArrayInitializer();
/*      */     }
/*      */     
/*  804 */     return parseExpression().toRvalueOrPE();
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public Java.ArrayInitializer parseArrayInitializer()
/*      */     throws CompileException, IOException
/*      */   {
/*  815 */     Location location = location();
/*  816 */     readOperator("{");
/*  817 */     List l = new ArrayList();
/*  818 */     while (!peekOperator("}")) {
/*  819 */       l.add(parseVariableInitializer());
/*  820 */       if (peekOperator("}")) break;
/*  821 */       if (!peekOperator(",")) throwCompileException("\",\" or \"}\" expected");
/*  822 */       eatToken();
/*      */     }
/*  824 */     eatToken();
/*  825 */     return new Java.ArrayInitializer(location, (Java.ArrayInitializerOrRvalue[])l.toArray(new Java.ArrayInitializerOrRvalue[l.size()]));
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public Java.FunctionDeclarator.FormalParameter[] parseFormalParameters()
/*      */     throws CompileException, IOException
/*      */   {
/*  838 */     readOperator("(");
/*  839 */     if (peekOperator(")")) {
/*  840 */       eatToken();
/*  841 */       return new Java.FunctionDeclarator.FormalParameter[0];
/*      */     }
/*      */     
/*  844 */     List l = new ArrayList();
/*      */     for (;;) {
/*  846 */       l.add(parseFormalParameter());
/*  847 */       if (!peekOperator(",")) break;
/*  848 */       eatToken();
/*      */     }
/*  850 */     readOperator(")");
/*  851 */     return (Java.FunctionDeclarator.FormalParameter[])l.toArray(new Java.FunctionDeclarator.FormalParameter[l.size()]);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public Java.FunctionDeclarator.FormalParameter parseFormalParameter()
/*      */     throws CompileException, IOException
/*      */   {
/*  862 */     boolean finaL = peekKeyword("final");
/*  863 */     if (finaL) { eatToken();
/*      */     }
/*  865 */     Java.Type type = parseType();
/*      */     
/*  867 */     Location location = location();
/*  868 */     String name = readIdentifier();
/*  869 */     verifyIdentifierIsConventionalLocalVariableOrParameterName(name, location);
/*      */     
/*  871 */     for (int i = parseBracketsOpt(); i > 0; i--) type = new Java.ArrayType(type);
/*  872 */     return new Java.FunctionDeclarator.FormalParameter(location, finaL, type, name);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   int parseBracketsOpt()
/*      */     throws CompileException, IOException
/*      */   {
/*  881 */     int res = 0;
/*      */     
/*      */ 
/*  884 */     while ((this.scanner.peek().isOperator("[")) && (this.scanner.peekNextButOne().isOperator("]")))
/*      */     {
/*  886 */       eatToken();
/*  887 */       eatToken();
/*  888 */       res++;
/*      */     }
/*  890 */     return res;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   public Java.Block parseMethodBody()
/*      */     throws CompileException, IOException
/*      */   {
/*  899 */     return parseBlock();
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   public Java.Block parseBlock()
/*      */     throws CompileException, IOException
/*      */   {
/*  908 */     Java.Block block = new Java.Block(location());
/*  909 */     readOperator("{");
/*  910 */     block.addStatements(parseBlockStatements());
/*  911 */     readOperator("}");
/*  912 */     return block;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   public List parseBlockStatements()
/*      */     throws CompileException, IOException
/*      */   {
/*  921 */     List l = new ArrayList();
/*      */     
/*      */ 
/*  924 */     while ((!peekOperator("}")) && (!peekKeyword("case")) && (!peekKeyword("default")))
/*      */     {
/*  926 */       l.add(parseBlockStatement()); }
/*  927 */     return l;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public Java.BlockStatement parseBlockStatement()
/*      */     throws CompileException, IOException
/*      */   {
/*  947 */     if ((!this.scanner.peek().isIdentifier()) || (!this.scanner.peekNextButOne().isOperator(":"))) { if (!peekKeyword(new String[] { "if", "for", "while", "do", "try", "switch", "synchronized", "return", "throw", "break", "continue" })) { if (!peekOperator(new String[] { "{", ";" })) {
/*      */           break label126;
/*      */         }
/*      */       }
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  957 */     return parseStatement();
/*      */     
/*      */     label126:
/*  960 */     if (peekKeyword("class"))
/*      */     {
/*      */ 
/*  963 */       String optionalDocComment = this.scanner.doc();
/*  964 */       if (optionalDocComment == null) { warning("LCDCM", "Local class doc comment missing", location());
/*      */       }
/*  966 */       eatToken();
/*  967 */       Java.LocalClassDeclaration lcd = (Java.LocalClassDeclaration)parseClassDeclarationRest(optionalDocComment, (short)0, ClassDeclarationContext.BLOCK);
/*      */       
/*      */ 
/*      */ 
/*      */ 
/*  972 */       return new Java.LocalClassDeclarationStatement(lcd);
/*      */     }
/*      */     
/*      */ 
/*  976 */     if (peekKeyword("final")) {
/*  977 */       Location location = location();
/*  978 */       eatToken();
/*  979 */       Java.Type variableType = parseType();
/*  980 */       Java.LocalVariableDeclarationStatement lvds = new Java.LocalVariableDeclarationStatement(location, (short)16, variableType, parseLocalVariableDeclarators());
/*      */       
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  986 */       readOperator(";");
/*  987 */       return lvds;
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*  992 */     Java.Atom a = parseExpression();
/*      */     
/*      */ 
/*  995 */     if (peekOperator(";")) {
/*  996 */       eatToken();
/*  997 */       return new Java.ExpressionStatement(a.toRvalueOrPE());
/*      */     }
/*      */     
/*      */ 
/* 1001 */     Java.Type variableType = a.toTypeOrPE();
/* 1002 */     Java.LocalVariableDeclarationStatement lvds = new Java.LocalVariableDeclarationStatement(a.getLocation(), (short)0, variableType, parseLocalVariableDeclarators());
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 1008 */     readOperator(";");
/* 1009 */     return lvds;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   public Java.VariableDeclarator[] parseLocalVariableDeclarators()
/*      */     throws CompileException, IOException
/*      */   {
/* 1018 */     List l = new ArrayList();
/*      */     for (;;) {
/* 1020 */       Java.VariableDeclarator vd = parseVariableDeclarator();
/* 1021 */       verifyIdentifierIsConventionalLocalVariableOrParameterName(vd.name, vd.getLocation());
/* 1022 */       l.add(vd);
/* 1023 */       if (!peekOperator(",")) break;
/* 1024 */       eatToken();
/*      */     }
/* 1026 */     return (Java.VariableDeclarator[])l.toArray(new Java.VariableDeclarator[l.size()]);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public Java.VariableDeclarator[] parseFieldDeclarationRest(String name)
/*      */     throws CompileException, IOException
/*      */   {
/* 1037 */     List l = new ArrayList();
/*      */     
/* 1039 */     Java.VariableDeclarator vd = parseVariableDeclaratorRest(name);
/* 1040 */     verifyIdentifierIsConventionalFieldName(vd.name, vd.getLocation());
/* 1041 */     l.add(vd);
/*      */     
/* 1043 */     while (peekOperator(",")) {
/* 1044 */       eatToken();
/*      */       
/* 1046 */       vd = parseVariableDeclarator();
/* 1047 */       verifyIdentifierIsConventionalFieldName(vd.name, vd.getLocation());
/* 1048 */       l.add(vd);
/*      */     }
/* 1050 */     return (Java.VariableDeclarator[])l.toArray(new Java.VariableDeclarator[l.size()]);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   public Java.VariableDeclarator parseVariableDeclarator()
/*      */     throws CompileException, IOException
/*      */   {
/* 1059 */     return parseVariableDeclaratorRest(readIdentifier());
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public Java.VariableDeclarator parseVariableDeclaratorRest(String name)
/*      */     throws CompileException, IOException
/*      */   {
/* 1069 */     Location loc = location();
/* 1070 */     int brackets = parseBracketsOpt();
/* 1071 */     Java.ArrayInitializerOrRvalue initializer = null;
/* 1072 */     if (peekOperator("=")) {
/* 1073 */       eatToken();
/* 1074 */       initializer = parseVariableInitializer();
/*      */     }
/* 1076 */     return new Java.VariableDeclarator(loc, name, brackets, initializer);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public Java.Statement parseStatement()
/*      */     throws CompileException, IOException
/*      */   {
/* 1100 */     if ((this.scanner.peek().isIdentifier()) && (this.scanner.peekNextButOne().isOperator(":")))
/*      */     {
/*      */ 
/*      */ 
/* 1104 */       return parseLabeledStatement();
/*      */     }
/*      */     
/* 1107 */     Scanner.Token t = this.scanner.peek();
/* 1108 */     Java.Statement stmt = t.isOperator(";") ? parseEmptyStatement() : t.isKeyword("continue") ? parseContinueStatement() : t.isKeyword("break") ? parseBreakStatement() : t.isKeyword("throw") ? parseThrowStatement() : t.isKeyword("return") ? parseReturnStatement() : t.isKeyword("synchronized") ? parseSynchronizedStatement() : t.isKeyword("switch") ? parseSwitchStatement() : t.isKeyword("try") ? parseTryStatement() : t.isKeyword("do") ? parseDoStatement() : t.isKeyword("while") ? parseWhileStatement() : t.isKeyword("for") ? parseForStatement() : t.isKeyword("if") ? parseIfStatement() : t.isOperator("{") ? parseBlock() : parseExpressionStatement();
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 1124 */     if (stmt == null) { throwCompileException("\"" + t.getKeyword() + "\" NYI");
/*      */     }
/* 1126 */     return stmt;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   public Java.Statement parseLabeledStatement()
/*      */     throws CompileException, IOException
/*      */   {
/* 1135 */     String label = readIdentifier();
/* 1136 */     readOperator(":");
/* 1137 */     return new Java.LabeledStatement(location(), label, parseStatement());
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public Java.Statement parseIfStatement()
/*      */     throws CompileException, IOException
/*      */   {
/* 1150 */     Location location = location();
/* 1151 */     readKeyword("if");
/* 1152 */     readOperator("(");
/* 1153 */     Java.Rvalue condition = parseExpression().toRvalueOrPE();
/* 1154 */     readOperator(")");
/*      */     
/* 1156 */     Java.Statement thenStatement = parseStatement();
/*      */     
/* 1158 */     Java.Statement optionalElseStatement = null;
/* 1159 */     if (peekKeyword("else")) {
/* 1160 */       eatToken();
/* 1161 */       optionalElseStatement = parseStatement();
/*      */     }
/*      */     
/* 1164 */     return new Java.IfStatement(location, condition, thenStatement, optionalElseStatement);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public Java.Statement parseForStatement()
/*      */     throws CompileException, IOException
/*      */   {
/* 1183 */     Location location = location();
/* 1184 */     readKeyword("for");
/*      */     
/* 1186 */     readOperator("(");
/*      */     
/* 1188 */     Java.BlockStatement optionalInit = null;
/* 1189 */     if (!peekOperator(";")) { optionalInit = parseForInit();
/*      */     }
/* 1191 */     readOperator(";");
/*      */     
/* 1193 */     Java.Rvalue optionalCondition = null;
/* 1194 */     if (!peekOperator(";")) { optionalCondition = parseExpression().toRvalueOrPE();
/*      */     }
/* 1196 */     readOperator(";");
/*      */     
/* 1198 */     Java.Rvalue[] optionalUpdate = null;
/* 1199 */     if (!peekOperator(")")) { optionalUpdate = parseExpressionList();
/*      */     }
/* 1201 */     readOperator(")");
/*      */     
/*      */ 
/*      */ 
/* 1205 */     return new Java.ForStatement(location, optionalInit, optionalCondition, optionalUpdate, parseStatement());
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private Java.BlockStatement parseForInit()
/*      */     throws CompileException, IOException
/*      */   {
/* 1234 */     if (peekKeyword(new String[] { "final", "byte", "short", "char", "int", "long", "float", "double", "boolean" }))
/*      */     {
/*      */ 
/* 1237 */       short modifiers = parseModifiersOpt();
/* 1238 */       Java.Type variableType = parseType();
/* 1239 */       return new Java.LocalVariableDeclarationStatement(location(), modifiers, variableType, parseLocalVariableDeclarators());
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 1247 */     Java.Atom a = parseExpression();
/*      */     
/*      */ 
/* 1250 */     if (this.scanner.peek().isIdentifier()) {
/* 1251 */       Java.Type variableType = a.toTypeOrPE();
/* 1252 */       return new Java.LocalVariableDeclarationStatement(a.getLocation(), (short)0, variableType, parseLocalVariableDeclarators());
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 1261 */     if (!peekOperator(",")) {
/* 1262 */       return new Java.ExpressionStatement(a.toRvalueOrPE());
/*      */     }
/* 1264 */     eatToken();
/* 1265 */     List l = new ArrayList();
/* 1266 */     l.add(new Java.ExpressionStatement(a.toRvalueOrPE()));
/*      */     for (;;) {
/* 1268 */       l.add(new Java.ExpressionStatement(parseExpression().toRvalueOrPE()));
/* 1269 */       if (!peekOperator(",")) break;
/* 1270 */       eatToken();
/*      */     }
/*      */     
/* 1273 */     Java.Block b = new Java.Block(a.getLocation());
/* 1274 */     b.addStatements(l);
/* 1275 */     return b;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   public Java.Statement parseWhileStatement()
/*      */     throws CompileException, IOException
/*      */   {
/* 1284 */     Location location = location();
/* 1285 */     readKeyword("while");
/*      */     
/* 1287 */     readOperator("(");
/* 1288 */     Java.Rvalue condition = parseExpression().toRvalueOrPE();
/* 1289 */     readOperator(")");
/*      */     
/* 1291 */     return new Java.WhileStatement(location, condition, parseStatement());
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public Java.Statement parseDoStatement()
/*      */     throws CompileException, IOException
/*      */   {
/* 1304 */     Location location = location();
/* 1305 */     readKeyword("do");
/*      */     
/* 1307 */     Java.Statement body = parseStatement();
/*      */     
/* 1309 */     readKeyword("while");
/* 1310 */     readOperator("(");
/* 1311 */     Java.Rvalue condition = parseExpression().toRvalueOrPE();
/* 1312 */     readOperator(")");
/* 1313 */     readOperator(";");
/*      */     
/* 1315 */     return new Java.DoStatement(location, body, condition);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public Java.Statement parseTryStatement()
/*      */     throws CompileException, IOException
/*      */   {
/* 1336 */     Location location = location();
/* 1337 */     readKeyword("try");
/*      */     
/* 1339 */     Java.Block body = parseBlock();
/*      */     
/*      */ 
/* 1342 */     List ccs = new ArrayList();
/* 1343 */     while (peekKeyword("catch")) {
/* 1344 */       Location loc = location();
/* 1345 */       eatToken();
/* 1346 */       readOperator("(");
/* 1347 */       Java.FunctionDeclarator.FormalParameter caughtException = parseFormalParameter();
/* 1348 */       readOperator(")");
/* 1349 */       ccs.add(new Java.CatchClause(loc, caughtException, parseBlock()));
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/* 1355 */     Java.Block optionalFinally = null;
/* 1356 */     if (peekKeyword("finally")) {
/* 1357 */       eatToken();
/* 1358 */       optionalFinally = parseBlock();
/*      */     }
/* 1360 */     if ((ccs.size() == 0) && (optionalFinally == null)) {
/* 1361 */       throwCompileException("\"try\" statement must have at least one \"catch\" clause or a \"finally\" clause");
/*      */     }
/*      */     
/*      */ 
/*      */ 
/* 1366 */     return new Java.TryStatement(location, body, ccs, optionalFinally);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public Java.Statement parseSwitchStatement()
/*      */     throws CompileException, IOException
/*      */   {
/* 1385 */     Location location = location();
/* 1386 */     readKeyword("switch");
/*      */     
/* 1388 */     readOperator("(");
/* 1389 */     Java.Rvalue condition = parseExpression().toRvalueOrPE();
/* 1390 */     readOperator(")");
/*      */     
/* 1392 */     readOperator("{");
/* 1393 */     List sbsgs = new ArrayList();
/* 1394 */     while (!peekOperator("}")) {
/* 1395 */       Location location2 = location();
/* 1396 */       boolean hasDefaultLabel = false;
/* 1397 */       List caseLabels = new ArrayList();
/*      */       do {
/* 1399 */         if (peekKeyword("case")) {
/* 1400 */           eatToken();
/* 1401 */           caseLabels.add(parseExpression().toRvalueOrPE());
/*      */         }
/* 1403 */         else if (peekKeyword("default")) {
/* 1404 */           eatToken();
/* 1405 */           if (hasDefaultLabel) throwCompileException("Duplicate \"default\" label");
/* 1406 */           hasDefaultLabel = true;
/*      */         } else {
/* 1408 */           throwCompileException("\"case\" or \"default\" expected");
/*      */         }
/* 1410 */         readOperator(":");
/* 1411 */       } while (peekKeyword(new String[] { "case", "default" }));
/*      */       
/* 1413 */       Java.SwitchStatement.SwitchBlockStatementGroup sbsg = new Java.SwitchStatement.SwitchBlockStatementGroup(location2, caseLabels, hasDefaultLabel, parseBlockStatements());
/*      */       
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 1419 */       sbsgs.add(sbsg);
/*      */     }
/* 1421 */     eatToken();
/* 1422 */     return new Java.SwitchStatement(location, condition, sbsgs);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public Java.Statement parseSynchronizedStatement()
/*      */     throws CompileException, IOException
/*      */   {
/* 1436 */     Location location = location();
/* 1437 */     readKeyword("synchronized");
/* 1438 */     readOperator("(");
/* 1439 */     Java.Rvalue expression = parseExpression().toRvalueOrPE();
/* 1440 */     readOperator(")");
/* 1441 */     return new Java.SynchronizedStatement(location, expression, parseBlock());
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public Java.Statement parseReturnStatement()
/*      */     throws CompileException, IOException
/*      */   {
/* 1454 */     Location location = location();
/* 1455 */     readKeyword("return");
/* 1456 */     Java.Rvalue returnValue = peekOperator(";") ? null : parseExpression().toRvalueOrPE();
/* 1457 */     readOperator(";");
/* 1458 */     return new Java.ReturnStatement(location, returnValue);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   public Java.Statement parseThrowStatement()
/*      */     throws CompileException, IOException
/*      */   {
/* 1467 */     Location location = location();
/* 1468 */     readKeyword("throw");
/* 1469 */     Java.Rvalue expression = parseExpression().toRvalueOrPE();
/* 1470 */     readOperator(";");
/*      */     
/* 1472 */     return new Java.ThrowStatement(location, expression);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   public Java.Statement parseBreakStatement()
/*      */     throws CompileException, IOException
/*      */   {
/* 1481 */     Location location = location();
/* 1482 */     readKeyword("break");
/* 1483 */     String optionalLabel = null;
/* 1484 */     if (this.scanner.peek().isIdentifier()) optionalLabel = readIdentifier();
/* 1485 */     readOperator(";");
/* 1486 */     return new Java.BreakStatement(location, optionalLabel);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   public Java.Statement parseContinueStatement()
/*      */     throws CompileException, IOException
/*      */   {
/* 1495 */     Location location = location();
/* 1496 */     readKeyword("continue");
/* 1497 */     String optionalLabel = null;
/* 1498 */     if (this.scanner.peek().isIdentifier()) optionalLabel = readIdentifier();
/* 1499 */     readOperator(";");
/* 1500 */     return new Java.ContinueStatement(location, optionalLabel);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   public Java.Statement parseEmptyStatement()
/*      */     throws CompileException, IOException
/*      */   {
/* 1509 */     Location location = location();
/* 1510 */     readOperator(";");
/* 1511 */     return new Java.EmptyStatement(location);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   public Java.Rvalue[] parseExpressionList()
/*      */     throws CompileException, IOException
/*      */   {
/* 1520 */     List l = new ArrayList();
/*      */     for (;;) {
/* 1522 */       l.add(parseExpression().toRvalueOrPE());
/* 1523 */       if (!peekOperator(",")) break;
/* 1524 */       eatToken();
/*      */     }
/* 1526 */     return (Java.Rvalue[])l.toArray(new Java.Rvalue[l.size()]);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public Java.Type parseType()
/*      */     throws CompileException, IOException
/*      */   {
/* 1539 */     Scanner.Token t = this.scanner.peek();
/* 1540 */     int bt = -1;
/* 1541 */     if (t.isKeyword("byte")) { bt = 1;
/* 1542 */     } else if (t.isKeyword("short")) { bt = 2;
/* 1543 */     } else if (t.isKeyword("char")) { bt = 3;
/* 1544 */     } else if (t.isKeyword("int")) { bt = 4;
/* 1545 */     } else if (t.isKeyword("long")) { bt = 5;
/* 1546 */     } else if (t.isKeyword("float")) { bt = 6;
/* 1547 */     } else if (t.isKeyword("double")) { bt = 7;
/* 1548 */     } else if (t.isKeyword("boolean")) bt = 8;
/*      */     Java.Type res;
/* 1550 */     if (bt != -1) {
/* 1551 */       Java.Type res = new Java.BasicType(t.getLocation(), bt);
/* 1552 */       eatToken();
/*      */     } else {
/* 1554 */       res = parseReferenceType();
/*      */     }
/* 1556 */     for (int i = parseBracketsOpt(); i > 0; i--) res = new Java.ArrayType(res);
/* 1557 */     return res;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   public Java.ReferenceType parseReferenceType()
/*      */     throws CompileException, IOException
/*      */   {
/* 1566 */     return new Java.ReferenceType(location(), parseQualifiedIdentifier());
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public Java.ReferenceType[] parseReferenceTypeList()
/*      */     throws CompileException, IOException
/*      */   {
/* 1578 */     List l = new ArrayList();
/* 1579 */     l.add(parseReferenceType());
/* 1580 */     while (peekOperator(",")) {
/* 1581 */       eatToken();
/* 1582 */       l.add(parseReferenceType());
/*      */     }
/* 1584 */     return (Java.ReferenceType[])l.toArray(new Java.ReferenceType[l.size()]);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   public Java.Atom parseExpression()
/*      */     throws CompileException, IOException
/*      */   {
/* 1593 */     return parseAssignmentExpression();
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public Java.Atom parseAssignmentExpression()
/*      */     throws CompileException, IOException
/*      */   {
/* 1607 */     Java.Atom a = parseConditionalExpression();
/* 1608 */     if (peekOperator(new String[] { "=", "+=", "-=", "*=", "/=", "&=", "|=", "^=", "%=", "<<=", ">>=", ">>>=" }))
/*      */     {
/*      */ 
/* 1611 */       Location location = location();
/* 1612 */       String operator = readOperator();
/* 1613 */       Java.Lvalue lhs = a.toLvalueOrPE();
/* 1614 */       Java.Rvalue rhs = parseAssignmentExpression().toRvalueOrPE();
/* 1615 */       return new Java.Assignment(location, lhs, operator, rhs);
/*      */     }
/* 1617 */     return a;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public Java.Atom parseConditionalExpression()
/*      */     throws CompileException, IOException
/*      */   {
/* 1627 */     Java.Atom a = parseConditionalOrExpression();
/* 1628 */     if (!peekOperator("?")) return a;
/* 1629 */     Location location = location();
/* 1630 */     eatToken();
/*      */     
/* 1632 */     Java.Rvalue lhs = a.toRvalueOrPE();
/* 1633 */     Java.Rvalue mhs = parseExpression().toRvalueOrPE();
/* 1634 */     readOperator(":");
/* 1635 */     Java.Rvalue rhs = parseConditionalExpression().toRvalueOrPE();
/* 1636 */     return new Java.ConditionalExpression(location, lhs, mhs, rhs);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public Java.Atom parseConditionalOrExpression()
/*      */     throws CompileException, IOException
/*      */   {
/* 1646 */     Java.Atom a = parseConditionalAndExpression();
/* 1647 */     while (peekOperator("||")) {
/* 1648 */       Location location = location();
/* 1649 */       eatToken();
/* 1650 */       a = new Java.BinaryOperation(location, a.toRvalueOrPE(), "||", parseConditionalAndExpression().toRvalueOrPE());
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 1657 */     return a;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public Java.Atom parseConditionalAndExpression()
/*      */     throws CompileException, IOException
/*      */   {
/* 1667 */     Java.Atom a = parseInclusiveOrExpression();
/* 1668 */     while (peekOperator("&&")) {
/* 1669 */       Location location = location();
/* 1670 */       eatToken();
/* 1671 */       a = new Java.BinaryOperation(location, a.toRvalueOrPE(), "&&", parseInclusiveOrExpression().toRvalueOrPE());
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 1678 */     return a;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public Java.Atom parseInclusiveOrExpression()
/*      */     throws CompileException, IOException
/*      */   {
/* 1688 */     Java.Atom a = parseExclusiveOrExpression();
/* 1689 */     while (peekOperator("|")) {
/* 1690 */       Location location = location();
/* 1691 */       eatToken();
/* 1692 */       a = new Java.BinaryOperation(location, a.toRvalueOrPE(), "|", parseExclusiveOrExpression().toRvalueOrPE());
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 1699 */     return a;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public Java.Atom parseExclusiveOrExpression()
/*      */     throws CompileException, IOException
/*      */   {
/* 1709 */     Java.Atom a = parseAndExpression();
/* 1710 */     while (peekOperator("^")) {
/* 1711 */       Location location = location();
/* 1712 */       eatToken();
/* 1713 */       a = new Java.BinaryOperation(location, a.toRvalueOrPE(), "^", parseAndExpression().toRvalueOrPE());
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 1720 */     return a;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public Java.Atom parseAndExpression()
/*      */     throws CompileException, IOException
/*      */   {
/* 1730 */     Java.Atom a = parseEqualityExpression();
/* 1731 */     while (peekOperator("&")) {
/* 1732 */       Location location = location();
/* 1733 */       eatToken();
/* 1734 */       a = new Java.BinaryOperation(location, a.toRvalueOrPE(), "&", parseEqualityExpression().toRvalueOrPE());
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 1741 */     return a;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public Java.Atom parseEqualityExpression()
/*      */     throws CompileException, IOException
/*      */   {
/* 1751 */     Java.Atom a = parseRelationalExpression();
/*      */     
/* 1753 */     while (peekOperator(new String[] { "==", "!=" })) {
/* 1754 */       a = new Java.BinaryOperation(location(), a.toRvalueOrPE(), readOperator(), parseRelationalExpression().toRvalueOrPE());
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 1761 */     return a;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public Java.Atom parseRelationalExpression()
/*      */     throws CompileException, IOException
/*      */   {
/* 1774 */     Java.Atom a = parseShiftExpression();
/*      */     for (;;)
/*      */     {
/* 1777 */       if (peekKeyword("instanceof")) {
/* 1778 */         Location location = location();
/* 1779 */         eatToken();
/* 1780 */         a = new Java.Instanceof(location, a.toRvalueOrPE(), parseType());
/*      */ 
/*      */       }
/*      */       else
/*      */       {
/*      */ 
/* 1786 */         if (!peekOperator(new String[] { "<", ">", "<=", ">=" })) break;
/* 1787 */         a = new Java.BinaryOperation(location(), a.toRvalueOrPE(), readOperator(), parseShiftExpression().toRvalueOrPE());
/*      */       }
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/* 1794 */     return a;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public Java.Atom parseShiftExpression()
/*      */     throws CompileException, IOException
/*      */   {
/* 1806 */     Java.Atom a = parseAdditiveExpression();
/*      */     
/* 1808 */     while (peekOperator(new String[] { "<<", ">>", ">>>" })) {
/* 1809 */       a = new Java.BinaryOperation(location(), a.toRvalueOrPE(), readOperator(), parseAdditiveExpression().toRvalueOrPE());
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 1816 */     return a;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public Java.Atom parseAdditiveExpression()
/*      */     throws CompileException, IOException
/*      */   {
/* 1826 */     Java.Atom a = parseMultiplicativeExpression();
/*      */     
/* 1828 */     while (peekOperator(new String[] { "+", "-" })) {
/* 1829 */       a = new Java.BinaryOperation(location(), a.toRvalueOrPE(), readOperator(), parseMultiplicativeExpression().toRvalueOrPE());
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 1836 */     return a;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public Java.Atom parseMultiplicativeExpression()
/*      */     throws CompileException, IOException
/*      */   {
/* 1846 */     Java.Atom a = parseUnaryExpression();
/*      */     
/* 1848 */     while (peekOperator(new String[] { "*", "/", "%" })) {
/* 1849 */       a = new Java.BinaryOperation(location(), a.toRvalueOrPE(), readOperator(), parseUnaryExpression().toRvalueOrPE());
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 1856 */     return a;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public Java.Atom parseUnaryExpression()
/*      */     throws CompileException, IOException
/*      */   {
/* 1870 */     if (peekOperator(new String[] { "++", "--" })) {
/* 1871 */       return new Java.Crement(location(), readOperator(), parseUnaryExpression().toLvalueOrPE());
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 1878 */     if (peekOperator(new String[] { "+", "-", "~", "!" })) {
/* 1879 */       return new Java.UnaryOperation(location(), readOperator(), parseUnaryExpression().toRvalueOrPE());
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 1886 */     Java.Atom a = parsePrimary();
/*      */     
/* 1888 */     while (peekOperator(new String[] { ".", "[" })) {
/* 1889 */       a = parseSelector(a);
/*      */     }
/*      */     
/* 1892 */     while (peekOperator(new String[] { "++", "--" })) {
/* 1893 */       a = new Java.Crement(location(), a.toLvalueOrPE(), readOperator());
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 1900 */     return a;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public Java.Atom parsePrimary()
/*      */     throws CompileException, IOException
/*      */   {
/* 1940 */     if (peekOperator("(")) {
/* 1941 */       eatToken();
/* 1942 */       if (peekKeyword(new String[] { "boolean", "char", "byte", "short", "int", "long", "float", "double" }))
/*      */       {
/*      */ 
/*      */ 
/* 1946 */         Java.Type type = parseType();
/* 1947 */         int brackets = parseBracketsOpt();
/* 1948 */         readOperator(")");
/* 1949 */         for (int i = 0; i < brackets; i++) type = new Java.ArrayType(type);
/* 1950 */         return new Java.Cast(location(), type, parseUnaryExpression().toRvalueOrPE());
/*      */       }
/*      */       
/*      */ 
/*      */ 
/*      */ 
/* 1956 */       Java.Atom a = parseExpression();
/* 1957 */       readOperator(")");
/*      */       
/* 1959 */       if ((!this.scanner.peek().isLiteral()) && (!this.scanner.peek().isIdentifier())) { if (!peekOperator(new String[] { "(", "~", "!" })) { if (!peekKeyword(new String[] { "this", "super", "new" })) {
/*      */             break label239;
/*      */           }
/*      */         }
/*      */       }
/*      */       
/*      */ 
/* 1966 */       return new Java.Cast(location(), a.toTypeOrPE(), parseUnaryExpression().toRvalueOrPE());
/*      */       
/*      */ 
/*      */ 
/*      */       label239:
/*      */       
/*      */ 
/*      */ 
/* 1974 */       return new Java.ParenthesizedExpression(a.getLocation(), a.toRvalueOrPE());
/*      */     }
/*      */     
/* 1977 */     if (this.scanner.peek().isLiteral())
/*      */     {
/* 1979 */       return parseLiteral();
/*      */     }
/*      */     
/* 1982 */     if (this.scanner.peek().isIdentifier()) {
/* 1983 */       Location location = location();
/* 1984 */       String[] qi = parseQualifiedIdentifier();
/* 1985 */       if (peekOperator("("))
/*      */       {
/* 1987 */         return new Java.MethodInvocation(location(), qi.length == 1 ? null : new Java.AmbiguousName(location, qi, qi.length - 1), qi[(qi.length - 1)], parseArguments());
/*      */       }
/*      */       
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 1998 */       if ((peekOperator("[")) && (this.scanner.peekNextButOne().isOperator("]")))
/*      */       {
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 2004 */         Java.Type res = new Java.ReferenceType(location, qi);
/*      */         
/*      */ 
/*      */ 
/* 2008 */         int brackets = parseBracketsOpt();
/* 2009 */         for (int i = 0; i < brackets; i++) res = new Java.ArrayType(res);
/* 2010 */         if ((peekOperator(".")) && (this.scanner.peekNextButOne().isKeyword("class")))
/*      */         {
/*      */ 
/*      */ 
/* 2014 */           eatToken();
/* 2015 */           Location location2 = location();
/* 2016 */           eatToken();
/* 2017 */           return new Java.ClassLiteral(location2, res);
/*      */         }
/* 2019 */         return res;
/*      */       }
/*      */       
/*      */ 
/* 2023 */       return new Java.AmbiguousName(location(), qi);
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/* 2029 */     if (peekKeyword("this")) {
/* 2030 */       Location location = location();
/* 2031 */       eatToken();
/* 2032 */       if (peekOperator("("))
/*      */       {
/*      */ 
/*      */ 
/* 2036 */         return new Java.AlternateConstructorInvocation(location, parseArguments());
/*      */       }
/*      */       
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 2044 */       return new Java.ThisReference(location);
/*      */     }
/*      */     
/*      */ 
/* 2048 */     if (peekKeyword("super")) {
/* 2049 */       eatToken();
/* 2050 */       if (peekOperator("("))
/*      */       {
/*      */ 
/*      */ 
/* 2054 */         return new Java.SuperConstructorInvocation(location(), (Java.Rvalue)null, parseArguments());
/*      */       }
/*      */       
/*      */ 
/*      */ 
/*      */ 
/* 2060 */       readOperator(".");
/* 2061 */       String name = readIdentifier();
/* 2062 */       if (peekOperator("("))
/*      */       {
/*      */ 
/* 2065 */         return new Java.SuperclassMethodInvocation(location(), name, parseArguments());
/*      */       }
/*      */       
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 2073 */       return new Java.SuperclassFieldAccessExpression(location(), (Java.Type)null, name);
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 2082 */     if (peekKeyword("new")) {
/* 2083 */       Location location = location();
/* 2084 */       eatToken();
/* 2085 */       Java.Type type = parseType();
/* 2086 */       if ((type instanceof Java.ArrayType))
/*      */       {
/* 2088 */         return new Java.NewInitializedArray(location, (Java.ArrayType)type, parseArrayInitializer());
/*      */       }
/* 2090 */       if (((type instanceof Java.ReferenceType)) && (peekOperator("(")))
/*      */       {
/*      */ 
/*      */ 
/*      */ 
/* 2095 */         Java.Rvalue[] arguments = parseArguments();
/* 2096 */         if (peekOperator("{"))
/*      */         {
/* 2098 */           Java.AnonymousClassDeclaration anonymousClassDeclaration = new Java.AnonymousClassDeclaration(location(), type);
/*      */           
/*      */ 
/*      */ 
/* 2102 */           parseClassBody(anonymousClassDeclaration);
/* 2103 */           return new Java.NewAnonymousClassInstance(location, (Java.Rvalue)null, anonymousClassDeclaration, arguments);
/*      */         }
/*      */         
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 2111 */         return new Java.NewClassInstance(location, (Java.Rvalue)null, type, arguments);
/*      */       }
/*      */       
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 2120 */       return new Java.NewArray(location, type, parseDimExprs(), parseBracketsOpt());
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 2129 */     if (peekKeyword(new String[] { "boolean", "char", "byte", "short", "int", "long", "float", "double" })) {
/* 2130 */       Java.Type res = parseType();
/* 2131 */       int brackets = parseBracketsOpt();
/* 2132 */       for (int i = 0; i < brackets; i++) res = new Java.ArrayType(res);
/* 2133 */       if ((peekOperator(".")) && (this.scanner.peekNextButOne().isKeyword("class")))
/*      */       {
/*      */ 
/*      */ 
/*      */ 
/* 2138 */         eatToken();
/* 2139 */         Location location = location();
/* 2140 */         eatToken();
/* 2141 */         return new Java.ClassLiteral(location, res);
/*      */       }
/*      */       
/* 2144 */       return res;
/*      */     }
/*      */     
/*      */ 
/* 2148 */     if (peekKeyword("void")) {
/* 2149 */       eatToken();
/* 2150 */       if ((peekOperator(".")) && (this.scanner.peekNextButOne().isKeyword("class")))
/*      */       {
/*      */ 
/*      */ 
/*      */ 
/* 2155 */         eatToken();
/* 2156 */         Location location = location();
/* 2157 */         eatToken();
/* 2158 */         return new Java.ClassLiteral(location, new Java.BasicType(location, 0));
/*      */       }
/* 2160 */       throwCompileException("\"void\" encountered in wrong context");
/*      */     }
/*      */     
/* 2163 */     throwCompileException("Unexpected token \"" + this.scanner.peek() + "\" in primary");
/* 2164 */     return null;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public Java.Atom parseSelector(Java.Atom atom)
/*      */     throws CompileException, IOException
/*      */   {
/* 2182 */     if (peekOperator(".")) {
/* 2183 */       eatToken();
/* 2184 */       if (this.scanner.peek().isIdentifier()) {
/* 2185 */         String identifier = readIdentifier();
/* 2186 */         if (peekOperator("("))
/*      */         {
/* 2188 */           return new Java.MethodInvocation(location(), atom.toRvalueOrPE(), identifier, parseArguments());
/*      */         }
/*      */         
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 2196 */         return new Java.FieldAccessExpression(location(), atom.toRvalueOrPE(), identifier);
/*      */       }
/*      */       
/*      */ 
/*      */ 
/*      */ 
/* 2202 */       if (peekKeyword("this"))
/*      */       {
/* 2204 */         Location location = location();
/* 2205 */         eatToken();
/* 2206 */         return new Java.QualifiedThisReference(location, atom.toTypeOrPE());
/*      */       }
/*      */       
/*      */ 
/*      */ 
/* 2211 */       if (peekKeyword("super")) {
/* 2212 */         Location location = location();
/* 2213 */         eatToken();
/* 2214 */         if (peekOperator("("))
/*      */         {
/*      */ 
/*      */ 
/* 2218 */           return new Java.SuperConstructorInvocation(location, atom.toRvalueOrPE(), parseArguments());
/*      */         }
/*      */         
/*      */ 
/*      */ 
/*      */ 
/* 2224 */         readOperator(".");
/* 2225 */         String identifier = readIdentifier();
/*      */         
/* 2227 */         if (peekOperator("("))
/*      */         {
/*      */ 
/*      */ 
/*      */ 
/* 2232 */           throwCompileException("Qualified superclass method invocation NYI");
/*      */ 
/*      */         }
/*      */         else
/*      */         {
/* 2237 */           return new Java.SuperclassFieldAccessExpression(location, atom.toTypeOrPE(), identifier);
/*      */         }
/*      */       }
/*      */       
/*      */ 
/*      */ 
/*      */ 
/* 2244 */       if (peekKeyword("new"))
/*      */       {
/* 2246 */         Java.Rvalue lhs = atom.toRvalue();
/* 2247 */         Location location = location();
/* 2248 */         eatToken();
/* 2249 */         String identifier = readIdentifier();
/* 2250 */         Java.Type type = new Java.RvalueMemberType(location, lhs, identifier);
/*      */         
/*      */ 
/*      */ 
/*      */ 
/* 2255 */         Java.Rvalue[] arguments = parseArguments();
/* 2256 */         if (peekOperator("{"))
/*      */         {
/* 2258 */           Java.AnonymousClassDeclaration anonymousClassDeclaration = new Java.AnonymousClassDeclaration(location(), type);
/*      */           
/*      */ 
/*      */ 
/* 2262 */           parseClassBody(anonymousClassDeclaration);
/* 2263 */           return new Java.NewAnonymousClassInstance(location, lhs, anonymousClassDeclaration, arguments);
/*      */         }
/*      */         
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 2271 */         return new Java.NewClassInstance(location, lhs, type, arguments);
/*      */       }
/*      */       
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 2279 */       if (peekKeyword("class"))
/*      */       {
/* 2281 */         Location location = location();
/* 2282 */         eatToken();
/* 2283 */         return new Java.ClassLiteral(location, atom.toTypeOrPE());
/*      */       }
/* 2285 */       throwCompileException("Unexpected selector \"" + this.scanner.peek() + "\" after \".\"");
/*      */     }
/* 2287 */     if (peekOperator("["))
/*      */     {
/* 2289 */       Location location = location();
/* 2290 */       eatToken();
/* 2291 */       Java.Rvalue index = parseExpression().toRvalueOrPE();
/* 2292 */       readOperator("]");
/* 2293 */       return new Java.ArrayAccessExpression(location, atom.toRvalueOrPE(), index);
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/* 2299 */     throwCompileException("Unexpected token \"" + this.scanner.peek() + "\" in selector");
/* 2300 */     return null;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   public Java.Rvalue[] parseDimExprs()
/*      */     throws CompileException, IOException
/*      */   {
/* 2309 */     List l = new ArrayList();
/* 2310 */     l.add(parseDimExpr());
/*      */     
/*      */ 
/* 2313 */     while ((peekOperator("[")) && (!this.scanner.peekNextButOne().isOperator("]")))
/* 2314 */       l.add(parseDimExpr());
/* 2315 */     return (Java.Rvalue[])l.toArray(new Java.Rvalue[l.size()]);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   public Java.Rvalue parseDimExpr()
/*      */     throws CompileException, IOException
/*      */   {
/* 2324 */     readOperator("[");
/* 2325 */     Java.Rvalue res = parseExpression().toRvalueOrPE();
/* 2326 */     readOperator("]");
/* 2327 */     return res;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   public Java.Rvalue[] parseArguments()
/*      */     throws CompileException, IOException
/*      */   {
/* 2336 */     readOperator("(");
/* 2337 */     if (peekOperator(")")) {
/* 2338 */       eatToken();
/* 2339 */       return new Java.Rvalue[0];
/*      */     }
/* 2341 */     Java.Rvalue[] arguments = parseArgumentList();
/* 2342 */     readOperator(")");
/* 2343 */     return arguments;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   public Java.Rvalue[] parseArgumentList()
/*      */     throws CompileException, IOException
/*      */   {
/* 2352 */     List l = new ArrayList();
/*      */     for (;;) {
/* 2354 */       l.add(parseExpression().toRvalueOrPE());
/* 2355 */       if (!peekOperator(",")) break;
/* 2356 */       eatToken();
/*      */     }
/* 2358 */     return (Java.Rvalue[])l.toArray(new Java.Rvalue[l.size()]);
/*      */   }
/*      */   
/*      */   public Java.Atom parseLiteral() throws CompileException, IOException {
/* 2362 */     Scanner.Token t = this.scanner.read();
/* 2363 */     if (!t.isLiteral()) throwCompileException("Literal expected");
/* 2364 */     return new Java.Literal(t.getLocation(), t.getLiteralValue());
/*      */   }
/*      */   
/*      */ 
/*      */ 
/* 2369 */   public Location location() { return this.scanner.location(); }
/* 2370 */   public void eatToken() throws CompileException, IOException { this.scanner.read(); }
/*      */   
/* 2372 */   public boolean peekKeyword() { return this.scanner.peek().isKeyword(); }
/* 2373 */   public boolean peekKeyword(String keyword) { return this.scanner.peek().isKeyword(keyword); }
/* 2374 */   public boolean peekKeyword(String[] keywords) { return this.scanner.peek().isKeyword(keywords); }
/*      */   
/* 2376 */   public void readKeyword(String keyword) throws CompileException, IOException { if (!this.scanner.read().isKeyword(keyword)) throwCompileException("\"" + keyword + "\" expected");
/*      */   }
/*      */   
/* 2379 */   public boolean peekOperator(String operator) { return this.scanner.peek().isOperator(operator); }
/* 2380 */   public boolean peekOperator(String[] operators) { return this.scanner.peek().isOperator(operators); }
/*      */   
/* 2382 */   public String readOperator() throws CompileException, IOException { Scanner.Token t = this.scanner.read();
/* 2383 */     if (!t.isOperator()) throwCompileException("Operator expected");
/* 2384 */     return t.getOperator();
/*      */   }
/*      */   
/* 2387 */   public void readOperator(String operator) throws CompileException, IOException { if (!this.scanner.read().isOperator(operator)) {
/* 2388 */       throwCompileException("Operator \"" + operator + "\" expected");
/*      */     }
/*      */   }
/*      */   
/* 2392 */   public boolean peekIdentifier() { return this.scanner.peek().isIdentifier(); }
/*      */   
/* 2394 */   public String readIdentifier() throws CompileException, IOException { Scanner.Token t = this.scanner.read();
/* 2395 */     if (!t.isIdentifier()) throwCompileException("Identifier expected");
/* 2396 */     return t.getIdentifier();
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   public Java.Statement parseExpressionStatement()
/*      */     throws CompileException, IOException
/*      */   {
/* 2405 */     Java.Rvalue rv = parseExpression().toRvalueOrPE();
/* 2406 */     readOperator(";");
/*      */     
/* 2408 */     return new Java.ExpressionStatement(rv);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   private void verifyStringIsConventionalPackageName(String s, Location loc)
/*      */   {
/* 2416 */     if (!Character.isLowerCase(s.charAt(0))) {
/* 2417 */       warning("UPN", "Package name \"" + s + "\" does not begin with a lower-case letter (see JLS2 6.8.1)", loc);
/*      */       
/*      */ 
/*      */ 
/*      */ 
/* 2422 */       return;
/*      */     }
/*      */     
/* 2425 */     for (int i = 0; i < s.length(); i++) {
/* 2426 */       char c = s.charAt(i);
/* 2427 */       if ((!Character.isLowerCase(c)) && (c != '_') && (c != '.')) {
/* 2428 */         warning("PPN", "Poorly chosen package name \"" + s + "\" contains bad character '" + c + "'", loc);
/* 2429 */         return;
/*      */       }
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   private void verifyIdentifierIsConventionalClassOrInterfaceName(String id, Location loc)
/*      */   {
/* 2439 */     if (!Character.isUpperCase(id.charAt(0))) {
/* 2440 */       warning("UCOIN1", "Class or interface name \"" + id + "\" does not begin with an upper-case letter (see JLS2 6.8.2)", loc);
/*      */       
/*      */ 
/*      */ 
/*      */ 
/* 2445 */       return;
/*      */     }
/* 2447 */     for (int i = 0; i < id.length(); i++) {
/* 2448 */       char c = id.charAt(i);
/* 2449 */       if ((!Character.isLetter(c)) && (!Character.isDigit(c))) {
/* 2450 */         warning("UCOIN", "Class or interface name \"" + id + "\" contains unconventional character \"" + c + "\" (see JLS2 6.8.2)", loc);
/*      */         
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 2457 */         return;
/*      */       }
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   private void verifyIdentifierIsConventionalMethodName(String id, Location loc)
/*      */   {
/* 2467 */     if (!Character.isLowerCase(id.charAt(0))) {
/* 2468 */       warning("UMN1", "Method name \"" + id + "\" does not begin with a lower-case letter (see JLS2 6.8.3)", loc);
/*      */       
/*      */ 
/*      */ 
/*      */ 
/* 2473 */       return;
/*      */     }
/* 2475 */     for (int i = 0; i < id.length(); i++) {
/* 2476 */       char c = id.charAt(i);
/* 2477 */       if ((!Character.isLetter(c)) && (!Character.isDigit(c))) {
/* 2478 */         warning("UMN", "Method name \"" + id + "\" contains unconventional character \"" + c + "\" (see JLS2 6.8.3)", loc);
/*      */         
/*      */ 
/*      */ 
/*      */ 
/* 2483 */         return;
/*      */       }
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private void verifyIdentifierIsConventionalFieldName(String id, Location loc)
/*      */   {
/* 2497 */     if (Character.isUpperCase(id.charAt(0))) {
/* 2498 */       for (int i = 0; i < id.length(); i++) {
/* 2499 */         char c = id.charAt(i);
/* 2500 */         if ((!Character.isUpperCase(c)) && (!Character.isDigit(c)) && (c != '_')) {
/* 2501 */           warning("UCN", "Constant name \"" + id + "\" contains unconventional character \"" + c + "\" (see JLS2 6.8.5)", loc);
/*      */           
/*      */ 
/*      */ 
/*      */ 
/* 2506 */           return;
/*      */         }
/*      */         
/*      */       }
/* 2510 */     } else if (Character.isLowerCase(id.charAt(0))) {
/* 2511 */       for (int i = 0; i < id.length(); i++) {
/* 2512 */         char c = id.charAt(i);
/* 2513 */         if ((!Character.isLetter(c)) && (!Character.isDigit(c))) {
/* 2514 */           warning("UFN", "Field name \"" + id + "\" contains unconventional character \"" + c + "\" (see JLS2 6.8.4)", loc);
/*      */           
/*      */ 
/*      */ 
/*      */ 
/* 2519 */           return;
/*      */         }
/*      */       }
/*      */     } else {
/* 2523 */       warning("UFN1", "\"" + id + "\" is neither a conventional field name (JLS2 6.8.4) nor a conventional constant name (JLS2 6.8.5)", loc);
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private void verifyIdentifierIsConventionalLocalVariableOrParameterName(String id, Location loc)
/*      */   {
/* 2536 */     if (!Character.isLowerCase(id.charAt(0))) {
/* 2537 */       warning("ULVN1", "Local variable name \"" + id + "\" does not begin with a lower-case letter (see JLS2 6.8.6)", loc);
/*      */       
/*      */ 
/*      */ 
/*      */ 
/* 2542 */       return;
/*      */     }
/* 2544 */     for (int i = 0; i < id.length(); i++) {
/* 2545 */       char c = id.charAt(i);
/* 2546 */       if ((!Character.isLetter(c)) && (!Character.isDigit(c))) {
/* 2547 */         warning("ULVN", "Local variable name \"" + id + "\" contains unconventional character \"" + c + "\" (see JLS2 6.8.6)", loc);
/*      */         
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 2554 */         return;
/*      */       }
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public void setWarningHandler(WarningHandler optionalWarningHandler)
/*      */   {
/* 2569 */     this.optionalWarningHandler = optionalWarningHandler;
/*      */   }
/*      */   
/*      */ 
/* 2573 */   private WarningHandler optionalWarningHandler = null;
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private void warning(String handle, String message, Location optionalLocation)
/*      */   {
/* 2584 */     if (this.optionalWarningHandler != null) {
/* 2585 */       this.optionalWarningHandler.handleWarning(handle, message, optionalLocation);
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */   protected final void throwCompileException(String message)
/*      */     throws CompileException
/*      */   {
/* 2593 */     throw new CompileException(message, location());
/*      */   }
/*      */   
/*      */   private static String join(String[] sa, String separator) {
/* 2597 */     if (sa == null) return "(null)";
/* 2598 */     if (sa.length == 0) return "(zero length array)";
/* 2599 */     StringBuffer sb = new StringBuffer(sa[0]);
/* 2600 */     for (int i = 1; i < sa.length; i++) {
/* 2601 */       sb.append(separator).append(sa[i]);
/*      */     }
/* 2603 */     return sb.toString();
/*      */   }
/*      */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\org\codehaus\janino\Parser.class
 * Java compiler version: 1 (45.3)
 * JD-Core Version:       0.7.1
 */