package prefuse;

import java.awt.geom.Rectangle2D;
import java.awt.geom.Rectangle2D.Double;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;
import prefuse.action.Action;
import prefuse.activity.Activity;
import prefuse.activity.ActivityMap;
import prefuse.data.Graph;
import prefuse.data.Schema;
import prefuse.data.Table;
import prefuse.data.Tree;
import prefuse.data.Tuple;
import prefuse.data.expression.Expression;
import prefuse.data.expression.Predicate;
import prefuse.data.expression.parser.ExpressionParser;
import prefuse.data.tuple.CompositeTupleSet;
import prefuse.data.tuple.DefaultTupleSet;
import prefuse.data.tuple.TupleManager;
import prefuse.data.tuple.TupleSet;
import prefuse.render.DefaultRendererFactory;
import prefuse.render.Renderer;
import prefuse.render.RendererFactory;
import prefuse.util.PrefuseConfig;
import prefuse.util.PrefuseLib;
import prefuse.util.collections.CompositeIterator;
import prefuse.visual.AggregateTable;
import prefuse.visual.VisualGraph;
import prefuse.visual.VisualItem;
import prefuse.visual.VisualTable;
import prefuse.visual.VisualTree;
import prefuse.visual.VisualTupleSet;
import prefuse.visual.expression.ValidatedPredicate;
import prefuse.visual.expression.VisiblePredicate;
import prefuse.visual.tuple.TableDecoratorItem;
import prefuse.visual.tuple.TableEdgeItem;
import prefuse.visual.tuple.TableNodeItem;

public class Visualization
{
  public static final String ALL_ITEMS = PrefuseConfig.get("visualization.allItems");
  public static final String FOCUS_ITEMS = PrefuseConfig.get("visualization.focusItems");
  public static final String SELECTED_ITEMS = PrefuseConfig.get("visualization.selectedItems");
  public static final String SEARCH_ITEMS = PrefuseConfig.get("visualization.searchItems");
  private Map m_visual = new HashMap();
  private Map m_source = new HashMap();
  private Map m_focus = new HashMap();
  private ActivityMap m_actions = new ActivityMap();
  private RendererFactory m_renderers = new DefaultRendererFactory();
  private ArrayList m_displays = new ArrayList();
  
  public Visualization()
  {
    addFocusGroup(FOCUS_ITEMS, new DefaultTupleSet());
    addFocusGroup(SELECTED_ITEMS, new DefaultTupleSet());
  }
  
  public synchronized VisualTupleSet add(String paramString, TupleSet paramTupleSet)
  {
    return add(paramString, paramTupleSet, null);
  }
  
  public synchronized VisualTupleSet add(String paramString, TupleSet paramTupleSet, Predicate paramPredicate)
  {
    if ((paramTupleSet instanceof Table)) {
      return addTable(paramString, (Table)paramTupleSet, paramPredicate);
    }
    if ((paramTupleSet instanceof Tree)) {
      return addTree(paramString, (Tree)paramTupleSet, paramPredicate);
    }
    if ((paramTupleSet instanceof Graph)) {
      return addGraph(paramString, (Graph)paramTupleSet, paramPredicate);
    }
    throw new IllegalArgumentException("Unsupported TupleSet type.");
  }
  
  public synchronized VisualTable addTable(String paramString)
  {
    VisualTable localVisualTable = new VisualTable(this, paramString);
    this.m_visual.put(paramString, localVisualTable);
    return localVisualTable;
  }
  
  public synchronized VisualTable addTable(String paramString, Schema paramSchema)
  {
    VisualTable localVisualTable = new VisualTable(this, paramString, paramSchema);
    this.m_visual.put(paramString, localVisualTable);
    return localVisualTable;
  }
  
  public synchronized VisualTable addTable(String paramString, Table paramTable)
  {
    return addTable(paramString, paramTable, (Predicate)null);
  }
  
  public synchronized VisualTable addTable(String paramString, Table paramTable, Predicate paramPredicate)
  {
    VisualTable localVisualTable = new VisualTable(paramTable, this, paramString, paramPredicate);
    this.m_visual.put(paramString, localVisualTable);
    this.m_source.put(paramString, paramTable);
    return localVisualTable;
  }
  
  public synchronized VisualTable addTable(String paramString, Table paramTable, Schema paramSchema)
  {
    return addTable(paramString, paramTable, null, paramSchema);
  }
  
  public synchronized VisualTable addTable(String paramString, Table paramTable, Predicate paramPredicate, Schema paramSchema)
  {
    VisualTable localVisualTable = new VisualTable(paramTable, this, paramString, paramPredicate, paramSchema);
    this.m_visual.put(paramString, localVisualTable);
    this.m_source.put(paramString, paramTable);
    return localVisualTable;
  }
  
  public synchronized VisualGraph addGraph(String paramString, Graph paramGraph)
  {
    return addGraph(paramString, paramGraph, null);
  }
  
  public synchronized VisualGraph addGraph(String paramString, Graph paramGraph, Predicate paramPredicate)
  {
    return addGraph(paramString, paramGraph, paramPredicate, VisualItem.SCHEMA, VisualItem.SCHEMA);
  }
  
  public synchronized VisualGraph addGraph(String paramString, Graph paramGraph, Predicate paramPredicate, Schema paramSchema1, Schema paramSchema2)
  {
    String str1 = PrefuseLib.getGroupName(paramString, Graph.NODES);
    String str2 = PrefuseLib.getGroupName(paramString, Graph.EDGES);
    VisualTable localVisualTable1 = addTable(str1, paramGraph.getNodeTable(), paramPredicate, paramSchema1);
    VisualTable localVisualTable2 = addTable(str2, paramGraph.getEdgeTable(), paramPredicate, paramSchema2);
    VisualGraph localVisualGraph = new VisualGraph(localVisualTable1, localVisualTable2, paramGraph.isDirected(), paramGraph.getNodeKeyField(), paramGraph.getEdgeSourceField(), paramGraph.getEdgeTargetField());
    localVisualGraph.setVisualization(this);
    localVisualGraph.setGroup(paramString);
    this.m_visual.put(paramString, localVisualGraph);
    this.m_source.put(paramString, paramGraph);
    TupleManager localTupleManager1 = new TupleManager(localVisualTable1, localVisualGraph, TableNodeItem.class);
    TupleManager localTupleManager2 = new TupleManager(localVisualTable2, localVisualGraph, TableEdgeItem.class);
    localVisualTable1.setTupleManager(localTupleManager1);
    localVisualTable2.setTupleManager(localTupleManager2);
    localVisualGraph.setTupleManagers(localTupleManager1, localTupleManager2);
    return localVisualGraph;
  }
  
  public synchronized VisualTree addTree(String paramString, Tree paramTree)
  {
    return addTree(paramString, paramTree, null);
  }
  
  public synchronized VisualTree addTree(String paramString, Tree paramTree, Predicate paramPredicate)
  {
    return addTree(paramString, paramTree, paramPredicate, VisualItem.SCHEMA, VisualItem.SCHEMA);
  }
  
  public synchronized VisualTree addTree(String paramString, Tree paramTree, Predicate paramPredicate, Schema paramSchema1, Schema paramSchema2)
  {
    String str1 = PrefuseLib.getGroupName(paramString, Graph.NODES);
    String str2 = PrefuseLib.getGroupName(paramString, Graph.EDGES);
    VisualTable localVisualTable1 = addTable(str1, paramTree.getNodeTable(), paramPredicate, paramSchema1);
    VisualTable localVisualTable2 = addTable(str2, paramTree.getEdgeTable(), paramPredicate, paramSchema2);
    VisualTree localVisualTree = new VisualTree(localVisualTable1, localVisualTable2, paramTree.getNodeKeyField(), paramTree.getEdgeSourceField(), paramTree.getEdgeTargetField());
    localVisualTree.setVisualization(this);
    localVisualTree.setGroup(paramString);
    this.m_visual.put(paramString, localVisualTree);
    this.m_source.put(paramString, paramTree);
    TupleManager localTupleManager1 = new TupleManager(localVisualTable1, localVisualTree, TableNodeItem.class);
    TupleManager localTupleManager2 = new TupleManager(localVisualTable2, localVisualTree, TableEdgeItem.class);
    localVisualTable1.setTupleManager(localTupleManager1);
    localVisualTable2.setTupleManager(localTupleManager2);
    localVisualTree.setTupleManagers(localTupleManager1, localTupleManager2);
    return localVisualTree;
  }
  
  public synchronized AggregateTable addAggregates(String paramString)
  {
    return addAggregates(paramString, VisualItem.SCHEMA);
  }
  
  public synchronized AggregateTable addAggregates(String paramString, Schema paramSchema)
  {
    AggregateTable localAggregateTable = new AggregateTable(this, paramString, paramSchema);
    this.m_visual.put(paramString, localAggregateTable);
    return localAggregateTable;
  }
  
  public synchronized VisualTable addDerivedTable(String paramString1, String paramString2, Predicate paramPredicate, Schema paramSchema)
  {
    VisualTable localVisualTable1 = (VisualTable)getGroup(paramString2);
    VisualTable localVisualTable2 = new VisualTable(localVisualTable1, this, paramString1, paramPredicate, paramSchema);
    this.m_visual.put(paramString1, localVisualTable2);
    return localVisualTable2;
  }
  
  public synchronized VisualTable addDecorators(String paramString1, String paramString2)
  {
    return addDecorators(paramString1, paramString2, (Predicate)null);
  }
  
  public synchronized VisualTable addDecorators(String paramString1, String paramString2, Schema paramSchema)
  {
    return addDecorators(paramString1, paramString2, null, paramSchema);
  }
  
  public synchronized VisualTable addDecorators(String paramString1, String paramString2, Predicate paramPredicate)
  {
    VisualTable localVisualTable = addDerivedTable(paramString1, paramString2, paramPredicate, VisualItem.SCHEMA);
    localVisualTable.setTupleManager(new TupleManager(localVisualTable, null, TableDecoratorItem.class));
    return localVisualTable;
  }
  
  public synchronized VisualTable addDecorators(String paramString1, String paramString2, Predicate paramPredicate, Schema paramSchema)
  {
    VisualTable localVisualTable = addDerivedTable(paramString1, paramString2, paramPredicate, paramSchema);
    localVisualTable.setTupleManager(new TupleManager(localVisualTable, null, TableDecoratorItem.class));
    return localVisualTable;
  }
  
  public synchronized boolean removeGroup(String paramString)
  {
    TupleSet localTupleSet = getFocusGroup(paramString);
    if (localTupleSet != null)
    {
      localObject1 = localTupleSet.tuples(ValidatedPredicate.TRUE);
      while (((Iterator)localObject1).hasNext()) {
        ((VisualItem)((Iterator)localObject1).next()).setValidated(false);
      }
      localTupleSet.clear();
      this.m_focus.remove(paramString);
      return true;
    }
    localTupleSet = getVisualGroup(paramString);
    if (localTupleSet == null) {
      return false;
    }
    Object localObject1 = new TupleSet[this.m_focus.size()];
    this.m_focus.values().toArray((Object[])localObject1);
    Object localObject2 = localTupleSet.tuples();
    Object localObject3;
    while (((Iterator)localObject2).hasNext())
    {
      localObject3 = (VisualItem)((Iterator)localObject2).next();
      for (int i = 0; i < localObject1.length; i++) {
        localObject1[i].removeTuple((Tuple)localObject3);
      }
      ((VisualItem)localObject3).setValidated(false);
    }
    if ((localTupleSet instanceof CompositeTupleSet))
    {
      localObject2 = (CompositeTupleSet)localTupleSet;
      localObject3 = ((CompositeTupleSet)localObject2).setNames();
      while (((Iterator)localObject3).hasNext())
      {
        String str = (String)((Iterator)localObject3).next();
        this.m_visual.remove(PrefuseLib.getGroupName(paramString, str));
      }
    }
    this.m_visual.remove(paramString);
    return true;
  }
  
  public synchronized void reset()
  {
    Iterator localIterator = this.m_focus.entrySet().iterator();
    Map.Entry localEntry;
    TupleSet localTupleSet;
    while (localIterator.hasNext())
    {
      localEntry = (Map.Entry)localIterator.next();
      localTupleSet = (TupleSet)localEntry.getValue();
      localTupleSet.clear();
    }
    localIterator = this.m_visual.entrySet().iterator();
    while (localIterator.hasNext())
    {
      localEntry = (Map.Entry)localIterator.next();
      localTupleSet = (TupleSet)localEntry.getValue();
      localTupleSet.clear();
    }
    this.m_visual.clear();
    this.m_source.clear();
  }
  
  public TupleSet getSourceData(String paramString)
  {
    return (TupleSet)this.m_source.get(paramString);
  }
  
  public Tuple getSourceTuple(VisualItem paramVisualItem)
  {
    String str = paramVisualItem.getGroup();
    Table localTable = (Table)getSourceData(str);
    if (localTable == null) {
      return null;
    }
    VisualTable localVisualTable = (VisualTable)paramVisualItem.getTable();
    return localTable.getTuple(localVisualTable.getParentRow(paramVisualItem.getRow()));
  }
  
  public TupleSet getGroup(String paramString)
  {
    TupleSet localTupleSet = getVisualGroup(paramString);
    if (localTupleSet == null) {
      localTupleSet = getFocusGroup(paramString);
    }
    return localTupleSet;
  }
  
  public boolean isInGroup(VisualItem paramVisualItem, String paramString)
  {
    if (ALL_ITEMS.equals(paramString)) {
      return true;
    }
    if (paramVisualItem.getGroup() == paramString) {
      return true;
    }
    TupleSet localTupleSet = getGroup(paramString);
    return localTupleSet == null ? false : localTupleSet.containsTuple(paramVisualItem);
  }
  
  public void addFocusGroup(String paramString)
  {
    this.m_focus.put(paramString, new DefaultTupleSet());
  }
  
  public void addFocusGroup(String paramString, TupleSet paramTupleSet)
  {
    this.m_focus.put(paramString, paramTupleSet);
  }
  
  public int size(String paramString)
  {
    TupleSet localTupleSet = getGroup(paramString);
    return localTupleSet == null ? -1 : localTupleSet.getTupleCount();
  }
  
  public TupleSet getVisualGroup(String paramString)
  {
    return (TupleSet)this.m_visual.get(paramString);
  }
  
  public TupleSet getFocusGroup(String paramString)
  {
    return (TupleSet)this.m_focus.get(paramString);
  }
  
  public void invalidate(String paramString)
  {
    Iterator localIterator = items(ValidatedPredicate.TRUE);
    while (localIterator.hasNext())
    {
      VisualItem localVisualItem = (VisualItem)localIterator.next();
      localVisualItem.setValidated(false);
    }
  }
  
  public void invalidateAll()
  {
    invalidate(ALL_ITEMS);
  }
  
  public Iterator visibleItems()
  {
    return items(VisiblePredicate.TRUE);
  }
  
  public Iterator visibleItems(String paramString)
  {
    return items(paramString, VisiblePredicate.TRUE);
  }
  
  public Iterator items()
  {
    return items((Predicate)null);
  }
  
  public Iterator items(Predicate paramPredicate)
  {
    int i = this.m_visual.size();
    if (i == 0) {
      return Collections.EMPTY_LIST.iterator();
    }
    if (i == 1)
    {
      localObject = this.m_visual.keySet().iterator();
      return items((String)((Iterator)localObject).next(), paramPredicate);
    }
    Object localObject = new CompositeIterator(this.m_visual.size());
    Iterator localIterator = this.m_visual.keySet().iterator();
    int j = 0;
    while (localIterator.hasNext())
    {
      String str = (String)localIterator.next();
      if (!PrefuseLib.isChildGroup(str)) {
        ((CompositeIterator)localObject).setIterator(j++, items(str, paramPredicate));
      }
    }
    return (Iterator)localObject;
  }
  
  public Iterator items(String paramString)
  {
    return items(paramString, (Predicate)null);
  }
  
  public Iterator items(String paramString1, String paramString2)
  {
    Expression localExpression = ExpressionParser.parse(paramString2);
    if ((!(localExpression instanceof Predicate)) || (ExpressionParser.getError() != null)) {
      return Collections.EMPTY_LIST.iterator();
    }
    return items(paramString1, (Predicate)localExpression);
  }
  
  public Iterator items(String paramString, Predicate paramPredicate)
  {
    if (ALL_ITEMS.equals(paramString)) {
      return items(paramPredicate);
    }
    TupleSet localTupleSet = getGroup(paramString);
    return localTupleSet == null ? Collections.EMPTY_LIST.iterator() : localTupleSet.tuples(paramPredicate);
  }
  
  public void setValue(String paramString1, Predicate paramPredicate, String paramString2, Object paramObject)
  {
    Iterator localIterator = items(paramString1, paramPredicate);
    while (localIterator.hasNext())
    {
      VisualItem localVisualItem = (VisualItem)localIterator.next();
      localVisualItem.set(paramString2, paramObject);
    }
  }
  
  public void setVisible(String paramString, Predicate paramPredicate, boolean paramBoolean)
  {
    Iterator localIterator = items(paramString, paramPredicate);
    while (localIterator.hasNext())
    {
      VisualItem localVisualItem = (VisualItem)localIterator.next();
      localVisualItem.setVisible(paramBoolean);
    }
  }
  
  public void setInteractive(String paramString, Predicate paramPredicate, boolean paramBoolean)
  {
    Iterator localIterator = items(paramString, paramPredicate);
    while (localIterator.hasNext())
    {
      VisualItem localVisualItem = (VisualItem)localIterator.next();
      localVisualItem.setInteractive(paramBoolean);
    }
  }
  
  public Action putAction(String paramString, Action paramAction)
  {
    paramAction.setVisualization(this);
    this.m_actions.put(paramString, paramAction);
    return paramAction;
  }
  
  public Action getAction(String paramString)
  {
    return (Action)this.m_actions.get(paramString);
  }
  
  public Activity run(String paramString)
  {
    return this.m_actions.run(paramString);
  }
  
  public Activity runAfter(String paramString, long paramLong)
  {
    return this.m_actions.runAt(paramString, System.currentTimeMillis() + paramLong);
  }
  
  public Activity runAt(String paramString, long paramLong)
  {
    return this.m_actions.runAt(paramString, paramLong);
  }
  
  public Activity runAfter(String paramString1, String paramString2)
  {
    return this.m_actions.runAfter(paramString1, paramString2);
  }
  
  public Activity alwaysRunAfter(String paramString1, String paramString2)
  {
    return this.m_actions.alwaysRunAfter(paramString1, paramString2);
  }
  
  public Activity cancel(String paramString)
  {
    return this.m_actions.cancel(paramString);
  }
  
  public void setRendererFactory(RendererFactory paramRendererFactory)
  {
    invalidateAll();
    this.m_renderers = paramRendererFactory;
  }
  
  public RendererFactory getRendererFactory()
  {
    return this.m_renderers;
  }
  
  public Renderer getRenderer(VisualItem paramVisualItem)
  {
    if (paramVisualItem.getVisualization() != this) {
      throw new IllegalArgumentException("Input item not a member of this visualization.");
    }
    return this.m_renderers.getRenderer(paramVisualItem);
  }
  
  public synchronized void repaint()
  {
    Iterator localIterator = items(ValidatedPredicate.FALSE);
    while (localIterator.hasNext()) {
      ((VisualItem)localIterator.next()).validateBounds();
    }
    for (int i = 0; i < this.m_displays.size(); i++) {
      getDisplay(i).repaint();
    }
  }
  
  public Rectangle2D getBounds(String paramString)
  {
    return getBounds(paramString, new Rectangle2D.Double());
  }
  
  public Rectangle2D getBounds(String paramString, Rectangle2D paramRectangle2D)
  {
    Iterator localIterator = visibleItems(paramString);
    VisualItem localVisualItem;
    if (localIterator.hasNext())
    {
      localVisualItem = (VisualItem)localIterator.next();
      paramRectangle2D.setRect(localVisualItem.getBounds());
    }
    while (localIterator.hasNext())
    {
      localVisualItem = (VisualItem)localIterator.next();
      Rectangle2D.union(localVisualItem.getBounds(), paramRectangle2D, paramRectangle2D);
    }
    return paramRectangle2D;
  }
  
  public int getDisplayCount()
  {
    return this.m_displays.size();
  }
  
  public void addDisplay(Display paramDisplay)
  {
    this.m_displays.add(paramDisplay);
  }
  
  public Display getDisplay(int paramInt)
  {
    return (Display)this.m_displays.get(paramInt);
  }
  
  public boolean removeDisplay(Display paramDisplay)
  {
    return this.m_displays.remove(paramDisplay);
  }
  
  public void damageReport(VisualItem paramVisualItem, Rectangle2D paramRectangle2D)
  {
    for (int i = 0; i < this.m_displays.size(); i++)
    {
      Display localDisplay = getDisplay(i);
      if (localDisplay.getPredicate().getBoolean(paramVisualItem)) {
        localDisplay.damageReport(paramRectangle2D);
      }
    }
  }
}


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\prefuse\Visualization.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */