package prefuse.action.distortion;

import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;

public class FisheyeDistortion
  extends Distortion
{
  private double dx;
  private double dy;
  private double sz = 3.0D;
  
  public FisheyeDistortion()
  {
    this(4.0D);
  }
  
  public FisheyeDistortion(double paramDouble)
  {
    this(paramDouble, paramDouble);
  }
  
  public FisheyeDistortion(double paramDouble1, double paramDouble2)
  {
    this.dx = paramDouble1;
    this.dy = paramDouble2;
    this.m_distortX = (this.dx > 0.0D);
    this.m_distortY = (this.dy > 0.0D);
  }
  
  public double getXDistortionFactor()
  {
    return this.dx;
  }
  
  public void setXDistortionFactor(double paramDouble)
  {
    this.dx = paramDouble;
    this.m_distortX = (this.dx > 0.0D);
  }
  
  public double getYDistortionFactor()
  {
    return this.dy;
  }
  
  public void setYDistortionFactor(double paramDouble)
  {
    this.dy = paramDouble;
    this.m_distortY = (this.dy > 0.0D);
  }
  
  protected double distortX(double paramDouble, Point2D paramPoint2D, Rectangle2D paramRectangle2D)
  {
    return fisheye(paramDouble, paramPoint2D.getX(), this.dx, paramRectangle2D.getMinX(), paramRectangle2D.getMaxX());
  }
  
  protected double distortY(double paramDouble, Point2D paramPoint2D, Rectangle2D paramRectangle2D)
  {
    return fisheye(paramDouble, paramPoint2D.getY(), this.dy, paramRectangle2D.getMinY(), paramRectangle2D.getMaxY());
  }
  
  protected double distortSize(Rectangle2D paramRectangle2D1, double paramDouble1, double paramDouble2, Point2D paramPoint2D, Rectangle2D paramRectangle2D2)
  {
    if ((!this.m_distortX) && (!this.m_distortY)) {
      return 1.0D;
    }
    double d1 = 1.0D;
    double d2 = 1.0D;
    double d4;
    double d5;
    double d6;
    if (this.m_distortX)
    {
      d3 = paramPoint2D.getX();
      d4 = paramRectangle2D1.getMinX();
      d5 = paramRectangle2D1.getMaxX();
      d6 = Math.abs(d4 - d3) > Math.abs(d5 - d3) ? d4 : d5;
      if ((d6 < paramRectangle2D2.getMinX()) || (d6 > paramRectangle2D2.getMaxX())) {
        d6 = d6 == d4 ? d5 : d4;
      }
      d1 = fisheye(d6, d3, this.dx, paramRectangle2D2.getMinX(), paramRectangle2D2.getMaxX());
      d1 = Math.abs(paramDouble1 - d1) / paramRectangle2D1.getWidth();
    }
    if (this.m_distortY)
    {
      d3 = paramPoint2D.getY();
      d4 = paramRectangle2D1.getMinY();
      d5 = paramRectangle2D1.getMaxY();
      d6 = Math.abs(d4 - d3) > Math.abs(d5 - d3) ? d4 : d5;
      if ((d6 < paramRectangle2D2.getMinY()) || (d6 > paramRectangle2D2.getMaxY())) {
        d6 = d6 == d4 ? d5 : d4;
      }
      d2 = fisheye(d6, d3, this.dy, paramRectangle2D2.getMinY(), paramRectangle2D2.getMaxY());
      d2 = Math.abs(paramDouble2 - d2) / paramRectangle2D1.getHeight();
    }
    double d3 = !this.m_distortX ? d2 : !this.m_distortY ? d1 : Math.min(d1, d2);
    if ((Double.isInfinite(d3)) || (Double.isNaN(d3))) {
      return 1.0D;
    }
    return this.sz * d3;
  }
  
  private double fisheye(double paramDouble1, double paramDouble2, double paramDouble3, double paramDouble4, double paramDouble5)
  {
    if (paramDouble3 != 0.0D)
    {
      int i = paramDouble1 < paramDouble2 ? 1 : 0;
      double d2 = i != 0 ? paramDouble2 - paramDouble4 : paramDouble5 - paramDouble2;
      if (d2 == 0.0D) {
        d2 = paramDouble5 - paramDouble4;
      }
      double d1 = Math.abs(paramDouble1 - paramDouble2) / d2;
      d1 = (paramDouble3 + 1.0D) / (paramDouble3 + 1.0D / d1);
      return (i != 0 ? -1 : 1) * d2 * d1 + paramDouble2;
    }
    return paramDouble1;
  }
}


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\prefuse\action\distortion\FisheyeDistortion.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */