package prefuse.data.column;

import java.util.Arrays;
import prefuse.data.DataReadOnlyException;
import prefuse.data.DataTypeException;

public class FloatColumn
  extends AbstractColumn
{
  private float[] m_values;
  private int m_size;
  
  public FloatColumn()
  {
    this(0, 10, 0.0F);
  }
  
  public FloatColumn(int paramInt)
  {
    this(paramInt, paramInt, 0.0F);
  }
  
  public FloatColumn(int paramInt1, int paramInt2, float paramFloat)
  {
    super(Float.TYPE, new Float(paramFloat));
    if (paramInt2 < paramInt1) {
      throw new IllegalArgumentException("Capacity value can not be less than the row count.");
    }
    this.m_values = new float[paramInt2];
    Arrays.fill(this.m_values, paramFloat);
    this.m_size = paramInt1;
  }
  
  public int getRowCount()
  {
    return this.m_size;
  }
  
  public void setMaximumRow(int paramInt)
  {
    if (paramInt > this.m_values.length)
    {
      int i = Math.max(3 * this.m_values.length / 2 + 1, paramInt);
      float[] arrayOfFloat = new float[i];
      System.arraycopy(this.m_values, 0, arrayOfFloat, 0, this.m_size);
      Arrays.fill(arrayOfFloat, this.m_size, i, ((Float)this.m_defaultValue).floatValue());
      this.m_values = arrayOfFloat;
    }
    this.m_size = paramInt;
  }
  
  public Object get(int paramInt)
  {
    return new Float(getFloat(paramInt));
  }
  
  public void set(Object paramObject, int paramInt)
    throws DataTypeException
  {
    if (this.m_readOnly) {
      throw new DataReadOnlyException();
    }
    if (paramObject != null)
    {
      if ((paramObject instanceof Number)) {
        setFloat(((Number)paramObject).floatValue(), paramInt);
      } else if ((paramObject instanceof String)) {
        setString((String)paramObject, paramInt);
      } else {
        throw new DataTypeException(paramObject.getClass());
      }
    }
    else {
      throw new DataTypeException("Column does not accept null values");
    }
  }
  
  public float getFloat(int paramInt)
    throws DataTypeException
  {
    if ((paramInt < 0) || (paramInt > this.m_size)) {
      throw new IllegalArgumentException("Row index out of bounds: " + paramInt);
    }
    return this.m_values[paramInt];
  }
  
  public void setFloat(float paramFloat, int paramInt)
    throws DataTypeException
  {
    if (this.m_readOnly) {
      throw new DataReadOnlyException();
    }
    if ((paramInt < 0) || (paramInt >= this.m_size)) {
      throw new IllegalArgumentException("Row index out of bounds: " + paramInt);
    }
    float f = this.m_values[paramInt];
    if (f == paramFloat) {
      return;
    }
    this.m_values[paramInt] = paramFloat;
    fireColumnEvent(paramInt, f);
  }
  
  public String getString(int paramInt)
    throws DataTypeException
  {
    return String.valueOf(getFloat(paramInt));
  }
  
  public void setString(String paramString, int paramInt)
    throws DataTypeException
  {
    setFloat(Float.parseFloat(paramString), paramInt);
  }
  
  public int getInt(int paramInt)
    throws DataTypeException
  {
    return (int)getFloat(paramInt);
  }
  
  public long getLong(int paramInt)
    throws DataTypeException
  {
    return getFloat(paramInt);
  }
  
  public double getDouble(int paramInt)
    throws DataTypeException
  {
    return getFloat(paramInt);
  }
}


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\prefuse\data\column\FloatColumn.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */