package prefuse.data.expression.parser;

import java.io.InputStream;
import java.io.PrintStream;
import java.io.Reader;
import java.io.StringReader;
import java.util.Vector;
import java.util.logging.Logger;
import prefuse.data.expression.AndPredicate;
import prefuse.data.expression.ArithmeticExpression;
import prefuse.data.expression.BooleanLiteral;
import prefuse.data.expression.ColumnExpression;
import prefuse.data.expression.ComparisonPredicate;
import prefuse.data.expression.Expression;
import prefuse.data.expression.Function;
import prefuse.data.expression.FunctionTable;
import prefuse.data.expression.IfExpression;
import prefuse.data.expression.NotPredicate;
import prefuse.data.expression.NumericLiteral;
import prefuse.data.expression.ObjectLiteral;
import prefuse.data.expression.OrPredicate;
import prefuse.data.expression.Predicate;
import prefuse.data.expression.XorPredicate;
import prefuse.util.StringLib;

public class ExpressionParser
  implements ExpressionParserConstants
{
  private static final Logger s_logger = Logger.getLogger(ExpressionParser.class.getName());
  private static boolean s_init = false;
  private static Throwable s_error;
  private static boolean jj_initialized_once = false;
  public static ExpressionParserTokenManager token_source;
  static JavaCharStream jj_input_stream;
  public static Token token;
  public static Token jj_nt;
  private static int jj_ntk;
  private static int jj_gen;
  private static final int[] jj_la1 = new int[19];
  private static int[] jj_la1_0;
  private static int[] jj_la1_1;
  private static Vector jj_expentries = new Vector();
  private static int[] jj_expentry;
  private static int jj_kind = -1;
  
  public static synchronized Expression parse(String paramString, boolean paramBoolean)
  {
    if (!s_init)
    {
      new ExpressionParser(new StringReader(paramString));
      s_init = true;
    }
    else
    {
      ReInit(new StringReader(paramString));
    }
    try
    {
      Expression localExpression = Expression();
      s_error = null;
      s_logger.info("Parsed Expression: " + localExpression);
      return localExpression;
    }
    catch (ParseException localParseException)
    {
      s_error = localParseException;
      if (paramBoolean) {
        throw localParseException;
      }
      s_logger.warning("Expression Parse Error: " + localParseException.getMessage() + "\n" + StringLib.getStackTrace(localParseException));
    }
    return null;
  }
  
  public static synchronized Expression parse(String paramString)
  {
    return parse(paramString, false);
  }
  
  public static synchronized Throwable getError()
  {
    return s_error;
  }
  
  public static void main(String[] paramArrayOfString)
  {
    String[] arrayOfString = { "District <= 0", "IF x < 0 THEN x^2 ELSE x%0", "ABS(3+5*EXP(3)/SIN(5)*SIGN(32.3f)-ROUND(23.2e-1))", "x && y", "x & y", "if x<0 else x^2 then x%0", "((3+6)*(4+1))", "2*3%6", "2+3%6", "2*3^6" };
    boolean[] arrayOfBoolean = { true, true, true, true, false, false, true, true, true, true };
    for (int i = 0; i < arrayOfString.length; i++) {
      try
      {
        System.out.println("-> " + arrayOfString[i]);
        Expression localExpression = parse(arrayOfString[i]);
        System.out.print("   PARSE_SUCCESSFUL ");
        System.out.println(arrayOfBoolean[i] != 0 ? "GOOD" : "BAD");
        System.out.println("   " + localExpression);
      }
      catch (Throwable localThrowable)
      {
        System.out.print("   PARSE_FAILED ");
        System.out.println(arrayOfBoolean[i] == 0 ? "GOOD" : "BAD");
        if (arrayOfBoolean[i] != 0) {
          System.out.println("   " + StringLib.getStackTrace(localThrowable));
        }
      }
    }
  }
  
  public static final String Name()
    throws ParseException
  {
    Token localToken = jj_consume_token(25);
    return localToken.image;
  }
  
  public static final String Quoted()
    throws ParseException
  {
    Token localToken = jj_consume_token(24);
    return localToken.image.substring(1, localToken.image.length() - 1);
  }
  
  public static final Expression Expression()
    throws ParseException
  {
    Expression localExpression = OrExpression();
    return localExpression;
  }
  
  public static final Expression OrExpression()
    throws ParseException
  {
    Object localObject = XorExpression();
    for (;;)
    {
      switch (jj_ntk == -1 ? jj_ntk() : jj_ntk)
      {
      case 12: 
        break;
      default: 
        jj_la1[0] = jj_gen;
        break;
      }
      jj_consume_token(12);
      Expression localExpression = XorExpression();
      if ((localObject instanceof OrPredicate)) {
        ((OrPredicate)localObject).add((Predicate)localExpression);
      } else {
        localObject = new OrPredicate((Predicate)localObject, (Predicate)localExpression);
      }
    }
    return (Expression)localObject;
  }
  
  public static final Expression XorExpression()
    throws ParseException
  {
    Object localObject = AndExpression();
    for (;;)
    {
      switch (jj_ntk == -1 ? jj_ntk() : jj_ntk)
      {
      case 14: 
        break;
      default: 
        jj_la1[1] = jj_gen;
        break;
      }
      jj_consume_token(14);
      Expression localExpression = AndExpression();
      if ((localObject instanceof XorPredicate)) {
        ((XorPredicate)localObject).add((Predicate)localExpression);
      } else {
        localObject = new XorPredicate((Predicate)localObject, (Predicate)localExpression);
      }
    }
    return (Expression)localObject;
  }
  
  public static final Expression AndExpression()
    throws ParseException
  {
    Object localObject = EqualityExpression();
    for (;;)
    {
      switch (jj_ntk == -1 ? jj_ntk() : jj_ntk)
      {
      case 11: 
        break;
      default: 
        jj_la1[2] = jj_gen;
        break;
      }
      jj_consume_token(11);
      Expression localExpression = EqualityExpression();
      if ((localObject instanceof AndPredicate)) {
        ((AndPredicate)localObject).add((Predicate)localExpression);
      } else {
        localObject = new AndPredicate((Predicate)localObject, (Predicate)localExpression);
      }
    }
    return (Expression)localObject;
  }
  
  public static final Expression EqualityExpression()
    throws ParseException
  {
    Expression localExpression;
    int i;
    for (Object localObject = RelationalExpression();; localObject = new ComparisonPredicate(i, (Expression)localObject, localExpression))
    {
      switch (jj_ntk == -1 ? jj_ntk() : jj_ntk)
      {
      case 30: 
      case 35: 
        break;
      default: 
        jj_la1[3] = jj_gen;
        break;
      }
      Token localToken;
      switch (jj_ntk == -1 ? jj_ntk() : jj_ntk)
      {
      case 30: 
        localToken = jj_consume_token(30);
        break;
      case 35: 
        localToken = jj_consume_token(35);
        break;
      default: 
        jj_la1[4] = jj_gen;
        jj_consume_token(-1);
        throw new ParseException();
      }
      localExpression = RelationalExpression();
      i = localToken.kind == 30 ? 2 : 3;
    }
    return (Expression)localObject;
  }
  
  public static final Expression RelationalExpression()
    throws ParseException
  {
    int i = -1;
    Expression localExpression;
    for (Object localObject = AdditiveExpression();; localObject = new ComparisonPredicate(i, (Expression)localObject, localExpression))
    {
      switch (jj_ntk == -1 ? jj_ntk() : jj_ntk)
      {
      case 31: 
      case 32: 
      case 33: 
      case 34: 
        break;
      default: 
        jj_la1[5] = jj_gen;
        break;
      }
      Token localToken;
      switch (jj_ntk == -1 ? jj_ntk() : jj_ntk)
      {
      case 32: 
        localToken = jj_consume_token(32);
        break;
      case 31: 
        localToken = jj_consume_token(31);
        break;
      case 33: 
        localToken = jj_consume_token(33);
        break;
      case 34: 
        localToken = jj_consume_token(34);
        break;
      default: 
        jj_la1[6] = jj_gen;
        jj_consume_token(-1);
        throw new ParseException();
      }
      localExpression = AdditiveExpression();
      switch (localToken.kind)
      {
      case 32: 
        i = 0;
        break;
      case 31: 
        i = 1;
        break;
      case 33: 
        i = 4;
        break;
      case 34: 
        i = 5;
      }
    }
    return (Expression)localObject;
  }
  
  public static final Expression AdditiveExpression()
    throws ParseException
  {
    int i = -1;
    Expression localExpression;
    for (Object localObject = MultiplicativeExpression();; localObject = new ArithmeticExpression(i, (Expression)localObject, localExpression))
    {
      switch (jj_ntk == -1 ? jj_ntk() : jj_ntk)
      {
      case 36: 
      case 37: 
      case 41: 
        break;
      default: 
        jj_la1[7] = jj_gen;
        break;
      }
      Token localToken;
      switch (jj_ntk == -1 ? jj_ntk() : jj_ntk)
      {
      case 36: 
        localToken = jj_consume_token(36);
        break;
      case 37: 
        localToken = jj_consume_token(37);
        break;
      case 41: 
        localToken = jj_consume_token(41);
        break;
      default: 
        jj_la1[8] = jj_gen;
        jj_consume_token(-1);
        throw new ParseException();
      }
      localExpression = MultiplicativeExpression();
      switch (localToken.kind)
      {
      case 36: 
        i = 0;
        break;
      case 37: 
        i = 1;
        break;
      case 41: 
        i = 5;
      }
    }
    return (Expression)localObject;
  }
  
  public static final Expression MultiplicativeExpression()
    throws ParseException
  {
    int i = -1;
    Expression localExpression;
    for (Object localObject = UnaryExpression();; localObject = new ArithmeticExpression(i, (Expression)localObject, localExpression))
    {
      switch (jj_ntk == -1 ? jj_ntk() : jj_ntk)
      {
      case 38: 
      case 39: 
      case 40: 
        break;
      default: 
        jj_la1[9] = jj_gen;
        break;
      }
      Token localToken;
      switch (jj_ntk == -1 ? jj_ntk() : jj_ntk)
      {
      case 38: 
        localToken = jj_consume_token(38);
        break;
      case 39: 
        localToken = jj_consume_token(39);
        break;
      case 40: 
        localToken = jj_consume_token(40);
        break;
      default: 
        jj_la1[10] = jj_gen;
        jj_consume_token(-1);
        throw new ParseException();
      }
      localExpression = UnaryExpression();
      switch (localToken.kind)
      {
      case 38: 
        i = 2;
        break;
      case 39: 
        i = 3;
        break;
      case 40: 
        i = 4;
      }
    }
    return (Expression)localObject;
  }
  
  public static final Expression UnaryExpression()
    throws ParseException
  {
    Expression localExpression;
    switch (jj_ntk == -1 ? jj_ntk() : jj_ntk)
    {
    case 36: 
    case 37: 
      Token localToken;
      switch (jj_ntk == -1 ? jj_ntk() : jj_ntk)
      {
      case 36: 
        localToken = jj_consume_token(36);
        break;
      case 37: 
        localToken = jj_consume_token(37);
        break;
      default: 
        jj_la1[11] = jj_gen;
        jj_consume_token(-1);
        throw new ParseException();
      }
      localExpression = UnaryExpression();
      if ((localToken.kind == 37) && ((localExpression instanceof NumericLiteral)))
      {
        Number localNumber = (Number)localExpression.get(null);
        if ((localNumber instanceof Integer)) {
          return new NumericLiteral(-1 * localNumber.intValue());
        }
        if ((localNumber instanceof Double)) {
          return new NumericLiteral(-1.0D * localNumber.doubleValue());
        }
        if ((localNumber instanceof Long)) {
          return new NumericLiteral(-1L * localNumber.longValue());
        }
        if ((localNumber instanceof Float)) {
          return new NumericLiteral(-1.0F * localNumber.floatValue());
        }
        return new ArithmeticExpression(2, new NumericLiteral(-1), localExpression);
      }
      if (localToken.kind == 37) {
        return new ArithmeticExpression(2, new NumericLiteral(-1), localExpression);
      }
      return localExpression;
    case 13: 
      localExpression = UnaryExpressionNotPlusMinus();
      return localExpression;
    case 6: 
    case 7: 
    case 8: 
    case 15: 
    case 19: 
    case 20: 
    case 21: 
    case 23: 
    case 24: 
    case 25: 
    case 28: 
      localExpression = PrimaryExpression();
      return localExpression;
    }
    jj_la1[12] = jj_gen;
    jj_consume_token(-1);
    throw new ParseException();
  }
  
  public static final Expression UnaryExpressionNotPlusMinus()
    throws ParseException
  {
    jj_consume_token(13);
    Expression localExpression = UnaryExpression();
    if ((localExpression instanceof NotPredicate)) {
      return ((NotPredicate)localExpression).getPredicate();
    }
    if (!(localExpression instanceof Predicate)) {
      throw new ParseException("Can't negate a non-predicate");
    }
    return new NotPredicate((Predicate)localExpression);
  }
  
  public static final Expression PrimaryExpression()
    throws ParseException
  {
    Expression localExpression;
    switch (jj_ntk == -1 ? jj_ntk() : jj_ntk)
    {
    case 6: 
    case 7: 
    case 15: 
    case 19: 
    case 20: 
    case 21: 
    case 23: 
      localExpression = Literal();
      return localExpression;
    case 8: 
      localExpression = IfStatement();
      return localExpression;
    case 24: 
    case 25: 
      localExpression = Identifier();
      return localExpression;
    case 28: 
      jj_consume_token(28);
      localExpression = Expression();
      jj_consume_token(29);
      return localExpression;
    }
    jj_la1[13] = jj_gen;
    jj_consume_token(-1);
    throw new ParseException();
  }
  
  public static final Expression Literal()
    throws ParseException
  {
    Token localToken;
    switch (jj_ntk == -1 ? jj_ntk() : jj_ntk)
    {
    case 15: 
      localToken = jj_consume_token(15);
      return new NumericLiteral(Integer.parseInt(localToken.image));
    case 19: 
      localToken = jj_consume_token(19);
      return new NumericLiteral(Long.parseLong(localToken.image));
    case 21: 
      localToken = jj_consume_token(21);
      return new NumericLiteral(Float.parseFloat(localToken.image));
    case 20: 
      localToken = jj_consume_token(20);
      return new NumericLiteral(Double.parseDouble(localToken.image));
    case 23: 
      localToken = jj_consume_token(23);
      return new ObjectLiteral(localToken.image.substring(1, localToken.image.length() - 1));
    case 6: 
      jj_consume_token(6);
      return new BooleanLiteral(true);
    case 7: 
      jj_consume_token(7);
      return new BooleanLiteral(false);
    }
    jj_la1[14] = jj_gen;
    jj_consume_token(-1);
    throw new ParseException();
  }
  
  public static final Expression Identifier()
    throws ParseException
  {
    Function localFunction = null;
    String str;
    switch (jj_ntk == -1 ? jj_ntk() : jj_ntk)
    {
    case 24: 
      str = Quoted();
      return new ColumnExpression(str);
    case 25: 
      str = Name();
      switch (jj_ntk == -1 ? jj_ntk() : jj_ntk)
      {
      case 28: 
        jj_consume_token(28);
        localFunction = FunctionTable.createFunction(str);
        switch (jj_ntk == -1 ? jj_ntk() : jj_ntk)
        {
        case 6: 
        case 7: 
        case 8: 
        case 13: 
        case 15: 
        case 19: 
        case 20: 
        case 21: 
        case 23: 
        case 24: 
        case 25: 
        case 28: 
        case 36: 
        case 37: 
          Expression localExpression = Expression();
          localFunction.addParameter(localExpression);
          for (;;)
          {
            switch (jj_ntk == -1 ? jj_ntk() : jj_ntk)
            {
            case 42: 
              break;
            default: 
              jj_la1[15] = jj_gen;
              break;
            }
            jj_consume_token(42);
            localExpression = Expression();
            localFunction.addParameter(localExpression);
          }
        }
        jj_la1[16] = jj_gen;
        jj_consume_token(29);
        break;
      default: 
        jj_la1[17] = jj_gen;
      }
      return localFunction == null ? new ColumnExpression(str) : localFunction;
    }
    jj_la1[18] = jj_gen;
    jj_consume_token(-1);
    throw new ParseException();
  }
  
  public static final Expression IfStatement()
    throws ParseException
  {
    jj_consume_token(8);
    Expression localExpression1 = Expression();
    jj_consume_token(9);
    Expression localExpression2 = Expression();
    jj_consume_token(10);
    Expression localExpression3 = Expression();
    if (!(localExpression1 instanceof Predicate)) {
      throw new ParseException("IF-statement test must be a predicate");
    }
    return new IfExpression((Predicate)localExpression1, localExpression2, localExpression3);
  }
  
  private static void jj_la1_0()
  {
    jj_la1_0 = new int[] { 4096, 16384, 2048, 1073741824, 1073741824, Integer.MIN_VALUE, Integer.MIN_VALUE, 0, 0, 0, 0, 0, 330867136, 330858944, 12091584, 0, 330867136, 268435456, 50331648 };
  }
  
  private static void jj_la1_1()
  {
    jj_la1_1 = new int[] { 0, 0, 0, 8, 8, 7, 7, 560, 560, 448, 448, 48, 48, 0, 0, 1024, 48, 0, 0 };
  }
  
  public ExpressionParser(InputStream paramInputStream)
  {
    if (jj_initialized_once)
    {
      System.out.println("ERROR: Second call to constructor of static parser.  You must");
      System.out.println("       either use ReInit() or set the JavaCC option STATIC to false");
      System.out.println("       during parser generation.");
      throw new Error();
    }
    jj_initialized_once = true;
    jj_input_stream = new JavaCharStream(paramInputStream, 1, 1);
    token_source = new ExpressionParserTokenManager(jj_input_stream);
    token = new Token();
    jj_ntk = -1;
    jj_gen = 0;
    for (int i = 0; i < 19; i++) {
      jj_la1[i] = -1;
    }
  }
  
  public static void ReInit(InputStream paramInputStream)
  {
    jj_input_stream.ReInit(paramInputStream, 1, 1);
    ExpressionParserTokenManager.ReInit(jj_input_stream);
    token = new Token();
    jj_ntk = -1;
    jj_gen = 0;
    for (int i = 0; i < 19; i++) {
      jj_la1[i] = -1;
    }
  }
  
  public ExpressionParser(Reader paramReader)
  {
    if (jj_initialized_once)
    {
      System.out.println("ERROR: Second call to constructor of static parser.  You must");
      System.out.println("       either use ReInit() or set the JavaCC option STATIC to false");
      System.out.println("       during parser generation.");
      throw new Error();
    }
    jj_initialized_once = true;
    jj_input_stream = new JavaCharStream(paramReader, 1, 1);
    token_source = new ExpressionParserTokenManager(jj_input_stream);
    token = new Token();
    jj_ntk = -1;
    jj_gen = 0;
    for (int i = 0; i < 19; i++) {
      jj_la1[i] = -1;
    }
  }
  
  public static void ReInit(Reader paramReader)
  {
    jj_input_stream.ReInit(paramReader, 1, 1);
    ExpressionParserTokenManager.ReInit(jj_input_stream);
    token = new Token();
    jj_ntk = -1;
    jj_gen = 0;
    for (int i = 0; i < 19; i++) {
      jj_la1[i] = -1;
    }
  }
  
  public ExpressionParser(ExpressionParserTokenManager paramExpressionParserTokenManager)
  {
    if (jj_initialized_once)
    {
      System.out.println("ERROR: Second call to constructor of static parser.  You must");
      System.out.println("       either use ReInit() or set the JavaCC option STATIC to false");
      System.out.println("       during parser generation.");
      throw new Error();
    }
    jj_initialized_once = true;
    token_source = paramExpressionParserTokenManager;
    token = new Token();
    jj_ntk = -1;
    jj_gen = 0;
    for (int i = 0; i < 19; i++) {
      jj_la1[i] = -1;
    }
  }
  
  public void ReInit(ExpressionParserTokenManager paramExpressionParserTokenManager)
  {
    token_source = paramExpressionParserTokenManager;
    token = new Token();
    jj_ntk = -1;
    jj_gen = 0;
    for (int i = 0; i < 19; i++) {
      jj_la1[i] = -1;
    }
  }
  
  private static final Token jj_consume_token(int paramInt)
    throws ParseException
  {
    Token localToken;
    if ((localToken = token).next != null) {
      token = token.next;
    } else {
      token = token.next = ExpressionParserTokenManager.getNextToken();
    }
    jj_ntk = -1;
    if (token.kind == paramInt)
    {
      jj_gen += 1;
      return token;
    }
    token = localToken;
    jj_kind = paramInt;
    throw generateParseException();
  }
  
  public static final Token getNextToken()
  {
    if (token.next != null) {
      token = token.next;
    } else {
      token = token.next = ExpressionParserTokenManager.getNextToken();
    }
    jj_ntk = -1;
    jj_gen += 1;
    return token;
  }
  
  public static final Token getToken(int paramInt)
  {
    Token localToken = token;
    for (int i = 0; i < paramInt; i++) {
      if (localToken.next != null) {
        localToken = localToken.next;
      } else {
        localToken = localToken.next = ExpressionParserTokenManager.getNextToken();
      }
    }
    return localToken;
  }
  
  private static final int jj_ntk()
  {
    if ((jj_nt = token.next) == null) {
      return jj_ntk = (token.next = ExpressionParserTokenManager.getNextToken()).kind;
    }
    return jj_ntk = jj_nt.kind;
  }
  
  public static ParseException generateParseException()
  {
    jj_expentries.removeAllElements();
    boolean[] arrayOfBoolean = new boolean[43];
    for (int i = 0; i < 43; i++) {
      arrayOfBoolean[i] = false;
    }
    if (jj_kind >= 0)
    {
      arrayOfBoolean[jj_kind] = true;
      jj_kind = -1;
    }
    for (i = 0; i < 19; i++) {
      if (jj_la1[i] == jj_gen) {
        for (j = 0; j < 32; j++)
        {
          if ((jj_la1_0[i] & 1 << j) != 0) {
            arrayOfBoolean[j] = true;
          }
          if ((jj_la1_1[i] & 1 << j) != 0) {
            arrayOfBoolean[(32 + j)] = true;
          }
        }
      }
    }
    for (i = 0; i < 43; i++) {
      if (arrayOfBoolean[i] != 0)
      {
        jj_expentry = new int[1];
        jj_expentry[0] = i;
        jj_expentries.addElement(jj_expentry);
      }
    }
    int[][] arrayOfInt = new int[jj_expentries.size()][];
    for (int j = 0; j < jj_expentries.size(); j++) {
      arrayOfInt[j] = ((int[])(int[])jj_expentries.elementAt(j));
    }
    return new ParseException(token, arrayOfInt, tokenImage);
  }
  
  public static final void enable_tracing() {}
  
  public static final void disable_tracing() {}
  
  static
  {
    jj_la1_0();
    jj_la1_1();
  }
}


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\prefuse\data\expression\parser\ExpressionParser.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */