package prefuse.data.tuple;

import java.util.Date;
import prefuse.data.Graph;
import prefuse.data.Schema;
import prefuse.data.Table;
import prefuse.data.Tuple;

public class TableTuple
  implements Tuple
{
  protected Table m_table;
  protected int m_row;
  
  protected void init(Table paramTable, Graph paramGraph, int paramInt)
  {
    this.m_table = paramTable;
    this.m_row = (this.m_table.isValidRow(paramInt) ? paramInt : -1);
  }
  
  public Schema getSchema()
  {
    return this.m_table.getSchema();
  }
  
  public Table getTable()
  {
    return this.m_table;
  }
  
  public int getRow()
  {
    return this.m_row;
  }
  
  public boolean isValid()
  {
    return this.m_row != -1;
  }
  
  void invalidate()
  {
    this.m_row = -1;
  }
  
  private void validityCheck()
  {
    if (this.m_row == -1) {
      throw new IllegalStateException("This tuple is no longer valid. It has been deleted from its table");
    }
  }
  
  public Class getColumnType(String paramString)
  {
    return this.m_table.getColumnType(paramString);
  }
  
  public Class getColumnType(int paramInt)
  {
    return this.m_table.getColumnType(paramInt);
  }
  
  public int getColumnIndex(String paramString)
  {
    return this.m_table.getColumnNumber(paramString);
  }
  
  public int getColumnCount()
  {
    return this.m_table.getColumnCount();
  }
  
  public String getColumnName(int paramInt)
  {
    return this.m_table.getColumnName(paramInt);
  }
  
  public boolean canGet(String paramString, Class paramClass)
  {
    return this.m_table.canGet(paramString, paramClass);
  }
  
  public boolean canSet(String paramString, Class paramClass)
  {
    return this.m_table.canSet(paramString, paramClass);
  }
  
  public final Object get(String paramString)
  {
    validityCheck();
    return this.m_table.get(this.m_row, paramString);
  }
  
  public final void set(String paramString, Object paramObject)
  {
    validityCheck();
    this.m_table.set(this.m_row, paramString, paramObject);
  }
  
  public final Object getValueAt(int paramInt)
  {
    validityCheck();
    return this.m_table.getValueAt(this.m_row, paramInt);
  }
  
  public final void setValueAt(int paramInt, Object paramObject)
  {
    validityCheck();
    this.m_table.setValueAt(this.m_row, paramInt, paramObject);
  }
  
  public Object getDefault(String paramString)
  {
    validityCheck();
    return this.m_table.getDefault(paramString);
  }
  
  public void revertToDefault(String paramString)
  {
    validityCheck();
    this.m_table.revertToDefault(this.m_row, paramString);
  }
  
  public final boolean canGetInt(String paramString)
  {
    return this.m_table.canGetInt(paramString);
  }
  
  public final boolean canSetInt(String paramString)
  {
    return this.m_table.canSetInt(paramString);
  }
  
  public final int getInt(String paramString)
  {
    validityCheck();
    return this.m_table.getInt(this.m_row, paramString);
  }
  
  public final void setInt(String paramString, int paramInt)
  {
    validityCheck();
    this.m_table.setInt(this.m_row, paramString, paramInt);
  }
  
  public final boolean canGetLong(String paramString)
  {
    return this.m_table.canGetLong(paramString);
  }
  
  public final boolean canSetLong(String paramString)
  {
    return this.m_table.canSetLong(paramString);
  }
  
  public final long getLong(String paramString)
  {
    validityCheck();
    return this.m_table.getLong(this.m_row, paramString);
  }
  
  public final void setLong(String paramString, long paramLong)
  {
    validityCheck();
    this.m_table.setLong(this.m_row, paramString, paramLong);
  }
  
  public final boolean canGetFloat(String paramString)
  {
    return this.m_table.canGetFloat(paramString);
  }
  
  public final boolean canSetFloat(String paramString)
  {
    return this.m_table.canSetFloat(paramString);
  }
  
  public final float getFloat(String paramString)
  {
    validityCheck();
    return this.m_table.getFloat(this.m_row, paramString);
  }
  
  public final void setFloat(String paramString, float paramFloat)
  {
    validityCheck();
    this.m_table.setFloat(this.m_row, paramString, paramFloat);
  }
  
  public final boolean canGetDouble(String paramString)
  {
    return this.m_table.canGetDouble(paramString);
  }
  
  public final boolean canSetDouble(String paramString)
  {
    return this.m_table.canSetDouble(paramString);
  }
  
  public final double getDouble(String paramString)
  {
    validityCheck();
    return this.m_table.getDouble(this.m_row, paramString);
  }
  
  public final void setDouble(String paramString, double paramDouble)
  {
    validityCheck();
    this.m_table.setDouble(this.m_row, paramString, paramDouble);
  }
  
  public final boolean canGetBoolean(String paramString)
  {
    return this.m_table.canGetBoolean(paramString);
  }
  
  public final boolean canSetBoolean(String paramString)
  {
    return this.m_table.canSetBoolean(paramString);
  }
  
  public final boolean getBoolean(String paramString)
  {
    validityCheck();
    return this.m_table.getBoolean(this.m_row, paramString);
  }
  
  public final void setBoolean(String paramString, boolean paramBoolean)
  {
    validityCheck();
    this.m_table.setBoolean(this.m_row, paramString, paramBoolean);
  }
  
  public final boolean canGetString(String paramString)
  {
    return this.m_table.canGetString(paramString);
  }
  
  public final boolean canSetString(String paramString)
  {
    return this.m_table.canSetString(paramString);
  }
  
  public final String getString(String paramString)
  {
    validityCheck();
    return this.m_table.getString(this.m_row, paramString);
  }
  
  public final void setString(String paramString1, String paramString2)
  {
    validityCheck();
    this.m_table.setString(this.m_row, paramString1, paramString2);
  }
  
  public final boolean canGetDate(String paramString)
  {
    return this.m_table.canGetDate(paramString);
  }
  
  public final boolean canSetDate(String paramString)
  {
    return this.m_table.canSetDate(paramString);
  }
  
  public final Date getDate(String paramString)
  {
    validityCheck();
    return this.m_table.getDate(this.m_row, paramString);
  }
  
  public final void setDate(String paramString, Date paramDate)
  {
    validityCheck();
    this.m_table.setDate(this.m_row, paramString, paramDate);
  }
  
  public String toString()
  {
    StringBuffer localStringBuffer = new StringBuffer();
    localStringBuffer.append("Tuple[");
    for (int i = 0; i < getColumnCount(); i++)
    {
      if (i > 0) {
        localStringBuffer.append(',');
      }
      try
      {
        localStringBuffer.append(getValueAt(i).toString());
      }
      catch (Exception localException)
      {
        localStringBuffer.append("?");
      }
    }
    localStringBuffer.append("]");
    return localStringBuffer.toString();
  }
}


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\prefuse\data\tuple\TableTuple.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */