package prefuse.render;

import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Shape;
import java.awt.geom.AffineTransform;
import java.awt.geom.Point2D;
import java.awt.geom.Point2D.Double;
import java.awt.geom.Rectangle2D;
import java.awt.geom.Rectangle2D.Double;
import java.awt.geom.RectangularShape;
import java.awt.geom.RoundRectangle2D;
import java.awt.geom.RoundRectangle2D.Double;
import prefuse.util.ColorLib;
import prefuse.util.FontLib;
import prefuse.util.GraphicsLib;
import prefuse.util.StringLib;
import prefuse.visual.VisualItem;

public class LabelRenderer
  extends AbstractShapeRenderer
{
  protected ImageFactory m_images = null;
  protected String m_labelName = "label";
  protected String m_imageName = null;
  protected int m_xAlign = 2;
  protected int m_yAlign = 2;
  protected int m_horizBorder = 2;
  protected int m_vertBorder = 0;
  protected int m_imageMargin = 0;
  protected int m_arcWidth = 0;
  protected int m_arcHeight = 0;
  protected int m_maxTextWidth = -1;
  AffineTransform m_transform = new AffineTransform();
  protected RectangularShape m_bbox = new Rectangle2D.Double();
  protected Point2D m_pt = new Point2D.Double();
  protected Font m_font;
  
  public LabelRenderer() {}
  
  public LabelRenderer(String paramString)
  {
    setTextField(paramString);
  }
  
  public LabelRenderer(String paramString1, String paramString2)
  {
    setTextField(paramString1);
    setImageField(paramString2);
  }
  
  public void setRoundedCorner(int paramInt1, int paramInt2)
  {
    if (((paramInt1 == 0) || (paramInt2 == 0)) && (!(this.m_bbox instanceof Rectangle2D)))
    {
      this.m_bbox = new Rectangle2D.Double();
    }
    else
    {
      if (!(this.m_bbox instanceof RoundRectangle2D)) {
        this.m_bbox = new RoundRectangle2D.Double();
      }
      ((RoundRectangle2D)this.m_bbox).setRoundRect(0.0D, 0.0D, 10.0D, 10.0D, paramInt1, paramInt2);
      this.m_arcWidth = paramInt1;
      this.m_arcHeight = paramInt2;
    }
  }
  
  public String getTextField()
  {
    return this.m_labelName;
  }
  
  public void setTextField(String paramString)
  {
    this.m_labelName = paramString;
  }
  
  public void setMaxTextWidth(int paramInt)
  {
    this.m_maxTextWidth = paramInt;
  }
  
  protected String getText(VisualItem paramVisualItem)
  {
    String str = null;
    if (paramVisualItem.canGetString(this.m_labelName))
    {
      str = paramVisualItem.getString(this.m_labelName);
      if (this.m_maxTextWidth > -1)
      {
        Font localFont = paramVisualItem.getFont();
        FontMetrics localFontMetrics = DEFAULT_GRAPHICS.getFontMetrics(localFont);
        if (localFontMetrics.stringWidth(str) > this.m_maxTextWidth) {
          str = StringLib.abbreviate(str, localFontMetrics, this.m_maxTextWidth);
        }
      }
    }
    return str;
  }
  
  public String getImageField()
  {
    return this.m_imageName;
  }
  
  public void setImageField(String paramString)
  {
    if (paramString != null) {
      this.m_images = new ImageFactory();
    }
    this.m_imageName = paramString;
  }
  
  public void setMaxImageDimensions(int paramInt1, int paramInt2)
  {
    if (this.m_images == null) {
      this.m_images = new ImageFactory();
    }
    this.m_images.setMaxImageDimensions(paramInt1, paramInt2);
  }
  
  protected String getImageLocation(VisualItem paramVisualItem)
  {
    return paramVisualItem.canGetString(this.m_imageName) ? paramVisualItem.getString(this.m_imageName) : null;
  }
  
  protected Image getImage(VisualItem paramVisualItem)
  {
    String str = getImageLocation(paramVisualItem);
    return str == null ? null : this.m_images.getImage(str);
  }
  
  protected Shape getRawShape(VisualItem paramVisualItem)
  {
    String str = getText(paramVisualItem);
    Image localImage = getImage(paramVisualItem);
    double d1 = paramVisualItem.getSize();
    int i = d1 != 1.0D ? 1 : 0;
    double d2 = 0.0D;
    double d3 = 0.0D;
    if (localImage != null)
    {
      d3 = localImage.getHeight(null);
      d2 = localImage.getWidth(null);
    }
    int j = 0;
    int k = 0;
    if (str != null)
    {
      this.m_font = paramVisualItem.getFont();
      if (i != 0) {
        this.m_font = FontLib.getFont(this.m_font.getName(), this.m_font.getStyle(), d1 * this.m_font.getSize());
      }
      FontMetrics localFontMetrics = DEFAULT_GRAPHICS.getFontMetrics(this.m_font);
      k = localFontMetrics.getHeight();
      j = localFontMetrics.stringWidth(str);
    }
    double d4 = j + d1 * (d2 + 2 * this.m_horizBorder + ((j > 0) && (d2 > 0.0D) ? this.m_imageMargin : 0));
    double d5 = Math.max(k, d1 * d3) + d1 * 2.0D * this.m_vertBorder;
    getAlignedPoint(this.m_pt, paramVisualItem, d4, d5, this.m_xAlign, this.m_yAlign);
    if ((this.m_bbox instanceof RoundRectangle2D))
    {
      RoundRectangle2D localRoundRectangle2D = (RoundRectangle2D)this.m_bbox;
      localRoundRectangle2D.setRoundRect(this.m_pt.getX(), this.m_pt.getY(), d4, d5, d1 * this.m_arcWidth, d1 * this.m_arcHeight);
    }
    else
    {
      this.m_bbox.setFrame(this.m_pt.getX(), this.m_pt.getY(), d4, d5);
    }
    return this.m_bbox;
  }
  
  protected static void getAlignedPoint(Point2D paramPoint2D, VisualItem paramVisualItem, double paramDouble1, double paramDouble2, int paramInt1, int paramInt2)
  {
    double d1 = paramVisualItem.getX();
    double d2 = paramVisualItem.getY();
    if ((Double.isNaN(d1)) || (Double.isInfinite(d1))) {
      d1 = 0.0D;
    }
    if ((Double.isNaN(d2)) || (Double.isInfinite(d2))) {
      d2 = 0.0D;
    }
    if (paramInt1 == 2) {
      d1 -= paramDouble1 / 2.0D;
    } else if (paramInt1 == 1) {
      d1 -= paramDouble1;
    }
    if (paramInt2 == 2) {
      d2 -= paramDouble2 / 2.0D;
    } else if (paramInt2 == 1) {
      d2 -= paramDouble2;
    }
    paramPoint2D.setLocation(d1, d2);
  }
  
  public void render(Graphics2D paramGraphics2D, VisualItem paramVisualItem)
  {
    RectangularShape localRectangularShape = (RectangularShape)getShape(paramVisualItem);
    if (localRectangularShape == null) {
      return;
    }
    int i = getRenderType(paramVisualItem);
    if ((i == 2) || (i == 3)) {
      GraphicsLib.paint(paramGraphics2D, paramVisualItem, localRectangularShape, getStroke(paramVisualItem), 2);
    }
    String str = getText(paramVisualItem);
    Image localImage = getImage(paramVisualItem);
    if ((str == null) && (localImage == null)) {
      return;
    }
    double d1 = paramVisualItem.getSize();
    int j = 1.5D > Math.max(paramGraphics2D.getTransform().getScaleX(), paramGraphics2D.getTransform().getScaleY()) ? 1 : 0;
    double d2 = localRectangularShape.getMinX() + d1 * this.m_horizBorder;
    double d4;
    if (localImage != null)
    {
      double d3 = d1 * localImage.getWidth(null);
      d4 = d1 * localImage.getHeight(null);
      double d5 = localRectangularShape.getMinY() + (localRectangularShape.getHeight() - d4) / 2.0D;
      if ((j != 0) && (d1 == 1.0D))
      {
        paramGraphics2D.drawImage(localImage, (int)d2, (int)d5, null);
      }
      else
      {
        this.m_transform.setTransform(d1, 0.0D, 0.0D, d1, d2, d5);
        paramGraphics2D.drawImage(localImage, this.m_transform, null);
      }
      d2 += d3 + ((str != null) && (d3 > 0.0D) ? d1 * this.m_imageMargin : 0.0D);
    }
    if (str != null)
    {
      int k = paramVisualItem.getTextColor();
      if (ColorLib.alpha(k) > 0)
      {
        paramGraphics2D.setPaint(ColorLib.getColor(k));
        paramGraphics2D.setFont(this.m_font);
        FontMetrics localFontMetrics = DEFAULT_GRAPHICS.getFontMetrics(this.m_font);
        d4 = localRectangularShape.getCenterY() - ((localFontMetrics.getHeight() >> 1) - localFontMetrics.getAscent());
        if (j != 0) {
          paramGraphics2D.drawString(str, (int)d2, (int)d4);
        } else {
          paramGraphics2D.drawString(str, (float)d2, (float)d4);
        }
      }
    }
    if ((i == 1) || (i == 3)) {
      GraphicsLib.paint(paramGraphics2D, paramVisualItem, localRectangularShape, getStroke(paramVisualItem), 1);
    }
  }
  
  public ImageFactory getImageFactory()
  {
    if (this.m_images == null) {
      this.m_images = new ImageFactory();
    }
    return this.m_images;
  }
  
  public void setImageFactory(ImageFactory paramImageFactory)
  {
    this.m_images = paramImageFactory;
  }
  
  public int getHorizontalAlignment()
  {
    return this.m_xAlign;
  }
  
  public int getVerticalAlignment()
  {
    return this.m_yAlign;
  }
  
  public void setHorizontalAlignment(int paramInt)
  {
    this.m_xAlign = paramInt;
  }
  
  public void setVerticalAlignment(int paramInt)
  {
    this.m_yAlign = paramInt;
  }
  
  public int getHorizontalPadding()
  {
    return this.m_horizBorder;
  }
  
  public void setHorizontalPadding(int paramInt)
  {
    this.m_horizBorder = paramInt;
  }
  
  public int getVerticalPadding()
  {
    return this.m_vertBorder;
  }
  
  public void setVerticalPadding(int paramInt)
  {
    this.m_vertBorder = paramInt;
  }
}


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\prefuse\render\LabelRenderer.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */