package prefuse.util;

import java.awt.BasicStroke;
import java.awt.Font;
import java.awt.Stroke;
import java.awt.geom.Rectangle2D;
import java.awt.geom.Rectangle2D.Double;
import prefuse.Display;
import prefuse.data.Schema;
import prefuse.visual.VisualItem;

public class PrefuseLib
{
  private static final String GROUP_DELIMITER = PrefuseConfig.get("data.delimiter");
  private static final double SIZE_SCALE_FACTOR = PrefuseConfig.getDouble("size.scale2D");
  public static final String FIELD_PREFIX = PrefuseConfig.get("data.visual.fieldPrefix");
  
  public static String getMemoryUsageInKB()
  {
    long l1 = Runtime.getRuntime().totalMemory() / 2048L;
    long l2 = Runtime.getRuntime().freeMemory() / 2048L;
    long l3 = Runtime.getRuntime().maxMemory() / 2048L;
    return "Memory: " + (l1 - l2) + "k / " + l1 + "k / " + l3 + "k";
  }
  
  public static String getMemoryUsageInMB()
  {
    long l1 = Runtime.getRuntime().totalMemory() / 2097152L;
    long l2 = Runtime.getRuntime().freeMemory() / 2097152L;
    long l3 = Runtime.getRuntime().maxMemory() / 2097152L;
    return "Memory: " + (l1 - l2) + "M / " + l1 + "M / " + l3 + "M";
  }
  
  public static String getDisplayStats(Display paramDisplay)
  {
    float f = (float)Math.round(paramDisplay.getFrameRate() * 100.0D) / 100.0F;
    Runtime localRuntime = Runtime.getRuntime();
    long l1 = localRuntime.totalMemory() / 2097152L;
    long l2 = localRuntime.freeMemory() / 2097152L;
    long l3 = localRuntime.maxMemory() / 2097152L;
    StringBuffer localStringBuffer = new StringBuffer();
    localStringBuffer.append("frame rate: ").append(f).append("fps - ");
    localStringBuffer.append(paramDisplay.getVisibleItemCount()).append(" items - ");
    localStringBuffer.append("fonts(").append(FontLib.getCacheMissCount());
    localStringBuffer.append(") colors(");
    localStringBuffer.append(ColorLib.getCacheMissCount()).append(')');
    localStringBuffer.append(" mem(");
    localStringBuffer.append(l1 - l2).append("M / ");
    localStringBuffer.append(l3).append("M)");
    localStringBuffer.append(" (x:");
    localStringBuffer.append(StringLib.formatNumber(paramDisplay.getDisplayX(), 2));
    localStringBuffer.append(", y:");
    localStringBuffer.append(StringLib.formatNumber(paramDisplay.getDisplayY(), 2));
    localStringBuffer.append(", z:");
    localStringBuffer.append(StringLib.formatNumber(paramDisplay.getScale(), 5)).append(")");
    return localStringBuffer.toString();
  }
  
  public static double getSize2D(double paramDouble)
  {
    return Math.pow(paramDouble, SIZE_SCALE_FACTOR);
  }
  
  public static double distance(VisualItem paramVisualItem1, VisualItem paramVisualItem2)
  {
    double d1 = paramVisualItem1.getX() - paramVisualItem2.getX();
    double d2 = paramVisualItem1.getY() - paramVisualItem2.getY();
    return Math.sqrt(d1 * d1 + d2 * d2);
  }
  
  public static void update(VisualItem paramVisualItem, String paramString, Object paramObject)
  {
    paramVisualItem.set(getStartField(paramString), paramVisualItem.get(paramString));
    paramVisualItem.set(paramString, paramObject);
    paramVisualItem.set(getEndField(paramString), paramObject);
  }
  
  public static void updateInt(VisualItem paramVisualItem, String paramString, int paramInt)
  {
    paramVisualItem.setInt(getStartField(paramString), paramVisualItem.getInt(paramString));
    paramVisualItem.setInt(paramString, paramInt);
    paramVisualItem.setInt(getEndField(paramString), paramInt);
  }
  
  public static void updateLong(VisualItem paramVisualItem, String paramString, long paramLong)
  {
    paramVisualItem.setLong(getStartField(paramString), paramVisualItem.getLong(paramString));
    paramVisualItem.setLong(paramString, paramLong);
    paramVisualItem.setLong(getEndField(paramString), paramLong);
  }
  
  public static void updateFloat(VisualItem paramVisualItem, String paramString, float paramFloat)
  {
    paramVisualItem.setFloat(getStartField(paramString), paramVisualItem.getFloat(paramString));
    paramVisualItem.setFloat(paramString, paramFloat);
    paramVisualItem.setFloat(getEndField(paramString), paramFloat);
  }
  
  public static void updateDouble(VisualItem paramVisualItem, String paramString, double paramDouble)
  {
    paramVisualItem.setDouble(getStartField(paramString), paramVisualItem.getDouble(paramString));
    paramVisualItem.setDouble(paramString, paramDouble);
    paramVisualItem.setDouble(getEndField(paramString), paramDouble);
  }
  
  public static void updateBoolean(VisualItem paramVisualItem, String paramString, boolean paramBoolean)
  {
    paramVisualItem.setBoolean(getStartField(paramString), paramVisualItem.getBoolean(paramString));
    paramVisualItem.setBoolean(paramString, paramBoolean);
    paramVisualItem.setBoolean(getEndField(paramString), paramBoolean);
  }
  
  public static void updateVisible(VisualItem paramVisualItem, boolean paramBoolean)
  {
    paramVisualItem.setStartVisible(paramVisualItem.isVisible());
    paramVisualItem.setVisible(paramBoolean);
    paramVisualItem.setEndVisible(paramBoolean);
  }
  
  public static void setX(VisualItem paramVisualItem1, VisualItem paramVisualItem2, double paramDouble)
  {
    double d = paramVisualItem1.getX();
    if (Double.isNaN(d)) {
      d = paramVisualItem2 != null ? paramVisualItem2.getX() : paramDouble;
    }
    paramVisualItem1.setStartX(d);
    paramVisualItem1.setEndX(paramDouble);
    paramVisualItem1.setX(paramDouble);
  }
  
  public static void setY(VisualItem paramVisualItem1, VisualItem paramVisualItem2, double paramDouble)
  {
    double d = paramVisualItem1.getY();
    if (Double.isNaN(d)) {
      d = paramVisualItem2 != null ? paramVisualItem2.getY() : paramDouble;
    }
    paramVisualItem1.setStartY(d);
    paramVisualItem1.setEndY(paramDouble);
    paramVisualItem1.setY(paramDouble);
  }
  
  public static boolean isChildGroup(String paramString)
  {
    return paramString.indexOf(GROUP_DELIMITER) != -1;
  }
  
  public static String getParentGroup(String paramString)
  {
    int i = paramString.lastIndexOf(GROUP_DELIMITER);
    return i < 0 ? null : paramString.substring(0, i);
  }
  
  public static String getChildGroup(String paramString)
  {
    int i = paramString.lastIndexOf(GROUP_DELIMITER);
    return i < 0 ? null : paramString.substring(i + 1);
  }
  
  public static String getGroupName(String paramString1, String paramString2)
  {
    return paramString1 + GROUP_DELIMITER + paramString2;
  }
  
  public static String getStartField(String paramString)
  {
    return paramString + ":start";
  }
  
  public static String getEndField(String paramString)
  {
    return paramString + ":end";
  }
  
  public static Schema getVisualItemSchema()
  {
    Schema localSchema = new Schema();
    localSchema.addColumn(VisualItem.VALIDATED, Boolean.TYPE, Boolean.FALSE);
    localSchema.addColumn(VisualItem.VISIBLE, Boolean.TYPE, Boolean.TRUE);
    localSchema.addColumn(VisualItem.STARTVISIBLE, Boolean.TYPE, Boolean.FALSE);
    localSchema.addColumn(VisualItem.ENDVISIBLE, Boolean.TYPE, Boolean.TRUE);
    localSchema.addColumn(VisualItem.INTERACTIVE, Boolean.TYPE, Boolean.TRUE);
    localSchema.addColumn(VisualItem.EXPANDED, Boolean.TYPE, Boolean.TRUE);
    localSchema.addColumn(VisualItem.FIXED, Boolean.TYPE, Boolean.FALSE);
    localSchema.addColumn(VisualItem.HIGHLIGHT, Boolean.TYPE, Boolean.FALSE);
    localSchema.addColumn(VisualItem.HOVER, Boolean.TYPE, Boolean.FALSE);
    localSchema.addInterpolatedColumn(VisualItem.X, Double.TYPE);
    localSchema.addInterpolatedColumn(VisualItem.Y, Double.TYPE);
    localSchema.addColumn(VisualItem.BOUNDS, Rectangle2D.class, new Rectangle2D.Double());
    Integer localInteger1 = new Integer(ColorLib.rgba(0, 0, 0, 0));
    localSchema.addInterpolatedColumn(VisualItem.STROKECOLOR, Integer.TYPE, localInteger1);
    Integer localInteger2 = new Integer(ColorLib.rgba(0, 0, 0, 0));
    localSchema.addInterpolatedColumn(VisualItem.FILLCOLOR, Integer.TYPE, localInteger2);
    Integer localInteger3 = new Integer(ColorLib.rgba(0, 0, 0, 0));
    localSchema.addInterpolatedColumn(VisualItem.TEXTCOLOR, Integer.TYPE, localInteger3);
    localSchema.addInterpolatedColumn(VisualItem.SIZE, Double.TYPE, new Double(1.0D));
    localSchema.addColumn(VisualItem.SHAPE, Integer.TYPE, new Integer(0));
    localSchema.addColumn(VisualItem.STROKE, Stroke.class, new BasicStroke());
    Font localFont = FontLib.getFont("SansSerif", 0, 10);
    localSchema.addInterpolatedColumn(VisualItem.FONT, Font.class, localFont);
    localSchema.addColumn(VisualItem.DOI, Double.TYPE, new Double(Double.MIN_VALUE));
    return localSchema;
  }
  
  public static Schema getMinimalVisualSchema()
  {
    Schema localSchema = new Schema();
    localSchema.addColumn(VisualItem.VALIDATED, Boolean.TYPE, Boolean.FALSE);
    localSchema.addColumn(VisualItem.VISIBLE, Boolean.TYPE, Boolean.TRUE);
    localSchema.addColumn(VisualItem.STARTVISIBLE, Boolean.TYPE, Boolean.FALSE);
    localSchema.addColumn(VisualItem.ENDVISIBLE, Boolean.TYPE, Boolean.TRUE);
    localSchema.addColumn(VisualItem.INTERACTIVE, Boolean.TYPE, Boolean.TRUE);
    localSchema.addColumn(VisualItem.BOUNDS, Rectangle2D.class, new Rectangle2D.Double());
    return localSchema;
  }
  
  public static Schema getAxisLabelSchema()
  {
    Schema localSchema = getVisualItemSchema();
    localSchema.setDefault(VisualItem.STARTVISIBLE, Boolean.FALSE);
    Integer localInteger = new Integer(ColorLib.gray(230));
    localSchema.setInterpolatedDefault(VisualItem.STROKECOLOR, localInteger);
    localInteger = new Integer(ColorLib.gray(150));
    localSchema.setInterpolatedDefault(VisualItem.TEXTCOLOR, localInteger);
    Double localDouble = new Double(NaN.0D);
    localSchema.addInterpolatedColumn(VisualItem.X2, Double.TYPE);
    localSchema.addInterpolatedColumn(VisualItem.Y2, Double.TYPE);
    localSchema.addColumn(VisualItem.LABEL, String.class);
    localSchema.addColumn(VisualItem.VALUE, Double.TYPE, localDouble);
    return localSchema;
  }
}


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\prefuse\util\PrefuseLib.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */